/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.bigdata.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.bigdata.entity.VisualCategory;
import org.springblade.bigdata.vo.VisualCategoryVO;
import org.springblade.bigdata.service.IVisualCategoryService;
import org.springblade.core.boot.ctrl.BladeController;

/**
 * 可视化分类表 控制器
 *
 * @author BladeX
 * @since 2021-03-05
 */
@RestController
@AllArgsConstructor
@RequestMapping("bigdata/visualcategory")
@Api(value = "可视化分类表", tags = "可视化分类表接口")
public class VisualCategoryController extends BladeController {

	private final IVisualCategoryService visualCategoryService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入visualCategory")
	public R<VisualCategory> detail(VisualCategory visualCategory) {
		VisualCategory detail = visualCategoryService.getOne(Condition.getQueryWrapper(visualCategory));
		return R.data(detail);
	}

	/**
	 * 分页 可视化分类表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入visualCategory")
	public R<IPage<VisualCategory>> list(VisualCategory visualCategory, Query query) {
		IPage<VisualCategory> pages = visualCategoryService.page(Condition.getPage(query), Condition.getQueryWrapper(visualCategory));
		return R.data(pages);
	}

	/**
	 * 自定义分页 可视化分类表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入visualCategory")
	public R<IPage<VisualCategoryVO>> page(VisualCategoryVO visualCategory, Query query) {
		IPage<VisualCategoryVO> pages = visualCategoryService.selectVisualCategoryPage(Condition.getPage(query), visualCategory);
		return R.data(pages);
	}

	/**
	 * 新增 可视化分类表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入visualCategory")
	public R save(@Valid @RequestBody VisualCategory visualCategory) {
		return R.status(visualCategoryService.save(visualCategory));
	}

	/**
	 * 修改 可视化分类表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入visualCategory")
	public R update(@Valid @RequestBody VisualCategory visualCategory) {
		return R.status(visualCategoryService.updateById(visualCategory));
	}

	/**
	 * 新增或修改 可视化分类表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入visualCategory")
	public R submit(@Valid @RequestBody VisualCategory visualCategory) {
		return R.status(visualCategoryService.saveOrUpdate(visualCategory));
	}

	
	/**
	 * 删除 可视化分类表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(visualCategoryService.removeByIds(Func.toLongList(ids)));
	}

	
}
