/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.bigdata.controller;

import java.util.ArrayList;
import java.util.List;

import javax.validation.Valid;

import org.springblade.bigdata.dto.VisualDataDTO;
import org.springblade.bigdata.entity.Visual;
import org.springblade.bigdata.entity.VisualConfig;
import org.springblade.bigdata.service.IVisualConfigService;
import org.springblade.bigdata.service.IVisualService;
import org.springblade.bigdata.vo.VisualDetailVO;
import org.springblade.bigdata.vo.VisualVO;
import org.springblade.bigdata.wrapper.VisualWrapper;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;


/**
 * 可视化表 控制器
 *
 * @author BladeX
 * @since 2021-03-01
 */
@RestController
@AllArgsConstructor
@RequestMapping("bigdata/visual")
@Api(value = "可视化表", tags = "可视化表接口")
public class VisualController extends BladeController {

	private final IVisualService visualService;

	private final IVisualConfigService visualConfigService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入visual")
	public R<VisualDetailVO> detail(Visual visual) {
		Visual detail = visualService.getOne(Condition.getQueryWrapper(visual));
		VisualConfig detailConfig = visualConfigService.getOne(new QueryWrapper<VisualConfig>().lambda().eq(VisualConfig::getVisualId, detail.getId()));
		VisualDetailVO visualDetailVO = new VisualDetailVO();
		visualDetailVO.setConfig(detailConfig);
		visualDetailVO.setVisual(detail);
		return R.data(visualDetailVO);
	}

	/**
	 * 分页 可视化表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "orderBy排序字段， sortType 0-倒序 1-正序")
	public R<IPage<VisualVO>> list(Visual visual, Query query) {
		QueryWrapper queryWrapper = Condition.getQueryWrapper(visual);
/*		if (StringUtils.hasText(orderBy)) {
			if (null != sortType && sortType == 0) {
				queryWrapper.orderByDesc(orderBy);
			} else if (null != sortType && sortType == 1) {
				queryWrapper.orderByAsc(orderBy);
			} else {
				queryWrapper.orderByDesc(orderBy);
			}
		} else {
			if (null != sortType && sortType == 0) {
				queryWrapper.orderByDesc("create_time");
			} else if (null != sortType && sortType == 1) {
				queryWrapper.orderByAsc("create_time");
			} else {
				queryWrapper.orderByDesc("create_time");
			}

		}*/
		queryWrapper.orderByAsc("order_index");
		queryWrapper.orderByDesc("create_time");
		queryWrapper.eq("is_deleted", 0);
		IPage<Visual> pages = visualService.page(Condition.getPage(query), queryWrapper);
		return R.data(VisualWrapper.build().pageVO(pages));
	}


	/**
	 * 自定义分页 可视化表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入visual")
	public R<IPage<VisualVO>> page(VisualVO visual, Query query) {
		query.setDescs("create_time");
		IPage<VisualVO> pages = visualService.selectVisualPage(Condition.getPage(query), visual);
		return R.data(pages);
	}

	/**
	 * 新增 可视化表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入visual")
	public R<Visual> save(@RequestBody JSONObject visualJson) {
		JSONObject visualObject = visualJson.getJSONObject("visual");
		Visual visual = new Visual();
		visual.setTitle(visualObject.getString("title"));
		visual.setCategory(visualObject.getInteger("category"));
		visual.setPassword(visualObject.getString("password"));
		Integer orderIndex = visualObject.getInteger("orderIndex");
		if (null == orderIndex) {
			orderIndex = 100;
		}
		visual.setOrderIndex(orderIndex);
		visual.setIsDeleted(0);
		//Visual visual = JSONObject.parseObject(visualJson.getString("visual"),Visual.class);
		if (visualService.save(visual)) {
			JSONObject configObject = visualJson.getJSONObject("config");
			VisualConfig visualConfig = new VisualConfig();
			visualConfig.setVisualId(visual.getId());
			visualConfig.setDetail(configObject.getString("detail"));
			visualConfig.setComponent(configObject.getString("component"));
			visualConfigService.save(visualConfig);
			return R.data(visual);
		} else {
			return R.status(false);
		}
	}

	/**
	 * 修改 可视化表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入visual")
	public R update(@Valid @RequestBody Visual visual) {
		return R.status(visualService.updateById(visual));
	}

	/**
	 * 新增或修改 可视化表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入visual")
	public R submit(@Valid @RequestBody Visual visual) {
		return R.status(visualService.saveOrUpdate(visual));
	}


	/**
	 * 删除 可视化表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(visualService.removeByIds(Func.toLongList(ids)));
//		return R.status(visualService.doRemove(Func.toLongList(ids)));
	}

	/**
	 * 详情
	 */
	@GetMapping("/detailList")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "详情列表", notes = "传入ids")
	public R<List<VisualDetailVO>> detailList(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		List<Long> idList = Func.toLongList(ids);
		List<VisualDetailVO> list = new ArrayList<>();
		if (null != idList && idList.size() > 0) {
			idList.forEach(id -> {
				Visual detail = visualService.getById(id);
				VisualConfig detailConfig = visualConfigService.getOne(new QueryWrapper<VisualConfig>().lambda().eq(VisualConfig::getVisualId, detail.getId()));
				VisualDetailVO visualDetailVO = new VisualDetailVO();
				visualDetailVO.setConfig(detailConfig);
				visualDetailVO.setVisual(detail);
				list.add(visualDetailVO);
			});
		}

		return R.data(list);
	}

	/**
	 *
	 */
	@PostMapping("updateVisual")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "更新页面和组件", notes = "传入VisualData")
	public R updateVisual(@RequestBody VisualDataDTO visualData) {
		try {
			Visual visual = visualData.getVisual();
			if (null == visual) {
				return R.fail("入参错误");
			}
			visualService.saveOrUpdate(visual);
			VisualConfig visualConfig = visualData.getConfig();
			QueryWrapper queryWrapper = new QueryWrapper();
			queryWrapper.eq("visual_id", visual.getId());
			visualConfigService.remove(queryWrapper);
			visualConfig.setVisualId(visual.getId());
			visualConfigService.save(visualConfig);
			return R.status(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return R.status(false);

	}

	/**
	 * 拷贝 可视化表
	 */
	@PostMapping("/copy")
	@ApiOperationSupport(order = 9)
	@ApiOperation(value = "拷贝模板", notes = "传入原模板id")
	public R<Visual> copy(@ApiParam(value = "原模板id", required = true) @RequestParam Integer id) {
		if (null == id) {
			return R.fail("id必传");
		}
		Visual visual = visualService.getById(id);
		if (null == visual) {
			return R.fail("原id错误");
		}
		Visual newVisual = new Visual();
		BeanUtils.copyProperties(visual, newVisual);
		newVisual.setId(null);
		visualService.save(newVisual);
		VisualConfig visualConfig = visualConfigService.selectByVisualId(id);
		if (null == visualConfig) {
			return R.fail("原模板未配置属性");
		}
		VisualConfig newConfig = new VisualConfig();
		BeanUtils.copyProperties(visualConfig, newConfig);
		newConfig.setVisualId(newVisual.getId());
		visualConfigService.save(newConfig);
		return R.data(newVisual);

	}

}
