/**
 *
 */
package org.springblade.cube.controller;

import java.util.ArrayList;
import java.util.Base64;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import cn.hutool.core.date.DateUtil;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.log.annotation.ApiLog;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.cube.dto.CubeDataParam;
import org.springblade.cube.dto.CubeDataPermRelation;
import org.springblade.cube.dto.CubeDataWithPerm;
import org.springblade.cube.dto.CubeStaticDataParam;
import org.springblade.cube.dto.CubeTableName;
import org.springblade.cube.dto.CubeVisualInfo;
import org.springblade.cube.dto.OrderByParam;
import org.springblade.cube.dto.PreviewResult;
import org.springblade.cube.dto.TableLinkJoin;
import org.springblade.cube.dto.TableLinkJoin.JoinKey;
import org.springblade.cube.dto.VirtualColumnParam;
import org.springblade.cube.enums.ColumnAttrEnum;
import org.springblade.cube.enums.FunctionTypeEnum;
import org.springblade.cube.enums.TableTypeEnum;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.model.CubeDatasouceFunction;
import org.springblade.cube.model.CubeTableColumn;
import org.springblade.cube.service.CubeDataPermissionService;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.service.CubeDatasouceFunctionService;
import org.springblade.cube.service.CubeTableStaticService;
import org.springblade.cube.service.CubeWorkspaceService;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.cube.utils.CubeUtils;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.enums.DateFormatEnum;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.enums.FunctionMethodEnum;
import org.springblade.datasource.enums.NumberFormatEnum;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.datasource.model.TableColumnDto;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.system.client.DataPermClient;
import org.springblade.visual.dto.ComponentFilter;
import org.springblade.visual.dto.ComponentFilterRelation;
import org.springblade.visual.dto.CubePermParseDto;
import org.springblade.visual.dto.PreviewBaseDto;
import org.springblade.visual.service.QuerySqlService;
import org.springblade.visual.utils.Tuple2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.lang.Tuple;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@RestController
@RequestMapping("/cube/data")
@Api(value = "数据集接口", tags = "数据集相关接口")
public class CubeDataController extends BladeController {

    @Autowired
    private QuerySqlService querySqlService;

    @Autowired
    private IReportDatasourceService reportDatasourceService;

    @Autowired
    private DynamicTableService dynamicTableService;

    @Autowired
    private CubeDataService cubeDataService;

    @Autowired
    private DataPermClient dataPermClient;

    @Autowired
    private CubeDataPermissionService cubeDataPermissionService;

    @Autowired
    private CubeDatasouceFunctionService cubeDatasouceFunctionService;

    @Autowired
    private DialectContext dialectContext;

    @Autowired
    private CubeTableStaticService cubeTableStaticService;

    @Autowired
    private CubeWorkspaceService cubeWorkspaceService;

    /**
     *
     * @param cubeData
     * @param query
     * @return
     */
    @GetMapping("/list")
    @ApiOperation(value = "数据集列表")
    public R<IPage<CubeDataWithPerm>> list(CubeData cubeData, Query query) {
        // 判断是否超级权限组「拥有所有权限」
        long userId = getUser().getUserId();
        Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(userId);
        LambdaQueryWrapper<CubeData> queryWrapper = Condition.getQueryWrapper(cubeData).lambda().eq(CubeData::getDisplayStatus, 1).orderByDesc(CubeData::getId);
        // 非超级权限组用户，查询有权限的数据集列表
        if (checkSuperPermRet == null || !checkSuperPermRet) {
            queryWrapper.and(wrapper -> {
                wrapper.eq(CubeData::getCreateUserId, userId).or().eq(CubeData::getPublicStatus, 1)
                        .or().inSql(CubeData::getId, "select a.cube_data_id from cube_data_permission a inner join data_permission_user b on a.data_perm_id = b.data_perm_id where b.user_id = " + userId);
            });
        }
        //查询数据
        IPage<CubeData> pageResult = cubeDataService.page(Condition.getPage(query), queryWrapper);
        IPage<CubeDataWithPerm> resultPage = null;
        //不为空，设置权限组名称
        if (pageResult != null && CollectionUtil.isNotEmpty(pageResult.getRecords())) {
            resultPage = new Page<>(pageResult.getCurrent(), pageResult.getSize(), pageResult.getTotal());
            List<Integer> cubeIdList = pageResult.getRecords().stream().map(CubeData::getId).collect(Collectors.toList());
            Map<Integer, List<CubeDataPermRelation>> permRelaMap = Optional.ofNullable(cubeDataPermissionService.selectListByCubeIdList(cubeIdList)).orElse(new ArrayList<>()).stream().collect(Collectors.groupingBy(CubeDataPermRelation::getCubeDataId));
            List<Integer> cubeIds = pageResult.getRecords().stream().map(CubeData::getId).collect(Collectors.toList());
            List<CubeVisualInfo> cubeVisualInfos = cubeDataService.getVisualInfos(cubeIds);
            Map<Integer, List<CubeVisualInfo>> listMap;
            if (CollectionUtil.isNotEmpty(cubeVisualInfos)) {
                listMap = cubeVisualInfos.stream().collect(Collectors.groupingBy(CubeVisualInfo::getCubeId));
            } else {
                listMap = null;
            }
            List<CubeDataWithPerm> records = new ArrayList<>();
            pageResult.getRecords().forEach(item -> {
                item.setTableNames(JSON.parseArray(item.getCubeTableNames(), CubeTableName.class));
                if (CollectionUtil.isNotEmpty(listMap) && CollectionUtil.isNotEmpty(listMap.get(item.getId()))) {
                    item.setVisualCount(listMap.get(item.getId()).size());
                    item.setCubeVisualInfos(listMap.get(item.getId()));
                }
                CubeDataWithPerm cubeDataWithPerm = BeanUtil.copyProperties(item, CubeDataWithPerm.class);
                cubeDataWithPerm.setDataPermRelationList(permRelaMap.get(item.getId()));
                records.add(cubeDataWithPerm);
            });

            resultPage.setRecords(records);
        }
        return R.data(resultPage);
    }

    /**
     *
     * @param cubeDataId
     * @return
     */
    @GetMapping("/info")
    @ApiOperation(value = "根据ID查询数据集详情")
    public R<CubeData> info(@RequestParam Integer cubeDataId) {
        return R.data(cubeDataService.getByCubeId(cubeDataId));
    }

    /**
     * 数据预览
     *
     * @param value
     * @return
     */
    @SuppressWarnings("rawtypes")
    @ApiOperation(value = "预览数据")
    @PostMapping("/preview")
    public R<PreviewResult> previewData(@RequestBody String value) throws Exception {
        String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
        CubeDataParam queryParam = JSON.parseObject(decoderStr, CubeDataParam.class);
        Integer datasourceId = queryParam.getDatasourceId();
        if (datasourceId == null || datasourceId <= 0) {
            return R.fail("数据源ID不能为空");
        }
        // 前提:查询数据源
        ReportDatasource reportDatasource = reportDatasourceService.getByDsId(datasourceId);
        if (reportDatasource == null) {
            return R.fail("数据源不存在");
        }
        CubeData cubeData = queryParam.getCubeData();
        DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(reportDatasource.getDsType());

        // 查询自定义函数列表[用于对虚拟字段中带有自定义函数的解析处理]
        List<CubeDatasouceFunction> functionList = cubeDatasouceFunctionService
                .list(Wrappers.<CubeDatasouceFunction>lambdaQuery().eq(CubeDatasouceFunction::getDsType, dsTypeEnum)
                        .eq(CubeDatasouceFunction::getFuncType, FunctionTypeEnum.custom));

        // 获取字段列表
        List<CubeTableColumn> columnList = queryParam.getColumnList();
        Map<String, CubeTableColumn> columnMap = columnList.stream().collect(Collectors.toMap(CubeTableColumn::getColumnTitle, t -> t, (n, n1) -> n));

        // 1.获取数据集,开始拼接FROM表相关SQL
        String fromSql = _buildFromSql(cubeData, columnMap, functionList);

        Map<String, List<Object>> extraParams = querySqlService.buildGlobalParams(null, getUser());

        for (int i = 0; i < columnList.size(); i++) {
            CubeTableColumn tableColumn = columnList.get(i);
            String columnAliasName = CubeConstants.COLUMN_ALIAS_NAME_PREFIX + i;
            tableColumn.setColumnAliasName(dsTypeEnum.isFieldLowerCase() ? columnAliasName.toLowerCase() : columnAliasName);
        }
        Map<String, String> fieldDict = columnList.stream().collect(Collectors.toMap(CubeTableColumn::getColumnAliasName, CubeTableColumn::getColumnTitle));
        List<String> selectSqlList = new ArrayList<String>();
        for (CubeTableColumn tableColumn : columnList) {
            if (tableColumn.getHiddenStatus() == 1) { //字段隐藏做查询
                continue;
            }
            String columnSql = tableColumn.getTableAliasName() + "." + tableColumn.getColumnName();
            // 虚拟字段
            if (tableColumn.getColumnAttr() == ColumnAttrEnum.VIRTUAL) {
                // 计算字段中：自定义函数前置解析处理
                Tuple customFunctionSqlTuple = _parseCustomFunctionSql(cubeData, tableColumn.getColumnTitle(), tableColumn.getColumnFormula(), functionList);
                String finColumnFormula = customFunctionSqlTuple != null ? customFunctionSqlTuple.get(0) : tableColumn.getColumnFormula();
                // 如果包含聚合函数，不执行
                if (StrUtil.containsAny(finColumnFormula.toLowerCase(), CubeConstants.AGGREGATION_FUNCTIONS)) {
                    columnSql = CubeConstants.DEFAULT_SHOW_CHAR;
                } else {
                    VirtualColumnParam virtualColumnParam = CubeUtils.formatColumnFormula(finColumnFormula, columnMap);
                    //全局变量计算SQL处理
                    columnSql = CubeUtils.formatVirtualSqlFragment(virtualColumnParam.getColumnSql(), extraParams);
                }
            }
            // 数据类型转换
            columnSql = dialectContext.getDialect(dsTypeEnum).convertDataType(tableColumn.getOldViewDataType(), tableColumn.getViewDataType(), columnSql, tableColumn.getTimeDataFormat());
            // 格式化日期，默认日期格式是：yyyy_MM_ddhh$mm$ss
            if (tableColumn.getTimeDataFormat() != null
                    && tableColumn.getTimeDataFormat() != DateFormatEnum.yyyy_MM_ddhh$mm$ss
                    && tableColumn.getViewDataType() == ViewColumnTypeEnum.DATETIME
                    && !CubeConstants.DEFAULT_SHOW_CHAR.equals(columnSql)) {
                columnSql = dialectContext.getDialect(dsTypeEnum).formatDateTime(tableColumn.getTimeDataFormat(), columnSql);
            }
            // 格式化数字
            if (tableColumn.getNumberDataFormat() != null && tableColumn.getViewDataType() == ViewColumnTypeEnum.NUMBER
                    && !CubeConstants.DEFAULT_SHOW_CHAR.equals(columnSql)) {
                columnSql = String.format(tableColumn.getNumberDataFormat().getMethod(), columnSql);
//				// 拼接百分号
                if (NumberFormatEnum.isPercentFormat(tableColumn.getNumberDataFormat())) {
                    columnSql = dialectContext.getDialect(dsTypeEnum).formatMethodFunction(FunctionMethodEnum.NUMBER_JOINT_PERCENT, columnSql, tableColumn.getNumberDataFormat().getOracleFmFormat());
                }
            }
            selectSqlList.add(columnSql + " as \"" + tableColumn.getColumnAliasName() + "\"");
        }
        String selectSql = String.join(",", selectSqlList);
        // 3.获取OrderBy信息,开始做排序相关SQL
        List<OrderByParam> orderByList = queryParam.getOrderByList();
        String orderBySql = "";
        if (orderByList != null && !orderByList.isEmpty()) {
            List<String> orderBySqlList = new ArrayList<String>();
            for (OrderByParam orderByParam : orderByList) {
                CubeTableColumn cubeTableColumn = columnMap.get(orderByParam.getColumnTitle());
                orderBySqlList.add(cubeTableColumn.getColumnAliasName() + " " + orderByParam.getOrderBy().name());
            }
            if (!orderBySqlList.isEmpty()) {
                orderBySql = "order by " + String.join(",", orderBySqlList);
            }
        }
        //组装where条件
        List<String> conditionSqlList = ListUtil.toList("1=1");
        //解析数据集全局过滤器
        if (StrUtil.isNotBlank(cubeData.getFilterJson())) {
            PreviewBaseDto cubePreviewDto = JSON.parseObject(cubeData.getFilterJson(), PreviewBaseDto.class);
            //解析SQL
            Tuple2<String, String> whereHavingSqlTuple = querySqlService.parseFilters(JSON.parseArray(cubePreviewDto.getFilterList(), ComponentFilter.class),
                    JSON.parseObject(cubePreviewDto.getFilterRelation(), ComponentFilterRelation.class), dsTypeEnum, extraParams);
            if (StrUtil.isNotBlank(whereHavingSqlTuple.getT1())) {
                conditionSqlList.add(whereHavingSqlTuple.getT1());
            }
        }
        // 组装权限组过滤条件
        if (queryParam.getCubeDataId() != null) {
            CubePermParseDto cubePermParseDto = cubeDataPermissionService.parseCubePermission(queryParam.getCubeDataId(), getUser());
            if (StrUtil.isNotBlank(cubePermParseDto.getFilterSql())) {
                conditionSqlList.add(cubePermParseDto.getFilterSql());
            }
        }
        String querySql = String.format(CubeConstants.SELECT_SQL_FORMAT, selectSql, fromSql, String.join(" AND ", conditionSqlList), orderBySql);
//		log.info("QuerySQL:" + querySql);
        if (CubeUtils.checkSqlInject(querySql)) {
            return R.fail("有SQL注入风险，请检查");
        }
        List<Map<String, Object>> resultList = null;
        if (queryParam.getLimit() > 0) { // 分页查询
            resultList = dynamicTableService.selectPageList(new Page(0, queryParam.getLimit()).setSearchCount(false),
                    reportDatasource, querySql);
        } else {
            resultList = dynamicTableService.selectList(reportDatasource, querySql);
        }
        return R.data(new PreviewResult(resultList, fieldDict, cubeData.getCubeName()), querySql);
    }

    /**
     * 保存或修改数据集
     *
     * @param value
     * @return
     */
    @ApiLog("数据集")
    @ApiOperation(value = "保存或修改数据集")
    @PostMapping("/saveOrUpdate")
    public R<Integer> saveOrUpdate(@RequestBody String value) throws Exception {
        String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
        CubeDataParam cubeDataParam = JSON.parseObject(decoderStr, CubeDataParam.class);
        Integer datasourceId = cubeDataParam.getDatasourceId();
        if (datasourceId == null || datasourceId <= 0) {
            return R.fail("数据源ID不能为空");
        }
        // 前提:查询数据源
        ReportDatasource reportDatasource = reportDatasourceService.getByDsId(datasourceId);
        if (reportDatasource == null) {
            return R.fail("数据源不存在");
        }
        DsTypeEnum dsTypeEnum = reportDatasource.getDsTypeSyl();

        // 获取字段列表,开始拼接SELECT字段相关SQL
        List<CubeTableColumn> columnList = cubeDataParam.getColumnList();
        Map<String, CubeTableColumn> columnMap = columnList.stream()
                .collect(Collectors.toMap(CubeTableColumn::getColumnTitle, t -> t, (n, n1) -> n));
        // 查询自定义函数列表[用于对虚拟字段中带有自定义函数的解析处理]
        List<CubeDatasouceFunction> functionList = cubeDatasouceFunctionService
                .list(Wrappers.<CubeDatasouceFunction>lambdaQuery().eq(CubeDatasouceFunction::getDsType, dsTypeEnum)
                        .eq(CubeDatasouceFunction::getFuncType, FunctionTypeEnum.custom));

        // 1.获取数据集,开始拼接FROM表相关SQL
        CubeData cubeData = cubeDataParam.getCubeData();
        if (cubeData.getId() == null) {
            cubeData.setCreateBy(SecureUtil.getNickName());
            cubeData.setCreateTime(new Date());
            cubeData.setCreateUserId(SecureUtil.getUserId());
        }
        String fromSql = _buildFromSql(cubeData, columnMap, functionList);
        cubeData.setQuerySql(fromSql);
        log.info("fromSql:" + fromSql);
        // 处理虚拟字段保存字段名和sql
        for (CubeTableColumn tableColumn : columnList) {
            String columnSql = tableColumn.getTableAliasName() + "." + tableColumn.getColumnName();
            if (tableColumn.getColumnAttr() == ColumnAttrEnum.VIRTUAL) {
                // 计算字段中：自定义函数前置解析处理
                Tuple customFunctionSqlTuple = _parseCustomFunctionSql(cubeData, tableColumn.getColumnTitle(), tableColumn.getColumnFormula(), functionList);
                String finColumnFormula = customFunctionSqlTuple != null ? customFunctionSqlTuple.get(0) : tableColumn.getColumnFormula();
                // 设置计算字段，是否包含聚合函数
                tableColumn.setVirtualAggregationStatus(StrUtil.containsAny(finColumnFormula.toLowerCase(), CubeConstants.AGGREGATION_FUNCTIONS) ? 1 : 0);
                tableColumn.setVirtualOnlyZhibiaoStatus(customFunctionSqlTuple != null ? customFunctionSqlTuple.get(1) : 0);
                // 格式化虚拟字段
                VirtualColumnParam virtualColumnParam = CubeUtils.formatColumnFormula(finColumnFormula, columnMap);
                tableColumn.setColumnName(String.join(",", virtualColumnParam.getColumnNames()));
                columnSql = virtualColumnParam.getColumnSql();
            }
            // 格式转换
            String _columnSql = dialectContext.getDialect(dsTypeEnum).convertDataType(tableColumn.getOldViewDataType(), tableColumn.getViewDataType(), columnSql, tableColumn.getTimeDataFormat());
            tableColumn.setColumnSql(_columnSql);
            if (CubeUtils.checkSqlInject(tableColumn.getColumnSql())) {
                return R.fail("[" + tableColumn.getColumnComment() + "]有SQL注入风险，请检查");
            }
        }
        return cubeDataService.doSaveOrUpdate(cubeData, columnList, cubeDataParam.getDataPermIdList(), getUser());
    }

    /**
     *
     * @return
     */
    @ApiLog("数据集")
    @ApiOperation(value = "根据表名创建数据集")
    @PostMapping("/createCubeByTable")
    public R<CubeData> createCubeByTable(
            @ApiParam(value = "数据源ID", required = true) Integer datasourceId,
            @ApiParam(value = "表注释", required = false) String tableComment,
            @ApiParam(value = "表名", required = true) String tableName,
            @ApiParam(value = "表单ID", required = false) String modelId,
            @ApiParam(value = "表单名称", required = false) String modelName) {
        //获取数据源
        ReportDatasource datasource = reportDatasourceService.getByDsId(datasourceId);
        if (datasource == null) {
            throw new ServiceException("当前代码行数："+Thread.currentThread().getStackTrace()[1].getLineNumber()+"数据源不存在");
        }
        List<TableColumnDto> reportColumnList = dialectContext.getDialect(datasource.getDsTypeSyl()).listTableColumnsByTableViewName(datasource, tableName);
        if (reportColumnList.isEmpty()) {
            throw new ServiceException("表[" + tableName + "]不存在");
        }
        tableComment = StrUtil.isNotBlank(tableComment) ? tableComment : tableName;
        String yyyyMMdd = DateUtil.format(new Date(), "yyyyMMdd");
        String cubeName = String.format("单表数据集-%s-%s-%s", tableName, modelName, yyyyMMdd);
        CubeStaticDataParam cubeStaticDataParam = new CubeStaticDataParam();
        cubeStaticDataParam.setCubeName(cubeName);
        cubeStaticDataParam.setTableName(tableName);
        cubeStaticDataParam.setTableComment(tableComment);
        cubeStaticDataParam.setDatasourceId(datasourceId);
        cubeStaticDataParam.setCubeWorkspaceId(cubeWorkspaceService.getIdOfModel(false));
        CubeData cubeData = cubeTableStaticService.saveCubeData(cubeStaticDataParam, reportColumnList, modelId,false);
        return R.data(cubeData);
    }

    /**
     * 保存或修改数据集
     */
    @ApiLog("数据集")
    @ApiOperation(value = "拷贝数据集")
    @PostMapping("/copy")
    public R<Integer> copyCubeData(@ApiParam(value = "新的数据集名称", required = true) @RequestParam String title,
                                   @ApiParam(value = "原数据集id", required = true) @RequestParam Integer cubeDataId,
                                   @ApiParam(value = "工作空间id", required = false) @RequestParam(required = false) Integer workspaceId,
                                   @ApiParam(value = "新的数据源id", required = true) @RequestParam Integer datasourceId,
                                   @ApiParam(value = "被拷贝的租户id") @RequestParam(required = false) String tenantId) {
        // 前提:查询数据源
        ReportDatasource reportDatasource;
        if (StrUtil.isNotEmpty(tenantId)) {
            reportDatasource = reportDatasourceService.getOne(Wrappers.<ReportDatasource>lambdaQuery()
                    .eq(ReportDatasource::getTenantId, tenantId)
                    .eq(ReportDatasource::getId, datasourceId));
        } else {
            reportDatasource = reportDatasourceService.getByDsId(datasourceId);
        }
        if (reportDatasource == null) {
            return R.fail("数据源不存在");
        }

        return cubeDataService.copy(title, cubeDataId, workspaceId, tenantId, getUser(), reportDatasource);
    }

    @ApiOperation(value = "根据租户id查询数据源")
    @GetMapping("/datasource/tenantid")
    public R<List<ReportDatasource>> getDatasourceByTenantId(@ApiParam(value = "租户id") @RequestParam(required = false) String tenantId) {
        if (StrUtil.isEmpty(tenantId)) {
            tenantId = getUser().getTenantId();
        }
        return R.data(reportDatasourceService.list(Wrappers.<ReportDatasource>lambdaQuery()
                .eq(ReportDatasource::getTenantId, tenantId)));
    }

    @ApiOperation(value = "根据租户id查询数据集")
    @GetMapping("/cube/tenantid")
    public R<List<CubeData>> getCubeByTenantId(
            @ApiParam(value = "租户id") @RequestParam(required = false) String tenantId,
            @ApiParam(value = "数据集名字") @RequestParam(required = false) String cubeName) {
        if (StrUtil.isEmpty(tenantId)) {
            tenantId = getUser().getTenantId();
        }
        return R.data(cubeDataService.list(Wrappers.<CubeData>lambdaQuery()
                .like(StrUtil.isNotBlank(cubeName), CubeData::getCubeName, cubeName)
                .eq(CubeData::getTenantId, tenantId)));
    }

    /**
     * 构造fromSql
     *
     * @param cubeData
     * @param columnMap
     * @return
     */
    private String _buildFromSql(CubeData cubeData, Map<String, CubeTableColumn> columnMap, List<CubeDatasouceFunction> functionList) {
        StringBuilder fromSqlBuffer = new StringBuilder();
        if (cubeData.getTableType() == TableTypeEnum.LOGIC) { // 有关联表
            List<TableLinkJoin> tableLinkJoinList = JSON.parseArray(cubeData.getLinksJoin(), TableLinkJoin.class);
            if (CollectionUtil.isEmpty(tableLinkJoinList)) {
                throw new ServiceException("你还没有指定关联表");
            }
            for (int i = 0; i < tableLinkJoinList.size(); i++) {
                TableLinkJoin tableLinkJoin = tableLinkJoinList.get(i);
                if (i == 0) {
                    fromSqlBuffer.append(tableLinkJoin.getSource()).append(" ").append(tableLinkJoin.getSourceId());
                }
                fromSqlBuffer.append(tableLinkJoin.getJoinType().getSql())
                        .append(tableLinkJoin.getTarget()).append(" ").append(tableLinkJoin.getTargetId());
                // 拼接ON
                fromSqlBuffer.append(" on ");
                int j = 1, size = tableLinkJoin.getJoinKeys().size();
                for (JoinKey joinKey : tableLinkJoin.getJoinKeys()) {
                    if (joinKey.isSourceKeyVtState()) {
                        // 计算字段中：自定义函数前置解析处理
                        Tuple customFunctionSqlTuple = _parseCustomFunctionSql(cubeData, "", joinKey.getSourceKey(), functionList);
                        String finColumnFormula = customFunctionSqlTuple != null ? customFunctionSqlTuple.get(0) : joinKey.getSourceKey();
                        fromSqlBuffer.append(CubeUtils.formatColumnFormula(finColumnFormula, columnMap).getColumnSql());
                    } else {
                        fromSqlBuffer.append(tableLinkJoin.getSourceId()).append(".").append(joinKey.getSourceKey());
                    }
                    fromSqlBuffer.append("=");
                    if (joinKey.isTargetKeyVtState()) {
                        // 计算字段中：自定义函数前置解析处理
                        Tuple customFunctionSqlTuple = _parseCustomFunctionSql(cubeData, "", joinKey.getTargetKey(), functionList);
                        String finColumnFormula = customFunctionSqlTuple != null ? customFunctionSqlTuple.get(0) : joinKey.getTargetKey();
                        fromSqlBuffer.append(CubeUtils.formatColumnFormula(finColumnFormula, columnMap).getColumnSql());
                    } else {
                        fromSqlBuffer.append(tableLinkJoin.getTargetId()).append(".").append(joinKey.getTargetKey());
                    }
                    fromSqlBuffer.append(j++ != size ? " and " : "");
                }
            }
        } else {
            List<CubeTableName> tableLinkJoinList = JSON.parseArray(cubeData.getCubeTableNames(), CubeTableName.class);
            if (CollectionUtil.isEmpty(tableLinkJoinList)) {
                throw new ServiceException("你还没有指定关联表");
            }
            CubeTableName tableName = tableLinkJoinList.get(0);
            fromSqlBuffer.append(tableName.getTableName()).append(" ").append(tableName.getTableAliasName());
        }
        return fromSqlBuffer.toString();
    }

    /**
     * 解析虚拟字段公式
     *
     * @param columnTitle   - 错误提示使用
     * @param columnFormula - 虚拟字段公式
     * @return
     */
    private Tuple _parseCustomFunctionSql(CubeData cubeData, String columnTitle, String columnFormula,
                                          List<CubeDatasouceFunction> functionList) {
        if (CollectionUtil.isEmpty(functionList)) {
            return null;
        }
        // 获取主表名和别名
        String masterTableName = "", masterAliasName = "";
        if (cubeData.getTableType() == TableTypeEnum.LOGIC) { // 有关联表
            List<TableLinkJoin> tableLinkJoinList = JSON.parseArray(cubeData.getLinksJoin(), TableLinkJoin.class);
            if (CollectionUtil.isEmpty(tableLinkJoinList)) {
                throw new ServiceException("你还没有指定关联表");
            }
            TableLinkJoin tableLinkJoin = tableLinkJoinList.get(0);
            masterTableName = tableLinkJoin.getSourceId();
            masterAliasName = tableLinkJoin.getSource();
        } else {
            List<CubeTableName> tableLinkJoinList = JSON.parseArray(cubeData.getCubeTableNames(), CubeTableName.class);
            if (CollectionUtil.isEmpty(tableLinkJoinList)) {
                throw new ServiceException("你还没有指定关联表");
            }
            CubeTableName tableName = tableLinkJoinList.get(0);
            masterTableName = tableName.getTableName();
            masterAliasName = tableName.getTableAliasName();
        }
        CubeDatasouceFunction function = functionList.stream().filter(item -> columnFormula.contains(item.getMethodName() + "(")).findAny().orElse(null);
        if (function != null) {
            // 获取自定义函数的参数
            String subBetweenStr = StrUtil.removeSuffix(StrUtil.removePrefixIgnoreCase(columnFormula, function.getMethodName() + "("), ")");
            String[] params = subBetweenStr.split(",");
            Map<String, String> paramMap = IntStream.range(0, params.length).boxed()
                    .collect(Collectors.toMap(t -> t.toString(), i -> params[i]));
            paramMap.put("master_table_name", masterTableName); // 主表名
            paramMap.put("master_alias_name", masterAliasName); // 主表别名
            String customSql = StrUtil.format(function.getFuncSql(), paramMap);
            if (customSql.contains("{") || customSql.contains("}")) {
                throw new ServiceException("虚拟字段[" + columnTitle + "]参数不对应");
            }
            return new Tuple(customSql, function.getSubQueryStatus());
        }
        return null;
    }

    /**
     * 列权限校验
     *
     * @param resultList
     * @param columnList
     * @param excludeColumnIdList
     */
    void _formatExcludeColumn(List<JSONObject> resultList, List<CubeTableColumn> columnList, Set<Integer> excludeColumnIdList) {
        // 列权限，删除掉不显示的列
        if (CollectionUtil.isNotEmpty(excludeColumnIdList) && CollectionUtil.isNotEmpty(resultList)) {
            Map<Integer, String> fieldIdMap = columnList.stream().collect(Collectors.toMap(CubeTableColumn::getId, CubeTableColumn::getColumnAliasName));
            List<String> excludeColumnAliasList = excludeColumnIdList.stream().map(it -> fieldIdMap.get(it)).filter(Objects::nonNull).collect(Collectors.toList());
            if (CollectionUtil.isNotEmpty(excludeColumnAliasList)) {
                // 格式化数据
                for (JSONObject jsonObj : resultList) {
                    excludeColumnAliasList.stream().forEach(item -> jsonObj.remove(item));
                }
            }
        }
    }
}
