/**
 * 
 */
package org.springblade.cube.controller;

import java.time.LocalDateTime;
import java.util.Map;

import io.swagger.annotations.ApiParam;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.cube.enums.ParamTypeEnum;
import org.springblade.cube.model.CubeGlobalParam;
import org.springblade.cube.service.CubeGlobalParamService;
import org.springblade.datasource.enums.DateFormatEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * @author wookvn
 *	
 */
@RestController
@RequestMapping("/cube/globalParam")
@Api(value = "数据集全局参数接口", tags = "数据集全局参数相关接口")
public class CubeGlobalParamController extends BladeController {

	@Autowired
	private CubeGlobalParamService cubeGlobalParamService;
	/**
	 * 
	 * @param query
	 * @param name
	 * @return
	 */
	@GetMapping("/page")
	@ApiOperation(value = "分页数据列表")
	public R<IPage<CubeGlobalParam>> page(CubeGlobalParam cubeGlobalParam, Query query){
		QueryWrapper<CubeGlobalParam> queryWrapper = Condition.getQueryWrapper(cubeGlobalParam);
		return R.data(cubeGlobalParamService.page(Condition.getPage(query), queryWrapper.lambda().orderByDesc(CubeGlobalParam::getId)));
	}
	
	/**
	 * 
	 * @return
	 */
	@PostMapping("/submit")
	@ApiOperation(value = "添加或修改数据")
	public R<String> doSubmit(@Validated @RequestBody CubeGlobalParam cubeGlobalParam){
		if(cubeGlobalParam.getId() == null) {
			cubeGlobalParam.setCreateUserId(getUser().getUserId());
			cubeGlobalParam.setCreateTime(LocalDateTime.now());
			cubeGlobalParam.setCreateBy(getUser().getNickName());
		}
		boolean isUpt = cubeGlobalParam.getId() != null;
		String dbParamKey = null;
		if(isUpt) {
			CubeGlobalParam dbParam = cubeGlobalParamService.getById(cubeGlobalParam.getId());
			if(dbParam == null) {
				return R.fail("变量不存在");
			}
			dbParamKey = dbParam.getParamKey();
		}
		//查询key是否存在
		long count = cubeGlobalParamService.count(Wrappers.<CubeGlobalParam>lambdaQuery().eq(CubeGlobalParam::getParamKey, cubeGlobalParam.getParamKey()));
		if(count > 0 && (!isUpt || (isUpt && !cubeGlobalParam.getParamKey().equals(dbParamKey)))) {
			return R.fail("变量key已存在");
		}
		return R.status(cubeGlobalParamService.saveOrUpdate(cubeGlobalParam));
	}
	
	/**
	 * 
	 * @return
	 */
	@PostMapping("/delete")
	@ApiOperation(value = "删除数据")
	public R<String> delete(@RequestParam Integer paramId){
		CubeGlobalParam cubeGlobalParam = cubeGlobalParamService.getById(paramId);
		if(cubeGlobalParam != null && cubeGlobalParam.getParamType() == ParamTypeEnum.SYSTEM) {
			return R.fail("系统参数不能删除");
		}
		return R.status(cubeGlobalParamService.removeById(paramId));
	}

	@GetMapping("/get/sys")
	@ApiOperation(value = "获取系统变量")
	public R<Map<String,Object>> getSysParams(@ApiParam("时间格式化类型") @RequestParam DateFormatEnum dateFormat) {

		return R.data(cubeGlobalParamService.getSysParams(dateFormat,getUser()));
	}
}
