/**
 *
 */
package org.springblade.cube.controller;

import java.util.Base64;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.cube.model.CubeSqlView;
import org.springblade.cube.service.CubeSqlViewService;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.cube.utils.CubeUtils;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.report.service.normal.ReportDatasourcePermissionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@RestController
@RequestMapping("/cube/sqlview")
@Api(value = "数据集sql视图接口", tags = "数据集_SQL视图接口")
public class CubeSqlViewController extends BladeController {

    @Autowired
    private DynamicTableService dynamicTableService;

    @Autowired
    private IReportDatasourceService reportDatasourceService;

    @Autowired
    private CubeSqlViewService cubeSqlViewService;

    @Autowired
    private ReportDatasourcePermissionService reportDatasourcePermissionService;

    /**
     *
     * @return
     */
    @SneakyThrows
    @PostMapping("/browser/{datasourceId}")
    @ApiOperation(value = "浏览视图数据", notes = "只显示前100条记录")
    @ApiParam(value = "value", type = "string", required = true, example = "base64加密字符串")
    public R<List<Map<String, Object>>> browser(@PathVariable Integer datasourceId, @RequestBody String value) {
        //sql语句
        String sqlStatement = new String(Base64.getDecoder().decode(value), "utf-8");
//        sqlStatement = sqlStatement + ";";
        //校验SQL是否有注入
        boolean checkSqlInject = CubeUtils.checkSqlInject(sqlStatement);
        if (checkSqlInject) {
            return R.fail("SQL有注入风险，请修正");
        }
        ReportDatasource reportDatasource = reportDatasourceService.getByDsId(datasourceId);
        if (reportDatasource == null) {
            return R.fail("数据源不存在");
        }
		if (reportDatasource.getDsTypeSyl().getHasLimitStatementFunction().apply(sqlStatement)) {
            return R.data(dynamicTableService.selectList(reportDatasource, sqlStatement));
        }
        return R.data(dynamicTableService.selectPageList(
                new Page<>(0, CubeConstants.DEFAULT_PREVIEW_DATA_SIZE).setSearchCount(false),
                reportDatasource, sqlStatement));
    }

    /**
     *
     * @return
     */
    @SneakyThrows
    @PostMapping("/tableBrowser/{datasourceId}")
    @ApiOperation(value = "浏览表数据", notes = "只显示前100条记录")
    @ApiParam(value = "value", type = "string", required = true, example = "base64加密字符串")
    public R<List<Map<String, Object>>> tableBrowser(@PathVariable Integer datasourceId, String tableName) {
        //sql语句
        String sqlStatement = "select * from " + tableName;
        ReportDatasource reportDatasource = reportDatasourceService.getByDsId(datasourceId);
        if (reportDatasource == null) {
            return R.fail("数据源不存在");
        }
        return R.data(dynamicTableService.selectPageList(
                new Page<>(0, CubeConstants.DEFAULT_PREVIEW_DATA_SIZE).setSearchCount(false),
                reportDatasource, sqlStatement));
    }

    @GetMapping("/data-browser")
    @ApiOperation(value = "浏览数据", notes = "支持自定义分页")
    public R<IPage<Map<String, Object>>> dataBrowser(@RequestParam Integer datasourceId,
                                                     @RequestParam String tableName,
                                                     Query query) {
        String sqlStatement = "select * from " + tableName;
        ReportDatasource reportDatasource = reportDatasourceService.getByDsId(datasourceId);
        if (reportDatasource == null) {
            return R.fail("数据源不存在");
        }

        return R.data(dynamicTableService.selectPage(new Page<>(query.getCurrent(), query.getSize()),
                reportDatasource, sqlStatement));
    }

    /**
     *
     * @param value
     * @return
     */
    @SneakyThrows
    @PostMapping("/submit")
    @ApiOperation(value = "创建视图", notes = "参数是Model类的base64加密字符串, Model类参考list接口返回值")
    public R<Integer> doSubmit(@RequestBody String value) {
        //modelJSON
        String modelJson = new String(Base64.getDecoder().decode(value), "utf-8");
        CubeSqlView cubeSqlView = JSON.parseObject(modelJson, CubeSqlView.class);
        if (cubeSqlView.getId() == null) {
            cubeSqlView.setCreateTime(new Date());
            cubeSqlView.setCreateUserId(getUser().getUserId());
        }
        if (StrUtil.isBlank(cubeSqlView.getSqlStatement()) || cubeSqlView.getDatasourceId() == null
                || StrUtil.isBlank(cubeSqlView.getViewName()) || StrUtil.isBlank(cubeSqlView.getSqlName())) {
            return R.fail("核心参数不能为空");
        }
        if (!StrUtil.startWithAnyIgnoreCase(cubeSqlView.getSqlStatement().trim(), "select", "with")) {
            return R.fail("请检查sql语句是否有误");
        }
        //校验SQL是否有注入
        boolean checkSqlInject = CubeUtils.checkSqlInject(cubeSqlView.getSqlStatement());
        if (checkSqlInject) {
            return R.fail("SQL有注入风险，请修正");
        }
        ReportDatasource reportDatasource = reportDatasourceService.getByDsId(cubeSqlView.getDatasourceId());
        if (reportDatasource == null) {
            return R.fail("数据源不存在");
        }
        //创建视图
        long startTime = System.currentTimeMillis();
        String createOrReplaceViewSql = String.format(CubeConstants.CREATE_REPLACE_VIEW_SQL_FORMAT, cubeSqlView.getViewName(), cubeSqlView.getSqlStatement());
        try {
            dynamicTableService.executeSql(reportDatasource, createOrReplaceViewSql);
        } catch (Exception e) {
            log.error(e.getMessage());
            R<Integer> failret = R.fail("请检查是否有视图创建权限，更多错误信息请查看F12");
            failret.setExt(e.getMessage());
            return failret;
        }
        log.info("sql视图创建完成|datasourceId={},viewName={},cost={}ms", cubeSqlView.getDatasourceId(), cubeSqlView.getViewName(), System.currentTimeMillis() - startTime);
        //添加数据
        boolean ret = cubeSqlViewService.saveOrUpdate(cubeSqlView);
        return ret ? R.data(cubeSqlView.getId()) : R.fail("操作失败");
    }

    /***
     *
     * @param datasourceId
     * @return
     */
    @GetMapping("/list")
    @ApiOperation(value = "获取SQL视图列表")
    public R<List<CubeSqlView>> list(@RequestParam Integer datasourceId) {
        List<CubeSqlView> dataList = cubeSqlViewService.list(Wrappers.<CubeSqlView>lambdaQuery().eq(CubeSqlView::getDatasourceId, datasourceId));
        if (CollectionUtil.isNotEmpty(dataList)) {// 权限组处理
            reportDatasourcePermissionService.dealTablePermission(getUser().getUserId(), datasourceId, permTableNameSet -> dataList.removeIf(it -> !permTableNameSet.contains(it.getViewName())));
        }
        return R.data(dataList);
    }
}
