/**
 *
 */
package org.springblade.cube.controller;

import java.util.List;
import java.util.Map;

import javax.validation.Valid;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.api.R;
import org.springblade.cube.dto.CubeStaticDataParam;
import org.springblade.cube.dto.TemporaryCubeDataParam;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.model.CubeTableStatic;
import org.springblade.cube.service.CubeTableStaticService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * @author wookvn
 *
 */
@RestController
@RequestMapping("/cube/static-data")
@Api(value = "静态数据集接口", tags = "静态数据集相关接口")
public class CubeStaticDataController extends BladeController {

    @Autowired
    private CubeTableStaticService cubeTableStaticService;

    /**
     * 上传静态数据创建数据集和物理表
     * data - 二维数组  例如：[["省","市","县","村"],["河南省","郑州市","高新区","丁楼村"]]
     * @return
     * @throws Exception
     */
    @ApiOperation(value = "上传静态数据创建数据集和物理表")
    @PostMapping("/submit")
    public R<CubeData> submit(@RequestBody @Valid CubeStaticDataParam dataParam) throws Exception {
        //前提：如果传递了cubeId,需要先进行一系列的删除操作
        //删除内容包括：数据集，数据集字段，静态文件表记录，物理表
        //1.识别表字段类型
        //2.创建表
        //3.存储静态文件表记录到业务库
        //4.存储上传的数据到物理表
        //5.创建数据集[该数据集不能显示]
        //6.创建关联的数据集字段
        return R.data(cubeTableStaticService.doSubmit(dataParam));
    }

    /**
     * 临时数据集--零代码自动建表相关逻辑
     * @param param
     * @return
     * @throws Exception
     */
    @ApiOperation(value = "临时创建数据集和物理表")
    @PostMapping("/temporary/submit")
    public R<Map> temporarySubmit(@RequestBody @Valid TemporaryCubeDataParam param) throws Exception {
        return R.data(cubeTableStaticService.temporaryCubeData(param));
    }


    /**
     *
     * @param datasourceId
     * @return
     */
    @ApiOperation(value = "获取静态数据表集合")
    @GetMapping("/list")
    public R<List<CubeTableStatic>> list(@RequestParam Integer datasourceId) {
        return R.data(cubeTableStaticService.list(Wrappers.<CubeTableStatic>lambdaQuery().eq(CubeTableStatic::getDatasourceId, datasourceId).orderByDesc(CubeTableStatic::getId)));
    }
}
