/**
 *
 */
package org.springblade.cube.controller;

import java.util.ArrayList;
import java.util.Base64;
import java.util.List;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.api.R;
import org.springblade.cube.dto.VirtualColumnCheck;
import org.springblade.cube.enums.AggregationTypeEnum;
import org.springblade.cube.enums.ColumnAttrEnum;
import org.springblade.cube.enums.DimensionTypeEnum;
import org.springblade.cube.enums.FunctionTypeEnum;
import org.springblade.cube.model.CubeDatasouceFunction;
import org.springblade.cube.model.CubeTableColumn;
import org.springblade.cube.service.CubeDataPermissionService;
import org.springblade.cube.service.CubeDatasouceFunctionService;
import org.springblade.cube.service.CubeTableColumnService;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.enums.DateFormatEnum;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.enums.NumberFormatEnum;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.datasource.model.TableColumnDto;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.resouces.CubeTableColumnResource;
import org.springblade.visual.dto.CubePermParseDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * @author wookvn
 */
@RestController
@RequestMapping("/cube/tableColumn")
@Api(value = "数据集_字段列表接口", tags = "数据集_字段列表接口")
public class CubeTableColumnController extends BladeController {

    @Autowired
    private CubeTableColumnService cubeTableColumnService;

//	@Autowired
//	private IReportBaseInfoService reportBaseInfoService;

    @Autowired
    private IReportDatasourceService reportDatasourceService;

    @Autowired
    private CubeDatasouceFunctionService cubeDatasouceFunctionService;

    @Autowired
    private CubeDataPermissionService cubeDataPermissionService;

    @Autowired
    private DialectContext dialectContext;
    /**
     * @param cubeData
     * @param query
     * @return
     */
    @GetMapping("/list")
    @ApiOperation(value = "根据数据集ID查询字段列表")
    public R<List<CubeTableColumn>> list(@RequestParam Integer cubeDataId) {
        List<CubeTableColumn> columnList = cubeTableColumnService.list(Wrappers.<CubeTableColumn>lambdaQuery().eq(CubeTableColumn::getCubeDataId, cubeDataId).orderByDesc(CubeTableColumn::getSortNum));
        //列权限，删除掉不显示的列
        CubePermParseDto cubePermParseDto = cubeDataPermissionService.parseCubePermission(cubeDataId, getUser());
        if (CollectionUtil.isNotEmpty(cubePermParseDto.getExcludeColumnIdList()) && CollectionUtil.isNotEmpty(columnList)) {
            columnList.removeIf(item -> cubePermParseDto.getExcludeColumnIdList().contains(item.getId()));
        }
        return R.data(columnList);
    }

    @PostMapping("/list/batch")
    @ApiOperation(value = "根据数据集IDS查询字段列表")
    public R<List<List<CubeTableColumn>>> list(@RequestBody List<Integer> cubeDataIds) {
        ArrayList<List<CubeTableColumn>> lists = new ArrayList<>();
        for (Integer cubeDataId : cubeDataIds) {
            List<CubeTableColumn> columnList = cubeTableColumnService.list(Wrappers.<CubeTableColumn>lambdaQuery().eq(CubeTableColumn::getCubeDataId, cubeDataId).orderByDesc(CubeTableColumn::getSortNum));
            //列权限，删除掉不显示的列
            CubePermParseDto cubePermParseDto = cubeDataPermissionService.parseCubePermission(cubeDataId, getUser());
            if (CollectionUtil.isNotEmpty(cubePermParseDto.getExcludeColumnIdList()) && CollectionUtil.isNotEmpty(columnList)) {
                columnList.removeIf(item -> cubePermParseDto.getExcludeColumnIdList().contains(item.getId()));
            }
            lists.add(columnList);
        }
        return R.data(lists);
    }

    /**
     * @param columnId
     * @return
     */
    @PostMapping("/delete")
    @ApiOperation(value = "根据ID删除字段")
    public R<String> delete(@RequestParam Integer columnId) {
        return R.status(cubeTableColumnService.removeById(columnId));
    }

    /**
     * @param dsId
     * @param cubeDataId
     * @param tableName
     * @return
     */
    @ApiOperation(value = "根据表名查询字段列表")
    @GetMapping("/listByTableName")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "tableName", value = "表名", dataType = "string", required = true),
                    @ApiImplicitParam(name = "dsId", value = "数据源ID", required = true)
            }
    )
    public R<List<CubeTableColumn>> listByTableName(Integer dsId, String tableName) {
        // 判断至少需要表名和数据源
        if (dsId == null || StrUtil.isBlank(tableName)) {
            return R.fail("数据源和表名不能为空");
        }
        // 1.查询数据源
        ReportDatasource reportDatasource = reportDatasourceService.getByDsId(dsId);
        if (reportDatasource == null) {
            return R.fail("数据源不存在");
        }
        // 2.查询表结构
//		List<ReportColumnDTO> reportColumnList = reportBaseInfoService.selectReportColumnList(reportDatasource, tableName);
        List<TableColumnDto> reportColumnList = dialectContext.getDialect(reportDatasource.getDsTypeSyl()).listTableColumnsByTableViewName(reportDatasource, tableName);
        if (reportColumnList.isEmpty()) {
            return R.fail("表不存在");
        }
        List<CubeTableColumn> resulitList = new ArrayList<CubeTableColumn>();
        int sortNum = 0;
        for (TableColumnDto columnDto : reportColumnList) {
            CubeTableColumn cubeTableColumn = new CubeTableColumn();
            cubeTableColumn.setTableName(tableName);
            cubeTableColumn.setAggregationType(AggregationTypeEnum.SUM);
            cubeTableColumn.setColumnAttr(ColumnAttrEnum.NORMAL);
            cubeTableColumn.setColumnComment(columnDto.getColumnComments());
            cubeTableColumn.setColumnName(columnDto.getColumnName());
            cubeTableColumn.setColumnTitle(StrUtil.isEmpty(columnDto.getColumnComments()) ? columnDto.getColumnName()
                    : columnDto.getColumnComments());
            cubeTableColumn.setViewDataType(columnDto.getColumnType());
            cubeTableColumn.setOldViewDataType(columnDto.getColumnType());
            cubeTableColumn.setDimensionType(columnDto.getColumnType() == ViewColumnTypeEnum.NUMBER ? DimensionTypeEnum.ZHIBIAO
                    : DimensionTypeEnum.WEIDU);
            cubeTableColumn.setHiddenStatus(0);
            cubeTableColumn.setSortNum(sortNum);
            cubeTableColumn.setNumberDataFormat(NumberFormatEnum.AUTO);
            cubeTableColumn.setTimeDataFormat(DateFormatEnum.yyyy_MM_ddhh$mm$ss);
            cubeTableColumn.setPrimaryKeyStatus(columnDto.getPrimaryKeyStatus());
            cubeTableColumn.setAutoIncrementStatus(columnDto.getAutoIncrementStatus());
            resulitList.add(cubeTableColumn);
            sortNum += 100;
        }
        return R.data(resulitList);
    }

    /**
     * @param columnId
     * @return
     * @throws Exception
     */
    @PostMapping("/checkAggregation")
    @ApiOperation(value = "校验是否包含聚合函数")
    public R<Boolean> checkAggregation(@RequestBody String value) throws Exception {
        String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
        VirtualColumnCheck checkParam = JSON.parseObject(decoderStr, VirtualColumnCheck.class);
        // 前提:查询数据源
        ReportDatasource reportDatasource = reportDatasourceService.getByDsId(checkParam.getDatasourceId());
        if (reportDatasource == null) {
            return R.fail("数据源不存在");
        }
        DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(reportDatasource.getDsType());
        //查询自定义函数列表[用于对虚拟字段中带有自定义函数的解析处理]
        List<CubeDatasouceFunction> functionList = cubeDatasouceFunctionService.list(Wrappers.<CubeDatasouceFunction>lambdaQuery()
                .eq(CubeDatasouceFunction::getDsType, dsTypeEnum).eq(CubeDatasouceFunction::getFuncType, FunctionTypeEnum.custom));
        if (CollectionUtil.isNotEmpty(functionList)) {
            CubeDatasouceFunction function = functionList.stream().filter(item -> checkParam.getColumnFormula().contains(item.getMethodName())).findAny().orElse(null);
            if (function != null) {
                checkParam.setColumnFormula(function.getFuncSql());
            }
        }
        return R.data(StrUtil.containsAny(checkParam.getColumnFormula().toLowerCase(), CubeConstants.AGGREGATION_FUNCTIONS));
    }
}
