/**
 * 
 */
package org.springblade.cube.controller;

import java.util.List;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.api.R;
import org.springblade.cube.model.CubeTableFile;
import org.springblade.cube.service.CubeTableFileService;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.model.TableColumnDto;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.report.service.normal.ReportDatasourcePermissionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import cn.hutool.core.collection.CollectionUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * @author wookvn
 *
 */
@RestController
@RequestMapping("/cube/tableFile")
@Api(value = "数据集_文件表接口", tags = "数据集_文件表接口")
public class CubeTableFileController extends BladeController {

	@Autowired
	private CubeTableFileService cubeTableFileService;

//	@Autowired
//	private IReportBaseInfoService reportBaseInfoService;
	
	@Autowired
	private IReportDatasourceService reportDatasourceService;

	@Autowired
	private ReportDatasourcePermissionService reportDatasourcePermissionService;
	
	@Autowired
	private DialectContext dialectContext;
	
	/**
	 * 
	 * @param dsId
	 * @return
	 */
	@GetMapping("/list")
	@ApiOperation(value = "文件表列表接口")
	public R<List<CubeTableFile>> list(@RequestParam Integer dsId) {
		List<CubeTableFile> dataList = cubeTableFileService.list(Wrappers.<CubeTableFile>lambdaQuery().eq(CubeTableFile::getDatasourceId, dsId));
		if(CollectionUtil.isNotEmpty(dataList)) {// 权限组处理
			reportDatasourcePermissionService.dealTablePermission(getUser().getUserId(), dsId, permTableNameSet -> dataList.removeIf(it -> !it.getCreateUserId().equals(getUser().getUserId()) && !permTableNameSet.contains(it.getTableName())));
		}
		return R.data(dataList);
	}

	/**
	 * 
	 * @param dsId
	 * @param tableName
	 * @return
	 */
	@PostMapping("/exists")
	@ApiOperation(value = "校验表或视图是否存在接口")
	public R<String> tableExists(Integer dsId, String tableName) {
		// 1.查询数据源
		ReportDatasource reportDatasource = reportDatasourceService.getByDsId(dsId);
		if (reportDatasource == null) {
			return R.fail("数据源不存在");
		}
		// 2.查询表结构
//		List<ReportColumnDTO> reportColumnList = reportBaseInfoService.selectReportColumnList(reportDatasource,
//				tableName);
		List<TableColumnDto> reportColumnList = dialectContext.getDialect(reportDatasource.getDsTypeSyl()).listTableColumnsByTableViewName(reportDatasource, tableName);
		if (!reportColumnList.isEmpty()) {
			return R.fail("表或视图已存在");
		}
		return R.success("表或视图不存在,可以使用.");
	}
}
