/**
 * 
 */
package org.springblade.cube.model;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import lombok.experimental.Accessors;
import org.springblade.core.mp.base.TenantAppEntity;
import org.springblade.core.mp.base.TenantEntity;
import org.springblade.cube.dto.CubeTableName;
import org.springblade.cube.dto.CubeVisualInfo;
import org.springblade.cube.enums.TableTypeEnum;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.SqlCondition;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.NullSerializer;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * @author wookvn
 *
 */
@Data
@TableName("cube_data")
@ApiModel("数据集结构体")
@Accessors(chain = true)
public class CubeData extends TenantAppEntity implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	@TableId(type = IdType.AUTO)
	private Integer id;
	@ApiModelProperty("数据集 ID")
	private Integer cubeId;
	@ApiModelProperty("数据源ID")
	private Integer datasourceId;
	
	@ApiModelProperty("工作空间ID")
	private Integer workspaceId;
	
	@ApiModelProperty("数据集名称")
	@TableField(condition = SqlCondition.LIKE)
	private String cubeName;
	
	@ApiModelProperty("数据集使用的表json数组")
	private String cubeTableNames;
	
	@ApiModelProperty("关联查询的json")
	private String linksJoin;
	
	@ApiModelProperty("全局数据过滤的json")
	private String filterJson;
	
	@ApiModelProperty(hidden = true)
	private String querySql;
	
	@ApiModelProperty("表类型")
	@JsonSerialize(nullsUsing = NullSerializer.class)
	private TableTypeEnum tableType;
	
	@ApiModelProperty("备注")
	private String remark;
	
	@ApiModelProperty("创建时间")
	private Date createTime;
	
	@ApiModelProperty("创建人")
	private String createBy;
	
	@ApiModelProperty("创建人ID")
	@JsonSerialize(using = ToStringSerializer.class)
	private Long createUserId;
	
	@ApiModelProperty("更新时间")
	private Date updateTime;
	
	@ApiModelProperty("是否公开(0-否/1-是)")
	private Integer publicStatus;

	@ApiModelProperty("被复制的数据集id")
	private Integer oldCubeDataId;

	@ApiModelProperty("是否可以被编辑 1：可以，0：不可以")
	private Integer isEdit;

	@TableField(exist = false)
	private List<CubeTableName> tableNames;

	@TableField(exist = false)
	private Integer visualCount;

	@TableField(exist = false)
	private List<CubeVisualInfo> cubeVisualInfos;
	
	@ApiModelProperty("是否显示[静态数据创建的数据集不显示] 1：可以[默认]，0：不可以")
	private Integer displayStatus;
	
	public String getFilterJson() {
		return "[]".equals(filterJson) ? null : filterJson;
	}
}
