/**
 * 
 */
package org.springblade.cube.service;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.api.R;
import org.springblade.cube.dto.CubeVisualInfo;
import org.springblade.cube.mapper.CubeDataMapper;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.model.CubeDataPermission;
import org.springblade.cube.model.CubeSqlView;
import org.springblade.cube.model.CubeTableColumn;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.cube.utils.SnowFlowIntUtil;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.model.TableColumnDto;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.visual.dto.ComponentFilter;
import org.springblade.visual.dto.ComponentFilterRelation;
import org.springblade.visual.dto.PreviewBaseDto;
import org.springblade.visual.service.QuerySqlService;
import org.springblade.visual.utils.Tuple2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Service
@Slf4j
public class CubeDataService extends ServiceImpl<CubeDataMapper, CubeData> {
	
	@Autowired
	private CubeTableColumnService cubeTableColumnService;
	
	@Autowired
	private CubeDataPermissionService cubeDataPermissionService;

	@Autowired
	private DialectContext dialectContext;

	@Autowired
	private CubeSqlViewService cubeSqlViewService;

	@Autowired
	private DynamicTableService dynamicTableService;

	@Autowired
	private QuerySqlService querySqlService;
	/**
	 * 
	 * @param tableName
	 * @return
	 */
	public List<CubeData> listByTableName(Integer dsId, String tableName){
		return baseMapper.listByTableName(dsId, tableName);
	}
	
	/**
	 * 查询拥有权限的数据集分类ID
	 * @param queryWrapper
	 * @return
	 */
	public List<Integer> workspaceIdList(Long userId){
		return baseMapper.workspaceIdList(userId);
	}
	
	/**
	 * 
	 * @param cubeData
	 * @param columnList
	 * @return
	 */
	@Transactional
	public R<Integer> doSaveOrUpdate(CubeData cubeData, List<CubeTableColumn> columnList, List<Integer> dataPermIdList, BladeUser bladeUser){
		cubeData.setTenantId(cubeData.getTenantId());
		int rows = cubeData.getId() != null && cubeData.getId() > 0 ? baseMapper.updateById(cubeData) : baseMapper.insert(cubeData);
		if(rows > 0) {
			List<CubeTableColumn> uptList = new ArrayList<>();
			List<CubeTableColumn> addList = new ArrayList<>();
			for(CubeTableColumn tableColumn : columnList) {
				tableColumn.setCubeDataId(cubeData.getCubeId());
				tableColumn.setTenantId(bladeUser.getTenantId());
				tableColumn.setCubeTableColumnId(SnowFlowIntUtil.nextId());
				if(tableColumn.getId() == null || tableColumn.getId() <= 0) {
					tableColumn.setId(null);
					addList.add(tableColumn);
				}else {
					uptList.add(tableColumn);
				}
			}
			//查询旧的ID
			List<Integer> columnIdList = cubeTableColumnService.listObjs(Wrappers.<CubeTableColumn>lambdaQuery().eq(CubeTableColumn::getCubeDataId, cubeData.getId()),t -> Integer.valueOf(t.toString()));
			if(!columnIdList.isEmpty()) {
				//查询需要更新的ID
				List<Integer> uptIdList = uptList.stream().map(CubeTableColumn::getId).collect(Collectors.toList());
				//分析出来需要删除的ID
				List<Integer> removeIdList = columnIdList.stream().filter(f -> !uptIdList.contains(f)).collect(Collectors.toList());
				if(!removeIdList.isEmpty()) {
					cubeTableColumnService.removeByIds(removeIdList);
				}
			}
			//新增和修改
			if(!addList.isEmpty()) cubeTableColumnService.saveBatch(addList);
			if(!uptList.isEmpty()) cubeTableColumnService.updateBatchById(uptList);
			
			//仅增加数据集时处理：权限组处理【前端在编辑时，不显示权限组选择】
			if(CollectionUtil.isNotEmpty(dataPermIdList)) {
				List<CubeDataPermission> entityList = new ArrayList<>();
				for(Integer dataPermId : dataPermIdList) {
					CubeDataPermission cubeDataPerm = new CubeDataPermission();
					cubeDataPerm.setCreateUserId(bladeUser.getUserId());
					cubeDataPerm.setCreateTime(new Date());
					cubeDataPerm.setCubeDataId(cubeData.getCubeId());
//					cubeDataPerm.setCubeWorkspaceId(cubeWorkspaceId);
					cubeDataPerm.setCubePermName("过滤器"+cubeData.getId());
					cubeDataPerm.setDataPermId(dataPermId);
					entityList.add(cubeDataPerm);
				}
				cubeDataPermissionService.saveBatch(entityList);
			}
		}
		return rows > 0 ? R.data(cubeData.getId(), "操作成功") : R.fail("操作失败");
	}

	/**
	 * 拷贝数据集
	 */
//	@Transactional
	public R<Integer> copy(String title,Integer cubeDataId,Integer workspaceId, String tenantId, BladeUser user, ReportDatasource reportDatasource) {
		CubeData cubeData = this.getById(cubeDataId);
		cubeData.setWorkspaceId(workspaceId);
		cubeData.setCubeName(title);
		String cubeTableNames = cubeData.getCubeTableNames();
		JSONArray jsonArray = JSONUtil.parseArray(cubeTableNames);
		JSONObject json = JSONUtil.parseObj(jsonArray.get(0));
		String tableName = null;
		if (json.containsKey("tableName")){
			tableName = json.getStr("tableName");
		}
		if (StrUtil.isEmpty(tableName)){
			R.fail("查询表名失败");
		}
		//首先去查询视图
		CubeSqlView cubeSqlView = cubeSqlViewService.getOne(Wrappers.<CubeSqlView>lambdaQuery()
				.eq(CubeSqlView::getDatasourceId, cubeData.getDatasourceId())
				.eq(CubeSqlView::getViewName, tableName));
		CubeSqlView view = null;
		// 视图不存在
		if (cubeSqlView == null || cubeSqlView.getId() < 1){
			// 判断一下新的数据源的表是否存在
			List<TableColumnDto> reportColumnList = dialectContext.getDialect(reportDatasource.getDsTypeSyl()).listTableColumnsByTableViewName(reportDatasource, tableName);
			if (reportColumnList.isEmpty()) {
				return R.fail("表不存在...");
			}
		} else {
			// 视图存在则需要创建视图
			view = createView(cubeSqlView, reportDatasource);
			if (view.getId() != null){
				return R.fail(view.getSqlStatement());
			}
		}

		// TODO 校验表或者视图的字段
		// 保存数据
		// 被复制的租户id不为空则表示为跨租户复制
		if (StrUtil.isNotEmpty(tenantId)) {
			cubeData.setTenantId(tenantId);
		}
		boolean save = isSave(user, reportDatasource, cubeData, view, tenantId);

		return save ? R.data(cubeData.getId(), "操作成功") : R.fail("操作失败");
	}

	private boolean isSave(BladeUser user, ReportDatasource reportDatasource, CubeData cubeData, CubeSqlView view, String tenantId) {
		// 查询数据集的列的集合
		List<CubeTableColumn> cubeTableColumns = cubeTableColumnService.list(Wrappers.<CubeTableColumn>lambdaQuery()
				.eq(CubeTableColumn::getCubeDataId, cubeData.getId()));
		// 查询数据集的权限组
		List<CubeDataPermission> cubeDataPermissions = cubeDataPermissionService.list(Wrappers.<CubeDataPermission>lambdaQuery()
				.eq(CubeDataPermission::getCubeDataId, cubeData.getId()));
		// 数据集数据复制
		Integer id = cubeData.getId();
		cubeData.setOldCubeDataId(id);
		cubeData.setId(null);
		cubeData.setCreateTime(new Date());
		if (StrUtil.isNotEmpty(tenantId)) {
			cubeData.setCreateBy("管理员");
			cubeData.setCreateUserId(0L);
		} else {
			cubeData.setCreateBy(user.getNickName());
			cubeData.setCreateUserId(user.getUserId());
		}
		cubeData.setDatasourceId(reportDatasource.getId());
		boolean save = this.save(cubeData);
		if (save){
			if (view != null){
				view.setCreateTime(new Date());
				view.setDatasourceId(reportDatasource.getId());
				if (StrUtil.isNotEmpty(tenantId)) {
					view.setTenantId(tenantId);
					view.setCreateUserId(0L);
				} else {
					view.setCreateUserId(user.getUserId());
				}
				cubeSqlViewService.save(view);
			}
			// 复制数据集的列
			List<CubeTableColumn> addList = new ArrayList<>();
			for(CubeTableColumn tableColumn : cubeTableColumns) {
				tableColumn.setCubeDataId(cubeData.getCubeId());
				if(tableColumn.getId() != null && tableColumn.getId() > 0) {
					// 因为是新增所以要将原来的id置为空
					tableColumn.setId(null);
					tableColumn.setCubeDataId(cubeData.getCubeId());
					if (StrUtil.isNotEmpty(tenantId)) {
						tableColumn.setTenantId(tenantId);
					}
					addList.add(tableColumn);
				}
			}
			if (CollectionUtil.isNotEmpty(addList)) {
				// 保存数据集的字段
				save = cubeTableColumnService.saveBatch(addList);
			}
			// 复制权限组
			if(CollectionUtil.isNotEmpty(cubeDataPermissions)) {
				List<CubeDataPermission> entityList = new ArrayList<>();
				for (CubeDataPermission cubeDataPerm : cubeDataPermissions) {
					cubeDataPerm.setId(null);
					if (StrUtil.isNotEmpty(tenantId)) {
						cubeDataPerm.setCreateUserId(0L);
					} else {
						cubeDataPerm.setCreateUserId(user.getUserId());
					}
					cubeDataPerm.setCreateTime(new Date());
					cubeDataPerm.setCubeDataId(cubeData.getCubeId());
					// 替换行过滤器
					String filterJson = exchangeFilterJson(addList,cubeDataPerm.getFilterJson());
					cubeDataPerm.setFilterJson(filterJson);
					// 替换列过滤器
					String excludeColumnIds = exchangeExcludeColumnIds(addList,cubeDataPerm.getExcludeColumnIds());
					cubeDataPerm.setExcludeColumnIds(excludeColumnIds);
					// 设置过滤sql
					DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(reportDatasource.getDsType());
					PreviewBaseDto previewDto = JSON.parseObject(filterJson, PreviewBaseDto.class);
					if (previewDto != null) {
						//解析SQL
						Tuple2<String, String> whereHavingSqlTuple = querySqlService.parseFilters(JSON.parseArray(previewDto.getFilterList(), ComponentFilter.class),
								JSON.parseObject(previewDto.getFilterRelation(), ComponentFilterRelation.class), dsTypeEnum, previewDto.getExtraParams());
						cubeDataPerm.setFilterSql(whereHavingSqlTuple.getT1());
					}
					entityList.add(cubeDataPerm);
				}
				save = cubeDataPermissionService.saveBatch(entityList);
			}
		}
		return save;
	}

	/**
	 * 替换列过滤器
	 */
	public String exchangeExcludeColumnIds(List<CubeTableColumn> addList, String excludeColumnIds) {
		StringBuilder columnIds = new StringBuilder();
		// 如果列过滤为空则直接返回空
		if (StrUtil.isEmpty(excludeColumnIds) || CollectionUtil.isEmpty(addList)) {
			return null;
		}
		List<String> list = Arrays.asList(excludeColumnIds.split(","));
		List<CubeTableColumn> cubeTableColumns = cubeTableColumnService.getBaseMapper().selectBatchIds(list);
		if (CollectionUtil.isNotEmpty(cubeTableColumns)) {
			for (CubeTableColumn cubeTableColumn : cubeTableColumns) {
				String columnComment = StrUtil.isNotEmpty(cubeTableColumn.getColumnComment()) ? cubeTableColumn.getColumnComment() : "";
				String columnName = StrUtil.isNotEmpty(cubeTableColumn.getColumnName()) ? cubeTableColumn.getColumnName() : "";
				String tableName = StrUtil.isNotEmpty(cubeTableColumn.getTableName()) ? cubeTableColumn.getTableName() : "";
				for (CubeTableColumn tableColumn : addList) {
					String tColumnComment = StrUtil.isNotEmpty(tableColumn.getColumnComment()) ? tableColumn.getColumnComment() : "";
					String tColumnName = StrUtil.isNotEmpty(tableColumn.getColumnName()) ? tableColumn.getColumnName() : "";
					String tTableName = StrUtil.isNotEmpty(tableColumn.getTableName()) ? tableColumn.getTableName() : "";
					if (columnComment.equals(tColumnComment) && columnName.equals(tColumnName)
							&& tableName.equals(tTableName)){
						// 复制的新的数据集需要过滤的列
						columnIds.append(tableColumn.getId()).append(",");
					}
				}
			}
		}
		return columnIds != null ? columnIds.substring(0,columnIds.length()-1) : null;
	}

	/**
	 * 替换行过滤器
	 */
	public String exchangeFilterJson(List<CubeTableColumn> addList, String filterJson) {
		// 如果为空则直接返回
		if (StrUtil.isEmpty(filterJson) || CollectionUtil.isEmpty(addList)) {
			return null;
		}
		JSONObject entries = JSONUtil.parseObj(filterJson);
		String filterList = entries.getStr("filterList");
		// 如果过滤列表为空则没有必要向下操作直接返回
		if (StrUtil.isEmpty(filterList)) {
			return filterJson;
		}
		JSONArray jsonArray = JSONUtil.parseArray(filterList);
		JSONArray arrays = new JSONArray();
		for (Object obj : jsonArray) {
			JSONObject json = JSONUtil.parseObj(obj);
			String column = json.getStr("column");
			// 如果过滤行为空则没有必要向下操作直接封装返回即可
			if (StrUtil.isEmpty(column)) {
				arrays.add(json);
			} else {
				JSONObject columJson = JSONUtil.parseObj(column);
				// 这里要做判空处理，因为有的字段是没有中文注释的
				String columnComment = StrUtil.isNotEmpty(columJson.getStr("columnComment")) ? columJson.getStr("columnComment") : "";
				String columnName = StrUtil.isNotEmpty(columJson.getStr("columnName")) ? columJson.getStr("columnName") : "";
				String tableName = StrUtil.isNotEmpty(columJson.getStr("tableName")) ? columJson.getStr("tableName") : "";
				for (CubeTableColumn cubeTableColumn : addList) {
					// 这里要做判空处理，因为有的字段是没有中文注释的
					String tColumnComment = StrUtil.isNotEmpty(cubeTableColumn.getColumnComment()) ? cubeTableColumn.getColumnComment() : "";
					String tColumnName = StrUtil.isNotEmpty(cubeTableColumn.getColumnName()) ? cubeTableColumn.getColumnName() : "";
					String tTableName = StrUtil.isNotEmpty(cubeTableColumn.getTableName()) ? cubeTableColumn.getTableName() : "";
					if (columnComment.equals(tColumnComment) && columnName.equals(tColumnName)
							&& tableName.equals(tTableName)){
						json.set("cubeColumnId",cubeTableColumn.getId());
						arrays.set(json);
					}
				}
			}

		}
		// 最后的结果
		entries.set("filterList",arrays);

		return entries.toString();
	}

	/**
	 * 根据表名创建数据集
	 * @param datasourceId
	 * @param tableName
	 * @return
	 */
	public Integer createCubeByTable(Integer datasourceId, String tableName, String tableComment) {
		//1.根据表名查询字段列表
		//2.
		return 0;
	}
	
	/**
	 * 创建视图
	 */
	private CubeSqlView createView(CubeSqlView cubeSqlView,ReportDatasource reportDatasource) {
		//创建视图
		long startTime = System.currentTimeMillis();
		String createOrReplaceViewSql = String.format(CubeConstants.CREATE_REPLACE_VIEW_SQL_FORMAT, cubeSqlView.getViewName(), cubeSqlView.getSqlStatement());
		try {
			dynamicTableService.executeSql(reportDatasource, createOrReplaceViewSql);
		} catch (Exception e) {
			log.error(e.getMessage());
			cubeSqlView.setSqlStatement(e.getCause().getMessage());
			return cubeSqlView;
		}
		log.info("sql视图创建完成|datasourceId={},viewName={},cost={}ms", cubeSqlView.getDatasourceId(), cubeSqlView.getViewName(), System.currentTimeMillis() - startTime);
		CubeSqlView cubeSqlView1 = new CubeSqlView();
		BeanUtil.copyProperties(cubeSqlView,cubeSqlView1);
		cubeSqlView1.setCreateTime(new Date());
		cubeSqlView1.setId(null);

		return cubeSqlView1;
	}
	
	/**
	 * 根据租户id查询数据集
	 */

	public List<CubeData> getByTenantId(String tenantId) {

		return baseMapper.getList(tenantId);
	}

	/**
	 * 根据租户id，数据集id集合获取数据集信息
	 * @param in
	 * @return
	 */
	public List<CubeData> getCubeDatalist(String tenantId,List<Integer> cubeDataIds) {

		return baseMapper.selects(tenantId,cubeDataIds);
	}

	/**
	 * 根据数据集 id 集合查询大屏
	 */
    public List<CubeVisualInfo> getVisualInfos(List<Integer> cubeIds) {

		return baseMapper.getVisualInfos(cubeIds);
    }

	public CubeData getByCubeId(Serializable id) {
		return	this.lambdaQuery().eq(CubeData::getCubeId, id).one();
	}
}
