/**
 * 
 */
package org.springblade.cube.service;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.cube.enums.AggregationTypeEnum;
import org.springblade.cube.enums.ColumnAttrEnum;
import org.springblade.cube.enums.DimensionTypeEnum;
import org.springblade.cube.mapper.CubeTableColumnMapper;
import org.springblade.cube.model.CubeTableColumn;
import org.springblade.datasource.enums.DateFormatEnum;
import org.springblade.datasource.enums.NumberFormatEnum;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.datasource.model.TableColumnDto;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author wookvn
 *
 */
@Service
public class CubeTableColumnService extends ServiceImpl<CubeTableColumnMapper, CubeTableColumn> {
    
    public List<CubeTableColumn> listByCubeId(Integer cubeId){
    	return baseMapper.listByCubeId(cubeId);
    }

    public List<CubeTableColumn> listByCubeIds(List<Integer> cubeIds) {
        return baseMapper.listByCubeIds(cubeIds);
    }

    /**
     * 物理表字段列表转数据集字段列表
     * @param physicsTableColumnList
     * @param tableName
     * @param tableAlias
     * @return
     */
    public List<CubeTableColumn> physicsTableColumnToCubeTableColumn(Integer cubeDataId, List<TableColumnDto> physicsTableColumnList, String tableName, String tableAlias, int sortNum) {
        List<CubeTableColumn> cubeTableColumnList = new ArrayList<CubeTableColumn>();
        for(TableColumnDto tableColumnDto : physicsTableColumnList){
            cubeTableColumnList.add(physicsTableColumnToCubeTableColumn(null, cubeDataId, tableColumnDto, tableName, tableAlias, sortNum));
        }
        return cubeTableColumnList;
    }

    /**
     *
     * @param physicsTableColumnMap - 数据集字段ID和物理表字段属性的关系
     * @return
     */
    public List<CubeTableColumn> physicsTableColumnToCubeTableColumn(Map<Integer, TableColumnDto> physicsTableColumnMap, Integer cubeDataId, int sortNum) {
        List<CubeTableColumn> cubeTableColumnList = new ArrayList<CubeTableColumn>();
        for(Map.Entry<Integer, TableColumnDto> entry : physicsTableColumnMap.entrySet()) {
            Integer cubeColumnId = entry.getKey();
            TableColumnDto tableColumnDto = entry.getValue();
            cubeTableColumnList.add(physicsTableColumnToCubeTableColumn(cubeColumnId, cubeDataId, tableColumnDto, tableColumnDto.getTableName(), tableColumnDto.getTableAlias(), sortNum+=100));
        }
        return cubeTableColumnList;
    }

    public CubeTableColumn physicsTableColumnToCubeTableColumn(Integer cubeColumnId, Integer cubeDataId, TableColumnDto columnDto, String tableName, String tableAlias, int sortNum){
        CubeTableColumn cubeTableColumn = new CubeTableColumn();
        cubeTableColumn.setId(cubeColumnId);
        cubeTableColumn.setCubeDataId(cubeDataId);
        cubeTableColumn.setTableName(tableName);
        cubeTableColumn.setAggregationType(AggregationTypeEnum.SUM);
        cubeTableColumn.setColumnAttr(ColumnAttrEnum.NORMAL);
        cubeTableColumn.setColumnComment(columnDto.getColumnComments());
        cubeTableColumn.setColumnName(columnDto.getColumnName());
        cubeTableColumn.setColumnTitle(StrUtil.isEmpty(columnDto.getColumnComments()) ? columnDto.getColumnName() : columnDto.getColumnComments());
        cubeTableColumn.setViewDataType(columnDto.getColumnType());
        cubeTableColumn.setOldViewDataType(columnDto.getColumnType());
        cubeTableColumn.setDimensionType(columnDto.getColumnType() == ViewColumnTypeEnum.NUMBER ? DimensionTypeEnum.ZHIBIAO : DimensionTypeEnum.WEIDU);
        cubeTableColumn.setHiddenStatus(0);
        cubeTableColumn.setSortNum(sortNum);
        cubeTableColumn.setTableAliasName(tableAlias);
        cubeTableColumn.setColumnSql(tableAlias + "." + columnDto.getColumnName());
        cubeTableColumn.setNumberDataFormat(NumberFormatEnum.AUTO);
        cubeTableColumn.setTimeDataFormat(DateFormatEnum.yyyy_MM_ddhh$mm$ss);
        return cubeTableColumn;
    }
}
