/**
 * 
 */
package org.springblade.datasource;

import org.springblade.cube.model.FileStructure;
import org.springblade.datasource.enums.DateFormatEnum;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.enums.FunctionMethodEnum;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.datasource.model.TableColumnDto;
import org.springblade.datasource.model.TableViewDto;
import org.springblade.report.entity.ReportDatasource;

import java.util.List;

/**
 * @author wookvn
 *
 *	
 */
public interface IGmetryDialect {

	/**
	 * 查询表或视图列表
	 * @param datasource
	 * @return
	 */
	public List<TableViewDto> listTableViewsByDatasource(ReportDatasource datasource);

	/**
	 * 查询某一个视图或表的信息
	 * @param datasource
	 * @param tableViewName
	 * @return
	 */
	public List<TableViewDto> listTableViewsByDatasource(ReportDatasource datasource, String tableViewName);
	
	/**
	 * 查询字段列表
	 * @param datasource
	 * @param tableViewName
	 * @return
	 */
	public List<TableColumnDto> listTableColumnsByTableViewName(ReportDatasource datasource, String tableViewName);
	
	/**
	 * 通过上传的文件生成创建表的SQL
	 * @param tableName
	 * @param tableComment
	 * @param structureList
	 * @return
	 */
	public List<String> buildCreateTableSql(String tableName, String tableComment, List<FileStructure> structureList);
	
	/**
	 * 构造创建视图的SQL
	 * @param viewName
	 * @param sqlStatement
	 * @return
	 */
	public String buildCreateViewSql(String viewName, String sqlStatement);
	
	/**
	 * 数据的类型转换
	 * @param oldColumnType
	 * @param newColumnType
	 * @param columnSql
	 * @param dateFormat
	 * @return
	 */
	public String convertDataType(ViewColumnTypeEnum oldColumnType, ViewColumnTypeEnum newColumnType, String columnSql, DateFormatEnum dateFormat);
	
	/**
	 * 获取字段类型数据
	 * @param columnType
	 * @return
	 */
	public String getColumnType(ViewColumnTypeEnum columnType, Integer columnSize, Integer decimalDigits);
	
	/**
	 * 获取日期格式化后的字符串
	 * @param dateTimeFormat
	 * @return
	 */
	public String formatDateTime(DateFormatEnum dateTimeFormat, String columnSql);
	
	/**
	 * 获取方法函数格式化后的字符串
	 * @param functionMethod
	 * @param args
	 * @return
	 */
	public String formatMethodFunction(FunctionMethodEnum functionMethod, Object... args);
	
	/**
	 * 数据源类型
	 * @return
	 */
	public DsTypeEnum dsType();
	
	/**
	 * 执行SQL
	 * @param sqls
	 */
	public void executeSql(List<String> sqls, ReportDatasource datasource) throws Exception;

	/**
	 * 复制数据源
	 * @param oldDs
	 * @return
	 */
	public ReportDatasource copy(ReportDatasource oldDs);
	/**
	 * 复制表
	 * @return
	 */
	public ReportDatasource tableCopy(ReportDatasource reportDatasource,List<TableViewDto> tableViewList);

	/**
	 * 创建数据库
	 */
	public void createDs(String dbName ,ReportDatasource ds ) throws Exception;

}
