/**
 * 
 */
package org.springblade.datasource.dialect;

import com.alibaba.druid.util.JdbcUtils;
import org.springblade.cube.model.FileStructure;
import org.springblade.datasource.AbstractGmetryDialect;
import org.springblade.datasource.enums.DateFormatEnum;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.enums.FunctionMethodEnum;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.datasource.model.TableViewDto;
import org.springblade.datasource.properties.CustomDsDetail;
import org.springblade.datasource.properties.CustomDsProperties;
import org.springblade.datasource.support.DynamicDataSourceSupport;
import org.springblade.datasource.util.NonNumericStringGenerator;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.Statement;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author wookvn
 *
 */
@Component
public class MysqlGmetryDialect extends AbstractGmetryDialect {
	@Autowired
	private CustomDsProperties customDsProperties;
	@Autowired
	private IReportDatasourceService reportDatasourceService;

	{
		//数据类型
		columnTypeMap.put(ViewColumnTypeEnum.NUMBER, "decimal(%s, %s)");
		columnTypeMap.put(ViewColumnTypeEnum.STRING, "varchar(%s)");
		columnTypeMap.put(ViewColumnTypeEnum.DATETIME, "datetime");
		columnTypeMap.put(ViewColumnTypeEnum.BIGINT, "bigint");
		columnTypeMap.put(ViewColumnTypeEnum.INT, "int");
		columnTypeMap.put(ViewColumnTypeEnum.LARGE_STRING, "text");

		//函数方法
		functionMethodMap.put(FunctionMethodEnum.DATE_TO_STR, "DATE_FORMAT(%s, '%s')");
		functionMethodMap.put(FunctionMethodEnum.STR_TO_DATE, "STR_TO_DATE(%s, '%s')");
		functionMethodMap.put(FunctionMethodEnum.NUMBER_TO_DATE, "STR_TO_DATE(CONCAT(%s, ''), '%s')");
		functionMethodMap.put(FunctionMethodEnum.NUMBER_JOINT_PERCENT, "CONCAT(%s, '%%')");
		
		//日期格式化函数
		dateFormatMap.put(DateFormatEnum.yyyy, "%Y");
		dateFormatMap.put(DateFormatEnum.yyyyMM, "%Y%m");
		dateFormatMap.put(DateFormatEnum.yyyy_MM, "%Y-%m");
		dateFormatMap.put(DateFormatEnum.yyyy$MM, "%Y/%m");
		dateFormatMap.put(DateFormatEnum.yyyyMMdd, "%Y%m%d");
		dateFormatMap.put(DateFormatEnum.yyyy_MM_dd, "%Y-%m-%d");
		dateFormatMap.put(DateFormatEnum.yyyy$MM$dd, "%Y/%m/%d");
		dateFormatMap.put(DateFormatEnum.yyyyMMddhh$mm$ss,"%Y%m%d %H:%i:%S");
		dateFormatMap.put(DateFormatEnum.yyyy_MM_ddhh$mm$ss,"%Y-%m-%d %H:%i:%S");
		dateFormatMap.put(DateFormatEnum.yyyy$MM$ddhh$mm$ss,"%Y/%m/%d %H:%i:%S");
		dateFormatMap.put(DateFormatEnum.hh,"%H");
		dateFormatMap.put(DateFormatEnum.hh$mm,"%H:%i");
		dateFormatMap.put(DateFormatEnum.hh$mm$ss,"%H:%i:%S");
	}
	
	@Override
	public List<String> buildCreateTableSql(String tableName, String tableComment, List<FileStructure> structureList) {
		StringBuilder fieldSql = new StringBuilder();
		structureList.stream().forEach(f -> fieldSql.append(String.format("%s %s %s comment '%s',",
				f.getColumnName(), getColumnType(f.getColumnType(), f.getColumnSize(), f.getDecimalDigits()),
				f.isPrimaryKey() ? "primary key" : "", f.getComment())));
		fieldSql.deleteCharAt(fieldSql.length() - 1);
		return Arrays.asList(String.format("create table %s (%s) comment '%s'", tableName, fieldSql.toString(), tableComment));
	}
	
	@Override
	public DsTypeEnum dsType() {
		return DsTypeEnum.mysql;
	}

	@Override
	public ReportDatasource copy(ReportDatasource oldDs) {
		ReportDatasource newDs = new ReportDatasource();
		CustomDsDetail customDsDetail = customDsProperties.getCopyInfo().get(DsTypeEnum.mysql);
//		CustomDsDetail customDsDetail = new CustomDsDetail();
//		customDsDetail.setIp("121.204.252.205");
//		customDsDetail.setPort(3306);
//		customDsDetail.setUserName("root");
//		customDsDetail.setPassWord("Platomix@mysql123");
		newDs.setDsIp(customDsDetail.getIp());
		newDs.setDsId(oldDs.getDsId());
		newDs.setDsPort(customDsDetail.getPort());
		newDs.setDsUser(customDsDetail.getUserName());
		newDs.setDsPassword(customDsDetail.getPassWord());
		newDs.setDsDb("");
		newDs.setUrl(DsTypeEnum.mysql.getDriverUrlFunction().apply(newDs));
		String newDsName=oldDs.getDsDb()+"_"+ NonNumericStringGenerator.generate(4);
		newDs.setDsDb(newDsName);
		newDs.setDriverClassName("com.mysql.jdbc.Driver");
		newDs.setDsTypeSyl(oldDs.getDsTypeSyl());
		newDs.setDsType(oldDs.getDsType());
		newDs.setCreateTime(new Date());
		//获取oldDs建表语句
		try {
			List<TableViewDto> tableViewDtos = super.listTableViewsByDatasource(oldDs);
			List<String> tables = tableViewDtos.stream().map(TableViewDto::getTableViewName).collect(Collectors.toList());
			List<String>createTableSql=getCreateTableSqls(oldDs,tables);
			super.createDs(newDs.getDsDb(),newDs);
			newDs.setUrl(DsTypeEnum.mysql.getDriverUrlFunction().apply(newDs));
			super.executeSql(createTableSql,newDs);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
        return newDs;
	}

	@Override
	public ReportDatasource tableCopy(ReportDatasource reportDatasource, List<TableViewDto> tableViewList) {
		ReportDatasource newDs = new ReportDatasource();
		if (reportDatasource != null){
			newDs=reportDatasource;
			newDs.setDsDb("");
			newDs.setUrl(DsTypeEnum.mysql.getDriverUrlFunction().apply(newDs));
			newDs.setDsDb(reportDatasource.getDsDb());
		}else {
			CustomDsDetail customDsDetail = customDsProperties.getCopyInfo().get(DsTypeEnum.mysql);
			newDs.setDsIp(customDsDetail.getIp());
//			newDs.setDsId(oldDs.getDsId());
			newDs.setDsPort(customDsDetail.getPort());
			newDs.setDsUser(customDsDetail.getUserName());
			newDs.setDsPassword(customDsDetail.getPassWord());
			newDs.setDsDb("");
			newDs.setUrl(DsTypeEnum.mysql.getDriverUrlFunction().apply(newDs));
			String newDsName="platomix_ds";
			newDs.setDsDb(newDsName);
			newDs.setDriverClassName("com.mysql.jdbc.Driver");
			newDs.setDsTypeSyl(DsTypeEnum.mysql);
			newDs.setDsType(0);
			newDs.setCreateTime(new Date());
		}
		//判断数据库是否已经存在，如果不存在，新建数据库
        try {
			List<String>createTableSqlList =new ArrayList<>();
			boolean checked = checkDs(newDs);
			if (!checked){
				super.createDs(newDs.getDsDb(),newDs);
			}else {
				newDs.setUrl(DsTypeEnum.mysql.getDriverUrlFunction().apply(newDs));
				List<TableViewDto> tableViewDtos = super.listTableViewsByDatasource(newDs);
				for (TableViewDto tableViewDto : tableViewDtos){
					for (TableViewDto tableViewDto1 : tableViewList){
						if (tableViewDto.getTableViewName().equals(tableViewDto1.getTableViewName())){
							throw new RuntimeException("表名重复");
						}
					}
				}
			}
			Map<Integer, ReportDatasource>datasourceMap = new HashMap<>();
			for (TableViewDto tableViewDto : tableViewList){
				ReportDatasource datasource = null;
				if (datasourceMap.containsKey(tableViewDto.getDatasourceId())){
					datasource = datasourceMap.get(tableViewDto.getDatasourceId());
				}else {
					datasource = reportDatasourceService.getById(tableViewDto.getDatasourceId());
					datasourceMap.put(tableViewDto.getDatasourceId(), datasource);
				}
				List<String> tables = new ArrayList<>();
				tables.add(tableViewDto.getTableViewName());
				List<String>createTableSql=getCreateTableSqls(datasource,tables);
				createTableSqlList.addAll(createTableSql);
			}
			super.executeSql(createTableSqlList,newDs);
		} catch (Exception e) {
            throw new RuntimeException(e);
        }
        return newDs;
	}

	public boolean checkDs(ReportDatasource datasource)  throws Exception {
		Connection connection = null;
		Statement statement = null;
		try {
			connection = DynamicDataSourceSupport.createConnection(datasource);
			DatabaseMetaData metaData = connection.getMetaData();
			ResultSet resultSet = metaData.getCatalogs();
			while (resultSet.next()) {
				String catalog = resultSet.getString(1);
				if (datasource.getDsDb().equalsIgnoreCase(catalog)) {
					return true;
				}
			}
		} catch(Exception e){
			if(connection != null){
				connection.rollback();
			}
			throw e;
		} finally {
			JdbcUtils.close(statement);
			JdbcUtils.close(connection);
		}
		return false;
	}
	public List<String> getCreateTableSqls(ReportDatasource datasource, List<String> tables)  throws Exception {
		Connection connection = null;
		Statement statement = null;
		List<String> createTableSqls = new ArrayList<>();
		try {
			connection = DynamicDataSourceSupport.createConnection(datasource);
			statement = connection.createStatement();
			for (String table : tables) {
				ResultSet resultSet = statement.executeQuery("SHOW CREATE TABLE " + table + ";");
				if (resultSet.next()) {
					createTableSqls.add(resultSet.getString(2));
				}
			}
		} catch(Exception e){
			if(connection != null){
				connection.rollback();
			}
			throw e;
		} finally {
			JdbcUtils.close(statement);
			JdbcUtils.close(connection);
		}
		return createTableSqls;
	}


	public static void main(String[] args) {
		ReportDatasource oldDs = new ReportDatasource();
		oldDs.setDsIp("121.204.252.205");
		oldDs.setDsPort(3306);
		oldDs.setDsUser("root");
		oldDs.setDsPassword("Platomix@mysql123");
		oldDs.setDsDb("apimngr");
		oldDs.setUrl(DsTypeEnum.mysql.getDriverUrlFunction().apply(oldDs));
		oldDs.setDriverClassName("com.mysql.jdbc.Driver");
		oldDs.setDsTypeSyl(DsTypeEnum.mysql);
		oldDs.setDsType(0);
		MysqlGmetryDialect mysqlGmetryDialect = new MysqlGmetryDialect();
		ReportDatasource copy = mysqlGmetryDialect.copy(oldDs);
		System.out.println(copy);

	}

}
