/**
 * 
 */
package org.springblade.datasource.dialect;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import org.springblade.cube.model.FileStructure;
import org.springblade.datasource.AbstractGmetryDialect;
import org.springblade.datasource.enums.DateFormatEnum;
import org.springblade.datasource.enums.DetailColumnTypeEnum;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.enums.FunctionMethodEnum;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.datasource.model.TableColumnDto;
import org.springblade.datasource.model.TableViewDto;
import org.springblade.report.dto.oracle.ReportColumnDTO;
import org.springblade.report.dto.oracle.ReportViewDTO;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.mapper.ReportBaseInfoMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.LinkedList;
import java.util.List;

/**
 * @author wookvn
 *
 */
@Component
public class SqlServerGmetryDialect extends AbstractGmetryDialect {
	
	@Autowired
	private ReportBaseInfoMapper reportBaseInfoMapper;
	
	{
		//数据类型
		columnTypeMap.put(ViewColumnTypeEnum.NUMBER, "decimal(%s, %s)");
		columnTypeMap.put(ViewColumnTypeEnum.STRING, "varchar(%s)");
		columnTypeMap.put(ViewColumnTypeEnum.DATETIME, "datetime");
		columnTypeMap.put(ViewColumnTypeEnum.BIGINT, "bigint");
		columnTypeMap.put(ViewColumnTypeEnum.INT, "int");
		columnTypeMap.put(ViewColumnTypeEnum.LARGE_STRING, "text");
		
		//函数方法
		functionMethodMap.put(FunctionMethodEnum.DATE_TO_STR, "FORMAT(%s, '%s')");
		functionMethodMap.put(FunctionMethodEnum.STR_TO_DATE, "CONVERT(datetime, %s)");
		functionMethodMap.put(FunctionMethodEnum.NUMBER_TO_DATE, "CONVERT(datetime, convert(varchar(50), %s))");
		functionMethodMap.put(FunctionMethodEnum.NUMBER_JOINT_PERCENT, "CONVERT(varchar(50), %s)+'%%'");
		
		//日期格式化
		dateFormatMap.put(DateFormatEnum.yyyy, "yyyy");
		dateFormatMap.put(DateFormatEnum.yyyyMM, "yyyyMM");
		dateFormatMap.put(DateFormatEnum.yyyy_MM, "yyyy-MM");
		dateFormatMap.put(DateFormatEnum.yyyy$MM, "yyyy/MM");
		dateFormatMap.put(DateFormatEnum.yyyyMMdd, "yyyyMMdd");
		dateFormatMap.put(DateFormatEnum.yyyy_MM_dd, "yyyy-MM-dd");
		dateFormatMap.put(DateFormatEnum.yyyy$MM$dd, "yyyy/MM/dd");
		dateFormatMap.put(DateFormatEnum.yyyyMMddhh$mm$ss,"yyyyMMdd hh:mm:ss");
		dateFormatMap.put(DateFormatEnum.yyyy_MM_ddhh$mm$ss,"yyyy-MM-dd hh:mm:ss");
		dateFormatMap.put(DateFormatEnum.yyyy$MM$ddhh$mm$ss,"yyyy/MM/dd hh:mm:ss");
		dateFormatMap.put(DateFormatEnum.hh,"hh");
		dateFormatMap.put(DateFormatEnum.hh$mm,"hh:mm");
		dateFormatMap.put(DateFormatEnum.hh$mm$ss,"hh:mm:ss");
	}

	@Override
	public List<String> buildCreateTableSql(String tableName, String tableComment, List<FileStructure> structureList) {
		String createColumnCommentSql = "EXEC sp_addextendedproperty N'MS_Description', N'%s', N'user', N'dbo', N'table', N'%s', N'COLUMN', N'%s'",
			   createTableCommentSql = "EXEC sp_addextendedproperty N'MS_Description', N'%s', N'user', N'dbo', N'table', N'%s', NULL, NULL",
			   createTableSql = "create table %s (%s)";
		List<String> sqlList = new LinkedList<String>();
		StringBuilder fieldSql = new StringBuilder();
		structureList.stream().forEach(f -> {
			fieldSql.append(String.format("%s %s %s,", 
					f.getColumnName(), getColumnType(f.getColumnType(), f.getColumnSize(), f.getDecimalDigits()),
					f.isPrimaryKey() ? "primary key" : ""));
			//添加字段注释SQL
			sqlList.add(String.format(createColumnCommentSql, f.getComment(), tableName, f.getColumnName()));
		});
		fieldSql.deleteCharAt(fieldSql.length() - 1);
		//创建表SQL
		sqlList.add(0, String.format(createTableSql, tableName, fieldSql.toString()));
		//添加表注释SQL
		sqlList.add(1, String.format(createTableCommentSql, tableComment, tableName));
		return sqlList;
	}
	
	@Override
	public List<TableColumnDto> listTableColumnsByTableViewName(ReportDatasource datasource, String tableViewName) {
		List<ReportColumnDTO> columnDataList = reportBaseInfoMapper.getColumnListOfSqlserver(datasource.getPoolName(), tableViewName);
		if(CollectionUtil.isNotEmpty(columnDataList)) {
			List<TableColumnDto> dataList = BeanUtil.copyToList(columnDataList, TableColumnDto.class);
			dataList.stream().forEach(f -> {
				if(StrUtil.containsAnyIgnoreCase(f.getDataType(), "char", "text")) {
					f.setColumnType(ViewColumnTypeEnum.STRING);
					f.setDetailColumnType(DetailColumnTypeEnum.STRING);
				}else if(StrUtil.containsIgnoreCase(f.getDataType(), "date")) {
					f.setColumnType(ViewColumnTypeEnum.DATETIME);
					f.setDetailColumnType(DetailColumnTypeEnum.DATETIME);
				}else if(StrUtil.containsAnyIgnoreCase(f.getDataType(), "float", "double", "real", "decimal")){
					f.setColumnType(ViewColumnTypeEnum.NUMBER);
					f.setDetailColumnType(DetailColumnTypeEnum.FLT);
				}else if(StrUtil.containsAnyIgnoreCase(f.getDataType(), "int64")){
					f.setColumnType(ViewColumnTypeEnum.NUMBER);
					f.setDetailColumnType(DetailColumnTypeEnum.LONG);
				}else{
					f.setColumnType(ViewColumnTypeEnum.NUMBER);
					f.setDetailColumnType(DetailColumnTypeEnum.INT);
				}
			});
			return dataList;
		}
		return null;
	}
	
	@Override
	public List<TableViewDto> listTableViewsByDatasource(ReportDatasource datasource) {
		List<ReportViewDTO> dataList = reportBaseInfoMapper.getTableListOfSqlserver(datasource.getPoolName(), null);
		if(CollectionUtil.isNotEmpty(dataList)) {
			dataList.stream().forEach(item -> item.setDatasourceId(datasource.getId()));
		}
		return BeanUtil.copyToList(dataList, TableViewDto.class);
	}
	
	@Override
	public List<TableViewDto> listTableViewsByDatasource(ReportDatasource datasource, String tableName) {
		List<ReportViewDTO> dataList = reportBaseInfoMapper.getTableListOfSqlserver(datasource.getPoolName(), tableName);
		return BeanUtil.copyToList(dataList, TableViewDto.class);
	}
	
	@Override
	public DsTypeEnum dsType() {
		return DsTypeEnum.sqlserver;
	}

}
