/**
 *
 */
package org.springblade.microapp.controller;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.microapp.entity.MicroApp;
import org.springblade.microapp.service.MicroAppService;
import org.springblade.system.client.DataPermClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * @author wookvn
 *
 */
@RestController
@RequestMapping("/microApp")
@Api(value = "应用接口", tags = "应用相关接口")
public class MicroAppController extends BladeController {

    @Autowired
    private MicroAppService microAppService;

    @Autowired
    private DataPermClient dataPermClient;

    /**
     *
     * @param microApp
     * @param query
     * @return
     */
    @GetMapping("/list")
    @ApiOperation(value = "系统应用列表")
    public R<List<MicroApp>> list(String appName, Integer publishStatus) {
        long userId = getUser().getUserId();
        //构造查询wrapper
        LambdaQueryWrapper<MicroApp> queryWrapper = Wrappers.<MicroApp>lambdaQuery()
                .like(StrUtil.isNotBlank(appName), MicroApp::getAppName, appName)
                .eq(publishStatus != null, MicroApp::getPublicStatus, publishStatus)
                .orderByDesc(MicroApp::getSortNum);
        Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(userId);
        // 超级权限组
        if (checkSuperPermRet != null && checkSuperPermRet) { // 查询非私有的应用【包括：公共的和权限组设置的两类】
            return R.data(microAppService.list(queryWrapper.ne(MicroApp::getPublicStatus, 0)));
        }
        // 查询非超级权限组
        // 1.公开状态(public_status=1)的数据；
        // 2.权限组状态(public_status=2)的数据。
        queryWrapper.and(t -> {
            t.eq(MicroApp::getPublicStatus, 1).or(y -> {
                y.eq(MicroApp::getPublicStatus, 2)
                        .inSql(MicroApp::getAppId, "select b.app_id from micro_app_data_permission b inner join data_permission_user c on b.data_perm_id = c.data_perm_id where c.user_id = " + userId);
            });
        });
        return R.data(microAppService.list(queryWrapper));
    }

    /**
     *
     * @param microApp
     * @param query
     * @return
     */
    @GetMapping("/mylist")
    @ApiOperation(value = "我的应用列表")
    public R<List<MicroApp>> mylist(String appName,Integer publishStatus) {
        LambdaQueryWrapper<MicroApp> queryWrapper = Wrappers.<MicroApp>lambdaQuery();
        if(publishStatus != null){
            queryWrapper.eq(MicroApp::getPublishStatus, publishStatus);
        }
        queryWrapper.eq(MicroApp::getCreateUserId, getUser().getUserId()).eq(MicroApp::getPublicStatus, 0)
                .like(StrUtil.isNotBlank(appName), MicroApp::getAppName, appName).orderByDesc(MicroApp::getSortNum);
        return R.data(microAppService.list(queryWrapper));
    }

    /**
     *
     * @param appId
     * @return
     */
    @GetMapping("/info")
    @ApiOperation(value = "应用详情")
    public R<MicroApp> info(@RequestParam Integer appId) {
        return R.data(microAppService.getById(appId));
    }

    /**
     *
     * @param microApp
     * @return
     */
    @PostMapping("/submit")
    @ApiOperation(value = "添加或修改数据")
    public R<Integer> doSubmit(@RequestBody MicroApp microApp) {
        if (microApp.getAppId() == null || microApp.getAppId() <= 0) {
            microApp.setCreateTime(new Date());
            microApp.setCreateUserId(getUser().getUserId());
        }
        return R.data(microAppService.doSaveOrUpdate(microApp, getUser()));
    }

    /**
     *
     * @return
     */
    @PostMapping("/delete")
    @ApiOperation(value = "删除应用", notes = "多个ID用逗号隔开")
    public R<String> doDelete(@RequestParam String ids) {
        return R.status(microAppService.doRemove(Func.toIntList(ids)));
    }

    /**
     * 调整顺序
     *
     * @return
     */
    @PostMapping("/adjustOrder")
    @ApiOperation(value = "调整顺序", notes = "传入微应用ID列表【切记：保证顺序】")
    public R<String> uptOrder(@ApiParam(value = "主键集合  1,2,3", required = true) @RequestParam String ids) {
        List<MicroApp> entityList = new ArrayList<MicroApp>();
        List<Integer> visualIdList = Func.toIntList(ids);
        Integer[] indexArr = {1};
        visualIdList.forEach(itemId -> {
            MicroApp data = new MicroApp();
            data.setAppId(itemId);
            data.setSortNum(indexArr[0]++);
            entityList.add(data);
        });
        return R.status(microAppService.updateBatchById(entityList));
    }

    /**
     *
     * @return
     */
    @GetMapping("/user/list")
    @ApiOperation(value = "查询人员绑定移动应用")
    public R<List<MicroApp>> getUserMicroApp(Long userId) {
        //构造查询wrapper
        LambdaQueryWrapper<MicroApp> queryWrapper = Wrappers.lambdaQuery();
        // 查询非超级权限组
        // 1.公开状态(public_status=1)的数据；
        // 2.权限组状态(public_status=2)的数据。
        queryWrapper.and(t -> {
            t.inSql(MicroApp::getAppId, "select b.app_id from micro_app_data_permission b inner join data_permission_user c on b.data_perm_id = c.data_perm_id where c.user_id = " + userId)
					.eq(MicroApp::getPublishStatus,1);
        });
        return R.data(microAppService.list(queryWrapper));
    }
}
