/**
 * 
 */
package org.springblade.microapp.controller;

import java.util.List;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.api.R;
import org.springblade.microapp.dto.MicroAppPermDoo;
import org.springblade.microapp.dto.MicroAppPermDto;
import org.springblade.microapp.entity.MicroAppDataPermission;
import org.springblade.microapp.service.MicroAppDataPermissionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * @author wookvn
 *
 */
@RestController
@RequestMapping("/microApp/permission")
@Api(value = "应用权限组接口", tags = "应用权限组接口")
public class MicroAppDataPermissionController extends BladeController {

	@Autowired
	private MicroAppDataPermissionService microAppDataPermissionService;

	/**
	 * 查询微应用有哪些权限组
	 * @param appId
	 * @return
	 */
	@GetMapping("/list/{appId}")
	@ApiOperation(value = "根据应用ID查询权限组列表")
	public R<List<JSONObject>> list(@PathVariable Integer appId) {
		return R.data(microAppDataPermissionService.listDataPerm(appId));
	}
	
	/**
	 * 根据权限组ID查询应用列表
	 * @param dataPermId
	 * @return
	 */
	@GetMapping("/appIdList/{dataPermId}")
	@ApiOperation(value = "根据权限组ID查询应用列表")
	public R<List<Integer>> appIdList(@PathVariable Integer dataPermId){
		LambdaQueryWrapper<MicroAppDataPermission> queryWrapper = Wrappers.<MicroAppDataPermission>lambdaQuery().select(MicroAppDataPermission::getAppId).eq(MicroAppDataPermission::getDataPermId, dataPermId);
		return R.data(microAppDataPermissionService.listObjs(queryWrapper, t -> Integer.valueOf(t.toString())));
	}

	/**
	 * 
	 * @return
	 */
	@PostMapping("/submit")
	@ApiOperation(value = "权限组入口->批量添加或修改", notes = "权限组管理->应用列表")
	public R<String> doSubmit(@RequestBody MicroAppPermDto mircoAppPermDto) {
		return R.status(microAppDataPermissionService.doSubmit(mircoAppPermDto, getUser()));
	}
	
	/**
	 * 
	 * @return
	 */
	@PostMapping("/perform")
	@ApiOperation(value = "应用列表入口->批量添加或修改", notes = "系统应用列表->权限组管理入口")
	public R<String> doPerform(@RequestBody MicroAppPermDoo mircoAppPermDoo) {
		return R.status(microAppDataPermissionService.doPermform(mircoAppPermDoo, getUser()));
	}

}
