/**
 * 
 */
package org.springblade.microapp.controller;

import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.microapp.entity.MicroAppMenu;
import org.springblade.microapp.service.MicroAppMenuService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * @author wookvn
 *
 */
@RestController
@RequestMapping("/microAppMenu")
@Api(value = "应用菜单接口", tags = "应用菜单相关接口")
public class MicroAppMenuController extends BladeController {

	@Autowired
	private MicroAppMenuService microAppMenuService;
	
	/**
	 * 
	 * @param microApp
	 * @param query
	 * @return
	 */
	@GetMapping("/list")
	@ApiOperation(value = "查询某个应用的菜单列表")
	public R<List<MicroAppMenu>> list(@RequestParam Integer appId){
		LambdaQueryWrapper<MicroAppMenu> queryWrapper = Wrappers.<MicroAppMenu>lambdaQuery().eq(MicroAppMenu::getAppId, appId).orderByDesc(MicroAppMenu::getSortNum);
		return R.data(microAppMenuService.list(queryWrapper));
	}
	
	/**
	 * 
	 * @param microApp
	 * @return
	 */
	@PostMapping("/submit")
	@ApiOperation(value = "添加或修改数据")
	public R<Integer> doSubmit(@RequestBody MicroAppMenu microAppMenu){
		if(microAppMenu.getAppId() == null) {
			return R.fail("应用ID不能为空");
		}
		if(microAppMenu.getMenuId() == null) {
			microAppMenu.setCreateTime(new Date());
			microAppMenu.setCreateUserId(getUser().getUserId());
		}
		microAppMenuService.doSubmit(microAppMenu);
		return R.data(microAppMenu.getMenuId());
	}
	
	@PostMapping("/batchUpdate")
	@ApiOperation(value = "批量更新", notes = "根据menuId批量更新基本信息【不需要更新的字段不传递，menuId必须传递】")
	public R<String> batchUpdate(@RequestBody List<MicroAppMenu> microAppMenus){
		for(Iterator<MicroAppMenu> it = microAppMenus.iterator(); it.hasNext(); ) {
			MicroAppMenu appMenu = it.next();
			if(appMenu.getMenuId() == null || appMenu.getMenuId() <= 0) {
				it.remove();
			}
			//设置应用首页显示状态不更新
			appMenu.setHomePageStatus(null);
		}
		return R.status(microAppMenus.isEmpty() ? false : microAppMenuService.updateBatchById(microAppMenus));
	}
	
	/**
	 * 
	 * @return
	 */
	@PostMapping("/delete")
	@ApiOperation(value = "删除菜单", notes = "多个ID用逗号隔开")
	public R<String> doDelete(@RequestParam String ids){
		List<Integer> menuIds = Func.toIntList(ids);
		long count = microAppMenuService.count(Wrappers.<MicroAppMenu>lambdaQuery().in(MicroAppMenu::getParentId, menuIds));
		if(count > 0) {
			return R.fail("请先删除子节点!");
		}
		return R.status(microAppMenuService.removeByIds(menuIds));
	}
}

