/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.report.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.report.entity.HistoryReportFolder;
import org.springblade.report.service.IHistoryReportFolderService;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.time.LocalDateTime;
import java.util.List;

/**
 * 历史报告文件夹 控制器
 *
 * @author BladeX
 * @since 2021-07-02
 */
@RestController
@AllArgsConstructor
@RequestMapping("/historyreportfolder")
@Api(value = "历史报告文件夹", tags = "历史报告文件夹接口 HistoryReportFolderController")
public class HistoryReportFolderController extends BladeController {

	private final IHistoryReportFolderService historyReportFolderService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情 detail", notes = "传入historyReportFolder")
	public R<HistoryReportFolder> detail(HistoryReportFolder historyReportFolder) {
		HistoryReportFolder detail = historyReportFolderService.getOne(Condition.getQueryWrapper(historyReportFolder));
		return R.data(detail);
	}

	/**
	 * 分页 历史报告文件夹
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "列表 list", notes = "传入historyReportFolder")
	public R<List<HistoryReportFolder>> list(HistoryReportFolder historyReportFolder) {
		QueryWrapper queryWrapper = Condition.getQueryWrapper(historyReportFolder);
		queryWrapper.eq("user_id", getUser().getUserId());
		queryWrapper.orderByAsc("sort_order");
		queryWrapper.orderByDesc("create_time");
		List<HistoryReportFolder> list = historyReportFolderService.list(queryWrapper);
		return R.data(list);
	}

	/**
	 * 新增 历史报告文件夹
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "新增 save", notes = "传入historyReportFolder")
	public R<Integer> save(@Valid @RequestBody HistoryReportFolder historyReportFolder) {
		historyReportFolder.setUserId(getUser().getUserId());
		historyReportFolder.setCreateTime(LocalDateTime.now());
		historyReportFolderService.save(historyReportFolder);
		return R.data(historyReportFolder.getId());
	}

	/**
	 * 修改 历史报告文件夹
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "修改 update", notes = "传入historyReportFolder")
	public R update(@Valid @RequestBody HistoryReportFolder historyReportFolder) {
		return R.status(historyReportFolderService.updateById(historyReportFolder));
	}

	/**
	 * 新增或修改 历史报告文件夹
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "新增或修改 submit", notes = "传入historyReportFolder")
	public R<Integer> submit(@Valid @RequestBody HistoryReportFolder historyReportFolder) {
		historyReportFolder.setUserId(getUser().getUserId());
		historyReportFolder.setCreateTime(LocalDateTime.now());
		historyReportFolderService.saveOrUpdate(historyReportFolder);
		return R.data(historyReportFolder.getId());
	}


	/**
	 * 删除 历史报告文件夹
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "删除 remove", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(historyReportFolderService.removeByIds(Func.toLongList(ids)));
	}


}
