/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.report.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.report.entity.IndexInfo;
import org.springblade.report.vo.IndexInfoVO;
import org.springblade.report.service.IIndexInfoService;
import org.springblade.core.boot.ctrl.BladeController;

import java.util.List;

/**
 * 报表指标信息 控制器
 *
 * @author BladeX
 * @since 2021-04-15
 */
@RestController
@AllArgsConstructor
@RequestMapping("/indexinfo")
@Api(value = "报表指标信息", tags = "报表指标信息接口 IndexInfoController")
public class IndexInfoController extends BladeController {

	private final IIndexInfoService indexInfoService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入indexInfo")
	public R<IndexInfo> detail(IndexInfo indexInfo) {
		IndexInfo detail = indexInfoService.getOne(Condition.getQueryWrapper(indexInfo));
		return R.data(detail);
	}

	/**
	 * 分页 报表指标信息
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "列表", notes = "传入indexInfo")
	public R<List<IndexInfo>> list(IndexInfo indexInfo) {
		QueryWrapper queryWrapper = Condition.getQueryWrapper(indexInfo);
		queryWrapper.orderByAsc("order_index");
		queryWrapper.orderByDesc("create_time");
		List<IndexInfo> list = indexInfoService.list(Condition.getQueryWrapper(indexInfo));
		return R.data(list);
	}

	/**
	 * 自定义分页 报表指标信息
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入indexInfo")
	public R<IPage<IndexInfoVO>> page(IndexInfoVO indexInfo, Query query) {
		IPage<IndexInfoVO> pages = indexInfoService.selectIndexInfoPage(Condition.getPage(query), indexInfo);
		return R.data(pages);
	}

	/**
	 * 新增 报表指标信息
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入indexInfo")
	public R save(@Valid @RequestBody IndexInfo indexInfo) {
		return R.status(indexInfoService.save(indexInfo));
	}

	/**
	 * 修改 报表指标信息
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入indexInfo")
	public R update(@Valid @RequestBody IndexInfo indexInfo) {
		return R.status(indexInfoService.updateById(indexInfo));
	}

	/**
	 * 新增或修改 报表指标信息
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入indexInfo")
	public R submit(@Valid @RequestBody IndexInfo indexInfo) {
		return R.status(indexInfoService.saveOrUpdate(indexInfo));
	}


	/**
	 * 删除 报表指标信息
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(indexInfoService.removeByIds(Func.toLongList(ids)));
	}


}
