/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.report.mapper;

import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.apache.ibatis.annotations.Param;
import org.springblade.report.dto.oracle.ReportColumnDTO;
import org.springblade.report.dto.oracle.ReportViewDTO;
import org.springblade.report.entity.OrderMonth;

import java.util.LinkedHashMap;
import java.util.List;

/**
 * 产品报表自定义 Mapper 接口
 *
 * @author BladeX
 * @since 2021-03-11
 */
@InterceptorIgnore(tenantLine = "true")
public interface ReportDaoMapper extends BaseMapper<OrderMonth> {

	/**
	 * oracle查询报表视图
	 *
	 * @return
	 */
	List<ReportViewDTO> selectReportViewList();

	/**
	 * sqlserver查询报表视图
	 *
	 * @return
	 */
	List<ReportViewDTO> selectReportViewListSqlServer();

	/**
	 * clickhouse查询报表视图
	 *
	 * @return
	 */
	List<ReportViewDTO> selectReportViewListClickHouse(@Param("dbName") String dbName);

	/**
	 * mysql查询报表视图
	 *
	 * @return
	 */
	List<ReportViewDTO> selectReportViewListMysql(@Param("dbName") String dbName);

	/**
	 * oracle查询视图字段列表
	 *
	 * @param viewName 视图名称
	 * @return
	 */
	List<ReportColumnDTO> selectReportColumnList(@Param("viewName") String viewName);

	/**
	 * sqlserver查询视图字段列表
	 *
	 * @param viewName 视图名称
	 * @return
	 */
	List<ReportColumnDTO> selectReportColumnListSqlServer(@Param("viewName") String viewName);

	/**
	 * mysql查询视图字段列表
	 *
	 * @param viewName 视图名称
	 * @return
	 */
	List<ReportColumnDTO> selectReportColumnListMysql(@Param("dbName") String dbName, @Param("viewName") String viewName);

	/**
	 * clickhouse查询视图字段列表
	 *
	 * @param viewName 视图名称
	 * @return
	 */
	List<ReportColumnDTO> selectReportColumnListClickHouse(@Param("dbName") String dbName, @Param("viewName") String viewName);

	List<Object> selectRowConvertCol();

	List<LinkedHashMap<String, String>> selectCategoryAndCpNameList();

	List<LinkedHashMap<String, Object>> getReportResult(@Param("sql") String sql);

	/**
	 * 动态查询报表数据
	 *
	 * @param tableViewName
	 * @param rowFieldsSql
	 * @param sql
	 * @return
	 */
	List<LinkedHashMap<String, Object>> getReportData(@Param("tableViewName") String tableViewName, @Param("rowFieldsSql") String rowFieldsSql,
													  @Param("sql") String sql, @Param("whereSql") String whereSql, @Param("orderBySql") String orderBySql);

	List<LinkedHashMap<String, Object>> selectRowToColList(@Param("tableViewName") String tableViewName, @Param("fieldSql") String fieldSql,
														   @Param("whereSql") String whereSql);

	List<LinkedHashMap<String, Object>> getReportDataByCondition(@Param("tableViewName") String tableViewName, @Param("selectSql") String selectSql,
																 @Param("whereSql") String whereSql, @Param("orderBySql") String orderBySql,
																 @Param("groupBySql") String groupBySql,@Param("limitSql") String limitSql);
}
