package org.springblade.resouces;

import cn.hutool.core.exceptions.ExceptionUtil;
import cn.hutool.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.cube.model.*;
import org.springblade.cube.service.*;
import org.springblade.cube.utils.SnowFlowIntUtil;
import org.springblade.datasource.DsHandler;
import org.springblade.report.entity.*;
import org.springblade.report.service.IDataApiService;
import org.springblade.report.service.IDimensionService;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.report.service.ITableCategoryService;
import org.springblade.report.service.normal.ReportDatasourcePermissionService;
import org.springblade.system.client.AppClient;
import org.springblade.system.client.model.PlatomixAppCopyLogModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author 喜欢番茄火锅
 * @date 2024/9/3 17:07
 */
@Service
@Slf4j
public class AppCopyService {
    @Autowired
    private CubeDataService cubeDataService;
    @Autowired
    private CubeDataPermissionService cubeDataPermissionService;
    @Autowired
    private CubeTableColumnService cubeTableColumnService;
    @Autowired
    private CubeTableStaticService cubeTableStaticService;
    @Autowired
    private IReportDatasourceService reportDatasourceService;
    @Autowired
    private CubeSqlViewService cubeSqlViewService;
    @Autowired
    private CubeTableFileService cubeTableFileService;
    @Autowired
    private IDataApiService dataApiService;
    @Autowired
    private IDimensionService dimensionService;
    @Autowired
    private ITableCategoryService tableCategoryService;
    @Autowired
    private ReportDatasourcePermissionService reportDatasourcePermissionService;
    @Autowired
    private DsHandler dsHandler;
    @Resource
    private AppClient appClient;
    @Async
    public void copy(List<Integer> cubeIds, List<Integer> dsIds, String appId, PlatomixAppCopyLogModel platomixAppCopyLogModel) {

        try {
            long start = System.currentTimeMillis();
            copyCube(cubeIds, appId);
            platomixAppCopyLogModel.setCubeResult(0);
            appClient.taskInfo(platomixAppCopyLogModel);
            System.out.println("copyCube 时长=" + (start - System.currentTimeMillis()));
        } catch (Exception e) {
            platomixAppCopyLogModel.setCubeResult(1);
            platomixAppCopyLogModel.setErrorInfo(ExceptionUtil.stacktraceToString(e));
            appClient.taskInfo(platomixAppCopyLogModel);
            log.error("copyCube复制失败:"+ExceptionUtil.stacktraceToString(e));
        }
        try {
            long start1 = System.currentTimeMillis();
            copyDs(dsIds, appId, platomixAppCopyLogModel);
            platomixAppCopyLogModel.setDsResult(0);
            appClient.taskInfo(platomixAppCopyLogModel);
            System.out.println("start1 时长=" + (start1 - System.currentTimeMillis()));
        } catch (Exception e) {
            platomixAppCopyLogModel.setDsResult(1);
            platomixAppCopyLogModel.setErrorInfo(ExceptionUtil.stacktraceToString(e));
            appClient.taskInfo(platomixAppCopyLogModel);
            log.error("数据源 复制失败:"+ExceptionUtil.stacktraceToString(e));
        }

    }

    private void copyDs(List<Integer> dsIds, String appId, PlatomixAppCopyLogModel platomixAppCopyLogModel) {
        List<ReportDatasource> list = reportDatasourceService.lambdaQuery()
                .in(ReportDatasource::getDsId, dsIds)
                .eq(ReportDatasource::getAppId,appId)
                .list();
        List<Integer> collect = list.stream().map(ReportDatasource::getDsId).collect(Collectors.toList());
        if (!list.isEmpty()) {
            List<Integer> newIds = new ArrayList<>();
            //newIds存放 ids 中不包含list.getId()的id
            for (Integer id : dsIds) {
                if (!collect.contains(id)) {
                    newIds.add(id);
                }
            }
            dsIds = newIds;
        }

        long start1 = System.currentTimeMillis();
        if (dsIds.isEmpty()){
            platomixAppCopyLogModel.setDbResult(0);
            appClient.taskInfo(platomixAppCopyLogModel);
            return;
        }
        if (!reportDatasourceService(dsIds, appId, platomixAppCopyLogModel)) {
            return;
        }

        System.out.println("start1 时长=" + (start1 - System.currentTimeMillis()));
        long start2 = System.currentTimeMillis();
        cubeSqlViewService(dsIds, appId);
        cubeTableFileService(dsIds, appId);
        dataApiService(dsIds, appId);
        dimensionService(dsIds, appId);
        dimensionService(dsIds, appId);
        reportDatasourcePermissionService(dsIds, appId);
        //未必需要
        tableCategoryService(dsIds, appId);
        System.out.println("start2 时长=" + (start2 - System.currentTimeMillis()));
    }

    private void tableCategoryService(List<Integer> dsIds, String appId) {
        List<TableCategory> list = tableCategoryService.lambdaQuery().in(TableCategory::getDatasourceId, dsIds).list();
        for (TableCategory obj : list) {
            obj.setId(null);
            obj.setAppId(appId);
            obj.setCreateTime(new Date());
            tableCategoryService.save(obj);
        }
    }

    private void reportDatasourcePermissionService(List<Integer> dsIds, String appId) {
        List<ReportDatasourcePermission> list = reportDatasourcePermissionService.lambdaQuery().in(ReportDatasourcePermission::getDatasourceId, dsIds).list();
        for (ReportDatasourcePermission obj : list) {
            obj.setId(null);
            obj.setAppId(appId);
            obj.setCreateTime(new Date());
            obj.setCreateUserId(AuthUtil.getUserId());
            reportDatasourcePermissionService.save(obj);
        }
    }

    private void dimensionService(List<Integer> dsIds, String appId) {
        List<Dimension> list = dimensionService.lambdaQuery().in(Dimension::getDatasourceId, dsIds).list();
        for (Dimension obj : list) {
            obj.setId(null);
            obj.setAppId(appId);
            obj.setCreateTime(new Date());
            dimensionService.save(obj);
        }
    }

    private void dataApiService(List<Integer> dsIds, String appId) {
        List<DataApi> list = dataApiService.lambdaQuery().in(DataApi::getDatasourceId, dsIds).list();
        for (DataApi obj : list) {
            obj.setId(null);
            obj.setAppId(appId);
            obj.setCreateTime(new Date());
            obj.setCreateUserId(AuthUtil.getUserId());
            dataApiService.save(obj);
        }
    }

    private void cubeTableFileService(List<Integer> dsIds, String appId) {
        List<CubeTableFile> list = cubeTableFileService.lambdaQuery().in(CubeTableFile::getDatasourceId, dsIds).list();
        for (CubeTableFile obj : list) {
            obj.setId(null);
            obj.setAppId(appId);
            obj.setCreateTime(new Date());
            obj.setCreateUserId(AuthUtil.getUserId());
            cubeTableFileService.save(obj);
        }
    }

    private void cubeSqlViewService(List<Integer> dsIds, String appId) {
        List<CubeSqlView> list = cubeSqlViewService.lambdaQuery().in(CubeSqlView::getDatasourceId, dsIds).list();
        for (CubeSqlView obj : list) {
            obj.setId(null);
            obj.setAppId(appId);
            obj.setCreateTime(new Date());
            obj.setCreateUserId(AuthUtil.getUserId());
            cubeSqlViewService.save(obj);
        }
    }

    private boolean reportDatasourceService(List<Integer> dsIds, String appId, PlatomixAppCopyLogModel platomixAppCopyLogModel) {
        List<ReportDatasource> list = reportDatasourceService.lambdaQuery().in(ReportDatasource::getDsId, dsIds).list();
        List<ReportDatasource> copy = null;
        try {
            String jsonStr = JSONUtil.toJsonStr(list);
            System.out.println("数据库复制参数"+jsonStr);
            copy = dsHandler.copy(list);
            platomixAppCopyLogModel.setDbResult(0);
            appClient.taskInfo(platomixAppCopyLogModel);
        } catch (Exception e) {
            platomixAppCopyLogModel.setDbResult(1);
            platomixAppCopyLogModel.setErrorInfo(ExceptionUtil.stacktraceToString(e));
            appClient.taskInfo(platomixAppCopyLogModel);
            log.error("数据库 复制失败:"+ExceptionUtil.stacktraceToString(e));
            return false;
        }
        for (ReportDatasource obj : copy) {
            obj.setId(null);
            obj.setAppId(appId);
            obj.setPoolName(SnowFlowIntUtil.nextId() + "");
            obj.setCreateTime(new Date());
            obj.setCreateUserId(AuthUtil.getUserId());
            reportDatasourceService.save(obj);
        }
        return true;
    }

    private void copyCube(List<Integer> ids, String appId) {
        List<CubeData> list = cubeDataService.lambdaQuery()
                .in(CubeData::getCubeId, ids)
                .eq(CubeData::getAppId, appId)
                .list();
        List<Integer> collect = list.stream().map(CubeData::getCubeId).collect(Collectors.toList());
        if (!list.isEmpty()) {
            List<Integer> newIds = new ArrayList<>();
            //newIds存放 ids 中不包含list.getId()的id
            for (Integer id : ids) {
                if (!collect.contains(id)) {
                    newIds.add(id);
                }
            }
            ids = newIds;
        }
        if (ids.isEmpty()) {
            return;
        }
        cubeDataService(ids, appId);
        cubeDataPermissionService(ids, appId);
        cubeTableColumnService(ids, appId);
        cubeTableStaticService(ids, appId);
    }

    private void cubeTableStaticService(List<Integer> ids, String appId) {
        List<CubeTableStatic> list = cubeTableStaticService.lambdaQuery().in(CubeTableStatic::getCubeDataId, ids).list();
        for (CubeTableStatic obj : list) {
            obj.setId(null);
            obj.setAppId(appId);
            obj.setCreateTime(new Date());
            obj.setCreateUserId(AuthUtil.getUserId());
            cubeTableStaticService.save(obj);
        }
    }

    private void cubeTableColumnService(List<Integer> ids, String appId) {
        List<CubeTableColumn> list = cubeTableColumnService.lambdaQuery().in(CubeTableColumn::getCubeDataId, ids).list();
        for (CubeTableColumn obj : list) {
            obj.setId(null);
            obj.setAppId(appId);
            obj.setCubeTableColumnId(SnowFlowIntUtil.nextId());
            cubeTableColumnService.save(obj);
        }
    }

    private void cubeDataPermissionService(List<Integer> ids, String appId) {
        List<CubeDataPermission> list = cubeDataPermissionService.lambdaQuery().in(CubeDataPermission::getCubeDataId, ids).list();
        for (CubeDataPermission obj : list) {
            obj.setId(null);
            obj.setAppId(appId);
            obj.setCreateTime(new Date());
            obj.setCreateUserId(AuthUtil.getUserId());
            cubeDataPermissionService.save(obj);
        }
    }

    private void cubeDataService(List<Integer> ids, String appId) {
        List<CubeData> list = cubeDataService.lambdaQuery().in(CubeData::getCubeId, ids).list();
        for (CubeData obj : list) {
            obj.setId(null);
            obj.setAppId(appId);
            obj.setCreateTime(new Date());
            obj.setCreateUserId(AuthUtil.getUserId());
            cubeDataService.save(obj);
        }
    }
}
