/**
 *
 */
package org.springblade.resouces;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.google.common.collect.Sets;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import cn.hutool.core.util.StrUtil;
import org.apache.commons.compress.utils.Lists;
import org.springblade.bi.client.CubeTableColumnClient;
import org.springblade.bi.client.form.ReportViewFrom;
import org.springblade.bi.client.model.CubeTableColumnModel;
import org.springblade.bi.client.model.TableColumnModel;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.exception.RpcException;
import org.springblade.cube.dto.CubeTableName;
import org.springblade.cube.dto.FieldCompare;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.model.CubeTableColumn;
import org.springblade.cube.model.CubeTableStatic;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.service.CubeTableColumnService;
import org.springblade.cube.service.CubeTableStaticService;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.model.TableColumnDto;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RestController;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import cn.hutool.core.collection.CollectionUtil;

import javax.annotation.Resource;

/**
 * @author wookvn
 */
@RestController
public class CubeTableColumnResource implements CubeTableColumnClient {

    @Resource
    private CubeTableColumnService cubeTableColumnService;
    @Resource
    private IReportDatasourceService reportDatasourceService;
    @Resource
    private DialectContext dialectContext;
    @Resource
    private CubeTableStaticService cubeTableStaticService;
    @Autowired
    private CubeDataService cubeDataService;

    @Override
    public CubeTableColumnModel getById(Integer id) {
        CubeTableColumn tableColumn = cubeTableColumnService.getById(id);
        CubeTableColumnModel tableColumnModel = BeanUtil.copyProperties(tableColumn, CubeTableColumnModel.class, "viewDataType");
        tableColumnModel.setViewDataType(tableColumn.getViewDataType().getValue());
        return tableColumnModel;
    }

    @Override
    public List<CubeTableColumnModel> listByCubeId(Integer cubeId) {
        List<CubeTableColumn> tableColumnList = cubeTableColumnService.listByCubeId(cubeId);
        if (CollectionUtil.isNotEmpty(tableColumnList)) {
            Map<Integer, CubeTableColumn> dataMap = tableColumnList.stream().collect(Collectors.toMap(CubeTableColumn::getId, t -> t));
            List<CubeTableColumnModel> tableColumnModelList = BeanUtil.copyToList(tableColumnList, CubeTableColumnModel.class,
                    CopyOptions.create().setIgnoreProperties("viewDataType"));
            if (tableColumnModelList != null) {
                tableColumnModelList.stream().forEach(item -> item.setViewDataType(dataMap.get(item.getId()).getViewDataType().getValue()));
            }
            return tableColumnModelList;
        }
        return new ArrayList<>();
    }

    @Override
    public List<Map<String,Object>> refresh(Integer cubeId) {

        ArrayList<FieldCompare> fieldCompares = Lists.newArrayList();

        CubeData cubeData = cubeDataService.getByCubeId(cubeId);
        cubeData.setTableNames(JSONUtil.toList(cubeData.getCubeTableNames(),CubeTableName.class));
        List<CubeTableColumn> cubeTableColumnList = cubeTableColumnService.listByCubeId(cubeId);
        //把tableColumnList转换成CubeTableColumn.columnName为 key, CubeTableColumn本身为 value 的 map
        for (CubeTableName tableName : cubeData.getTableNames()) {
            Map<String, CubeTableColumn> cubeTableColumnMap = cubeTableColumnList.stream().filter(v -> v.getTableName().equalsIgnoreCase(tableName.getTableName())).collect(Collectors.toMap(CubeTableColumn::getColumnName, column -> column));
            FieldCompare fieldCompare = fieldCompare(tableName, cubeData.getDatasourceId(), cubeTableColumnMap);
            fieldCompares.add(fieldCompare);
        }
        //处理cubeTableColumnList,修改数据集 @aokunsang
        List<Integer> deleteCubeColumnIdList = fieldCompares.stream().flatMap(item -> Optional.ofNullable(item.getFieldsDeleted()).orElse(new HashSet<>()).stream()).map(CubeTableColumn::getId).collect(Collectors.toList());
        List<CubeTableColumn> cteCubeColumnList = fieldCompares.stream().flatMap(item -> Optional.ofNullable(item.getFieldsCreat()).orElse(new HashSet<>()).stream()).map(item -> cubeTableColumnService.physicsTableColumnToCubeTableColumn(null, cubeId, item, item.getTableName(), item.getTableAlias(), 999)).collect(Collectors.toList());
        List<CubeTableColumn> uptCubeColumnList = fieldCompares.stream().flatMap(item -> Optional.ofNullable(item.getFieldsUpdate()).orElse(new HashSet<>()).stream()).map(item -> cubeTableColumnService.physicsTableColumnToCubeTableColumn(item, cubeId, 999)).flatMap(item -> item.stream()).collect(Collectors.toList());
        this.doHandle(deleteCubeColumnIdList, cteCubeColumnList, uptCubeColumnList);

        return fieldCompares.stream().map(BeanUtil::beanToMap).collect(Collectors.toList());
    }

    @Transactional
    public void doHandle(List<Integer> deleteCubeColumnIdList, List<CubeTableColumn> cteCubeColumnList, List<CubeTableColumn> uptCubeColumnList) {
        cteCubeColumnList.addAll(uptCubeColumnList);
        cubeTableColumnService.removeByIds(deleteCubeColumnIdList);
        cubeTableColumnService.saveOrUpdateBatch(cteCubeColumnList);
    }

    private FieldCompare fieldCompare(CubeTableName cubeTableName, Integer datasourceId, Map<String, CubeTableColumn> cubeTableColumnMap) {
        FieldCompare fieldCompare = new FieldCompare();
        fieldCompare.setFieldsDeleted(Sets.newHashSet());
        fieldCompare.setFieldsCreat(Sets.newHashSet());
        fieldCompare.setFieldsUpdate(Sets.newHashSet());


        ReportDatasource datasource = reportDatasourceService.getByDsId(datasourceId);
        List<TableColumnDto> tableColumnModels = dialectContext.getDialect(datasource.getDsTypeSyl()).listTableColumnsByTableViewName(datasource, cubeTableName.getTableName());

        Map<String, TableColumnDto> tableColumnModelMap = tableColumnModels.stream().collect(Collectors.toMap(TableColumnDto::getColumnName, column -> column));

        //遍历数据集集合,对比数据集字段和数据源字段
        for (Map.Entry<String, CubeTableColumn> entry : cubeTableColumnMap.entrySet()) {
            TableColumnDto tableColumnDto = tableColumnModelMap.get(entry.getKey());
            if (tableColumnDto == null) {
                fieldCompare.getFieldsDeleted().add(entry.getValue());
                continue;
            }
            if (!tableColumnDto.getColumnType().equals(entry.getValue().getViewDataType())) {
                tableColumnDto.setTableName(cubeTableName.getTableName());
                tableColumnDto.setTableAlias(cubeTableName.getTableAliasName());
                fieldCompare.getFieldsUpdate().add(Collections.singletonMap(entry.getValue().getId(), tableColumnDto));
            }

        }
        //遍历数据源集合,对比数据集字段和数据源字段
        for (Map.Entry<String, TableColumnDto> entry : tableColumnModelMap.entrySet()) {
            CubeTableColumn cubeTableColumn = cubeTableColumnMap.get(entry.getKey());
            entry.getValue().setTableName(cubeTableName.getTableName());
            entry.getValue().setTableAlias(cubeTableName.getTableAliasName());
            if (cubeTableColumn == null) {
                fieldCompare.getFieldsCreat().add(entry.getValue());
                continue;
            }
            if (!cubeTableColumn.getViewDataType().equals(entry.getValue().getColumnType())) {
                fieldCompare.getFieldsUpdate().add(Collections.singletonMap(cubeTableColumn.getId(), entry.getValue()));
            }
        }

        return fieldCompare;

    }

    @Override
    public List<TableColumnModel> listColumnsByDatasource(ReportViewFrom reportViewFrom) {
        String viewName = reportViewFrom.getTableViewName();
        if (StrUtil.isAllBlank(viewName)) {
            throw new RpcException("视图名称不可以为空");
        }
        ReportDatasource datasource = reportDatasourceService.getByDsId(reportViewFrom.getDatasourceId());
        if (null == datasource) {
            throw new RpcException("数据源id错误");
        }
        List<TableColumnDto> list = dialectContext.getDialect(datasource.getDsTypeSyl()).listTableColumnsByTableViewName(datasource, viewName);
        return list.stream().map(v -> BeanUtil.copyProperties(v, TableColumnModel.class)).collect(Collectors.toList());
    }

}
