/**
 * 
 */
package org.springblade.subscribe.controller;

import java.util.Date;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.subscribe.entity.VisualSubscribePushLog;
import org.springblade.subscribe.service.VisualSubscribePushLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;

import cn.hutool.core.collection.CollectionUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * @author wookvn
 *
 */
@RestController
@RequestMapping("/subscribeLog")
@Api(value = "订阅日志接口", tags = "订阅日志接口")
public class VisualSubscribePushLogController extends BladeController {

	@Autowired
	private VisualSubscribePushLogService visualSubscribePushLogService;

	/**
	 * 
	 * @return
	 */
	@GetMapping("/list")
	@ApiOperation(value = "订阅日志列表")
	public R<IPage<VisualSubscribePushLog>> list(VisualSubscribePushLog pushLog,
			@DateTimeFormat(pattern = "yyyy-MM-dd") Date startTime, @DateTimeFormat(pattern = "yyyy-MM-dd") Date endTime, Query query){
		QueryWrapper<VisualSubscribePushLog> queryWrapper = Condition.getQueryWrapper(pushLog);
		queryWrapper.lambda().ge(startTime != null, VisualSubscribePushLog::getCreateTime, startTime)
					.lt(endTime != null, VisualSubscribePushLog::getCreateTime, endTime)
					.orderByDesc(VisualSubscribePushLog::getId);
		IPage<VisualSubscribePushLog> page = visualSubscribePushLogService.page(Condition.getPage(query), queryWrapper);
		if(page != null && CollectionUtil.isNotEmpty(page.getRecords())) {
			page.getRecords().stream().forEach(it -> it.setContent(null));
		}
		return R.data(page);
	}
	
	/**
	 * 
	 * @param id
	 * @return
	 */
	@GetMapping("/info/{id}")
	@ApiOperation(value = "订阅日志详情")
	public R<VisualSubscribePushLog> info(@PathVariable Integer id){
		return R.data(visualSubscribePushLogService.getById(id));
	}
	
	/**
	 * 
	 * @return
	 */
	@GetMapping("/statInfor")
	@ApiOperation(value = "订阅统计信息", notes = "userCnt:订阅人数；dataCnt:订阅内容；sucCnt:成功数量；errCnt:失败数量")
	public R<JSONObject> statInfor(){
		JSONObject result = new JSONObject();
		JSONObject statSubInfor = visualSubscribePushLogService.statSubInfor();
		JSONObject statTodayLogInfor = visualSubscribePushLogService.statTodayLogInfor();
		result.putAll(statSubInfor);
		result.putAll(statTodayLogInfor);
		return R.data(result);
	}
}
