package org.springblade.usergroup.controller;

import java.util.ArrayList;
import java.util.Base64;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.StringUtil;
import org.springblade.cube.dto.PreviewResult;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.model.FileStructure;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.usergroup.dto.ColumnDto;
import org.springblade.usergroup.dto.DefineRuleDto;
import org.springblade.usergroup.entity.DefineGroup;
import org.springblade.usergroup.enums.DbClickHouseEnums;
import org.springblade.usergroup.enums.DbCreateCKTableEnum;
import org.springblade.usergroup.service.DefineGroupService;
import org.springblade.usergroup.service.ExecuteClickService;
import org.springblade.visual.dto.ComponentColumn;
import org.springblade.visual.dto.QuerySqlParam;
import org.springblade.visual.service.DefineGroupSqlService;
import org.springblade.visual.utils.Tuple2;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.xxl.job.core.biz.AdminBiz;
import com.xxl.job.core.biz.client.AdminBizClient;
import com.xxl.job.core.biz.model.JobInfo;
import com.xxl.job.core.biz.model.ReturnT;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import io.github.resilience4j.core.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@RestController
@RequestMapping("/group/defineGroup")
@Api(value = "自定义人群管理", tags = "自定义人群管理")
public class DefineGroupController extends BladeController {

	@Autowired
	private CubeDataService cubeDataService;

	@Autowired
	private DefineGroupService defineGroupService;

	@Autowired
	private DefineGroupSqlService querySqlService;

	@Autowired
	private ExecuteClickService executeClickService;

	@Autowired
	private DynamicTableService dynamicTableService;

	@Value("${xxl.job.admin.addresses}")
	private String addressUrl;

	@Value("${xxl.job.accessToken}")
	private String accessToken;
	
	@Value("${xxl.job.executor.groupid}")
    private int groupId;



	/**
	 * 列表
	 */
	@GetMapping("/list")
	@ApiOperation(value = "自定义人群列表")
	@ApiOperationSupport(order = 1)
	public R<IPage<DefineGroup>> list(DefineGroup defineGroup, Query query){
		QueryWrapper queryWrapper = Condition.getQueryWrapper(defineGroup);
		if (StrUtil.isNotBlank(defineGroup.getGroupName())) {
			queryWrapper.like("group_name", "%" + defineGroup.getGroupName() + "%");
		}
		if (defineGroup.getClassifyId() != null) {
			queryWrapper.eq("classify_id", defineGroup.getClassifyId());
		}
		if (defineGroup.getStatus() !=null){
			queryWrapper.eq("status", defineGroup.getStatus());
		}
		queryWrapper.orderByDesc("id");
		return R.data(defineGroupService.page(Condition.getPage(query), queryWrapper));
	}



	/**
	 * 单条数据查询接口
	 * @param id
	 * @return
	 */
	@GetMapping("/detail")
	@ApiOperation(value = "根据ID获取人群信息", httpMethod = "GET")
	@ApiOperationSupport(order = 2)
	public R<DefineGroup> info(@RequestParam Integer id) {
		return R.data(defineGroupService.getById(id));
	}




	/**
	 * 	删除
	 * @param ids
	 * @return
	 */
	@PostMapping("/remove")
	@ApiOperation(value = "根据ID数组删除自定义人群(多个逗号分割)", httpMethod = "POST")
	@ApiOperationSupport(order = 3)
	public R<String> doRemove(@ApiParam(value = "主键", required = true) @RequestParam String ids) {
		//删除对应的定时任务
		List<Integer> groupIds = Func.toIntList(ids);
		AdminBiz adminBiz = new AdminBizClient(addressUrl, accessToken);
		for (Integer id : groupIds){
			DefineGroup defineGroup = defineGroupService.getById(id);
			adminBiz.jobRemove(defineGroup.getJobId());
		}
		return R.status(defineGroupService.removeByIds(groupIds));
	}


	/**
	 *	新增修改数据接口
	 * @param value
	 * @return
	 * @throws Exception
	 */
	@PostMapping("/submit")
	@ApiOperation(value = "增加或更新自定义人群", httpMethod = "POST")
	@ApiOperationSupport(order = 4)
	public R doSubmit(@RequestBody String value) throws Exception {
		String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
		DefineGroup defineGroup = JSON.parseObject(decoderStr, DefineGroup.class);
		//保存数据集过滤器解析后的结果
		List<QuerySqlParam> sqlList= new ArrayList<>();
		Boolean flag=true;
		try {
			if(defineGroup != null ) {
				if (defineGroup.getId() == null){
					defineGroup.setCreateBy(SecureUtil.getNickName());
					defineGroup.setCreateTime(new Date());
					defineGroup.setCreateUserId(SecureUtil.getUserId());
					//未开始
					defineGroup.setStatus(0);
					//保存defineGroup
					defineGroupService.saveOrUpdate(defineGroup);
					//解析sql
					List<DefineRuleDto> ruleDtoList = JSON.parseArray(defineGroup.getDefineRule(), DefineRuleDto.class);
					if (ruleDtoList != null && ruleDtoList.size() > 0) {
						for (int i = 0; i < ruleDtoList.size(); i++) {
							//解析sql
							R<QuerySqlParam> r = querySqlService.parseDefineGroupQuerySql(ruleDtoList.get(i).getCubeId(), ruleDtoList.get(i), getUser());
							if (!r.isSuccess()){
								flag=false;
								break;
							}
							sqlList.add(r.getData());
						}
					}
					if (flag){
						String	tableName="defineGroup_"+defineGroup.getId()+"_"+ DateUtil.format(new Date(), "yyyyMMddHHmm");
						if (sqlList.size()>0){
//							StringBuilder sb=new StringBuilder();
//							sb.append("[");
//							for (int i = 0; i < sqlList.size(); i++) {
//								if (i == sqlList.size()-1){
//									sb.append(sqlList.get(i).getCubeData().getId()+"]");
//								}else {
//									sb.append(sqlList.get(i).getCubeData().getId()+",");
//								}
//							}
							List<Integer> cubeIds = new ArrayList<>();
							for (int i = 0; i < sqlList.size(); i++) {
								cubeIds.add(sqlList.get(i).getCubeData().getId());
							}
							defineGroup.setCubeIds(cubeIds);
						}
						//创建clickhouse表
						Tuple2<String, String> tuple = createClickTable(defineGroup, tableName, sqlList);
						defineGroup.setCubeColumns(tuple.getT1());
						defineGroup.setResultColumns(tuple.getT2());

						defineGroup.setResultTable(tableName);

						//创建xxl_job任务
						String jobId = saveJob(defineGroup,0);
						if (StringUtils.isNotEmpty(jobId)){
							defineGroup.setJobId(Integer.valueOf(jobId));
						}
						defineGroupService.updateById(defineGroup);
						return R.data(defineGroup.getId());
					}else {
						//删除mysql记录
						defineGroupService.removeById(defineGroup.getId());
						return R.fail("查询SQL解析失败");
					}
				}else {
					DefineGroup group = defineGroupService.getById(defineGroup.getId());
					if (group.getRequestFreq() != defineGroup.getRequestFreq()){
						saveJob(defineGroup,group.getJobId());
					}
					if (!group.getDefineRule().equals(defineGroup.getDefineRule())){
						//解析sql
						List<DefineRuleDto> ruleDtoList = JSON.parseArray(defineGroup.getDefineRule(), DefineRuleDto.class);
						if (ruleDtoList != null && ruleDtoList.size() > 0) {
							for (int i = 0; i < ruleDtoList.size(); i++) {
								//解析sql
								R<QuerySqlParam> r = querySqlService.parseDefineGroupQuerySql(ruleDtoList.get(i).getCubeId(), ruleDtoList.get(i), getUser());
								if (!r.isSuccess()){
									break;
								}
								sqlList.add(r.getData());
							}
						}
						if (sqlList.size()>0){
							List<Integer> cubeIds = new ArrayList<>();
							for (int i = 0; i < sqlList.size(); i++) {
								cubeIds.add(sqlList.get(i).getCubeData().getId());
							}
							defineGroup.setCubeIds(cubeIds);
						}
						Tuple2<String, String> tuple = updateClickTable(group, sqlList);
						defineGroup.setCubeColumns(tuple.getT1());
						defineGroup.setResultColumns(tuple.getT2());
					}
					defineGroup.setUpdateTime(new Date());
					defineGroupService.updateById(defineGroup);
					return R.success("编辑成功");
				}
			}else {
				return R.fail("参数解码失败");
			}


		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return R.fail("保存失败");
		}
	}

	/**
	 * 	预览
	 * @return
	 */
	@PostMapping("/preview")
	@ApiOperation(value = "数据预览", notes = "传入DefineRuleDto")
	public R<PreviewResult> preview(@RequestBody String value) throws Exception{
		String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
		DefineRuleDto defineRuleDto = JSON.parseObject(decoderStr, DefineRuleDto.class);
		if(StrUtil.isEmpty(defineRuleDto.getPhoneField())) {
			return R.fail("主键字段不能为空");
		}

		R<QuerySqlParam> r = querySqlService.parseDefineGroupQuerySql(defineRuleDto.getCubeId(), defineRuleDto, getUser());
		if(!r.isSuccess()) {
			return R.fail(r.getMsg());
		}
		QuerySqlParam querySqlParam = r.getData();
		ReportDatasource reportDatasource = querySqlParam.getReportDatasource();
		String querySql = querySqlParam.getQuerySql();
		Map<String, String> fieldDict = querySqlParam.getSelectColumnList().stream().collect(Collectors.toMap(ComponentColumn::getColumnAliasName, ComponentColumn::getColumnTitle));
		List<Map<String, Object>> resultList = dynamicTableService.selectPageList(new Page<>(0, 10).setSearchCount(false),reportDatasource, querySql);
//		if (warningDto.getLimitVal() != null && warningDto.getLimitVal() > 0) { // 分页查询
//			resultList = dynamicTableService.selectPage(new Page<>(0, warningDto.getLimitVal()).setSearchCount(false),
//					reportDatasource, querySql);
//		} else {
//			resultList = dynamicTableService.selectList(reportDatasource, querySql);
//		}
		return R.data(new PreviewResult(resultList, fieldDict, null), querySql);
	}

	/**
	 * 查询结果表结构
	 * @param id
	 * @return
	 */
	@GetMapping("/getTableColumn/{id}")
	@ApiOperation(value = "根据ID查询结果表结构", httpMethod = "GET")
	@ApiOperationSupport(order = 5)
	public R getTableColumn(@PathVariable Integer id) {
		DefineGroup defineGroup = defineGroupService.getById(id);
		if (defineGroup != null){
			String resultTable = defineGroup.getResultTable();
			String columnSql="select name,type,comment from system.columns where table="+"'"+resultTable+"'";
			return R.data(executeClickService.selectList(columnSql));
		}

		return R.fail("查询表字段失败！");
	}





	//创建click表
	public Tuple2<String, String> createClickTable(DefineGroup defineGroup, String tableName, List<QuerySqlParam> sqlParamList){
		//总结果字段
		List<FileStructure> allStructureList = new ArrayList<>();
		List<String> columnString = new ArrayList<>();
		List<ComponentColumn> componentColumns = new ArrayList<>();
//		Map<Integer,List<ComponentColumn>>columnMap = new HashMap<>();
		List<ColumnDto> columnDtos =new ArrayList<>();
		if (sqlParamList != null && sqlParamList.size()>0){
			for (int i = 0; i < sqlParamList.size(); i++) {
				CubeData cubeData = sqlParamList.get(i).getCubeData();
				ColumnDto columnDto =new ColumnDto();
				columnDto.setCubeId(cubeData.getId());
				List<ComponentColumn> columnList =sqlParamList.get(i).getSelectColumnList();
				List<ComponentColumn> columns =new ArrayList<>();
				if (columnList !=null  && columnList.size()>0) {
					//1.构建表字段
					List<FileStructure> structureList = new ArrayList<>();
					for (int j = 0; j < columnList.size(); j++) {
						ComponentColumn column = new ComponentColumn();
						//columnName为空时，判断columnAliasName不为空，将columnAliasName赋给columnName
						if (StringUtil.isBlank(columnList.get(j).getColumnName()) && StringUtil.isNotBlank(columnList.get(j).getColumnAliasName())){
							columnList.get(j).setColumnName(columnList.get(j).getColumnAliasName());
						}
						BeanUtils.copyProperties(columnList.get(j),column);
						FileStructure fileStructure = new FileStructure();
						fileStructure.setColumnType(columnList.get(j).getViewDataType());
						fileStructure.setComment(columnList.get(j).getColumnTitle());
						fileStructure.setColumnIndex(j);
						//判断字段是否重复
						if (columnString.contains(columnList.get(j).getColumnName())){
							fileStructure.setColumnName(columnList.get(j).getColumnName()+"_"+cubeData.getId()+"_"+j);
							structureList.add(fileStructure);

//							FileStructure fileStructure1 = new FileStructure();
//							BeanUtils.copyProperties(fileStructure,fileStructure1);
//							fileStructure1.setColumnName(columnList.get(j).getColumnName()+"_"+i);
							column.setUserGroupColumnAliasName(fileStructure.getColumnName());
//							columnString.add(fileStructure1.getColumnName());
							if ( (i>0 && j!=0) || (i==0)){
								allStructureList.add(fileStructure);
								componentColumns.add(column);
							}
						}else {
							fileStructure.setColumnName(columnList.get(j).getColumnName());
							structureList.add(fileStructure);

							column.setUserGroupColumnAliasName(column.getColumnName());
							columnString.add(fileStructure.getColumnName());
							if ( (i>0 && j!=0) || (i==0)){
								allStructureList.add(fileStructure);
								componentColumns.add(column);
							}
						}

						columns.add(column);
					}
					int length = structureList.size();
					//添加一列int类型作为时间分区字段
					structureList.add(new FileStructure("the_value", ViewColumnTypeEnum.NUMBER, "分区字段", length));
					//2.构造建表语句
					List<String> sqlList = new ArrayList<>();
//					if (sqlParamList.size() == 1){
//						sqlList = DbCreateCKTableEnum.Clickhouse.createTableSql(tableName, "自定义人群结果集", structureList,"the_value",structureList.get(0).getColumnName());
//					}else {
						sqlList = DbCreateCKTableEnum.Clickhouse.createTableSql("defineGroup_"+defineGroup.getId()+"_"+cubeData.getId(), "自定义人群: "+cubeData.getCubeName(), structureList,"the_value",structureList.get(0).getColumnName());

//					}
					executeClickService.executeSql(sqlList.stream().toArray(String[]::new));
				}

				columnDto.setColumns(columns);
				columnDtos.add(columnDto);
			}
			//创建总结果集
			if (allStructureList.size()>0){
				//系统默认添加一列[时间分区]
				allStructureList.add(new FileStructure("the_value", ViewColumnTypeEnum.NUMBER, "分区字段", allStructureList.size()));
				//2.构造建表语句
				List<String> sqlList = DbCreateCKTableEnum.Clickhouse.createTableSql(tableName, "自定义人群结果集", allStructureList,"the_value",allStructureList.get(0).getColumnName());
				executeClickService.executeSql(sqlList.stream().toArray(String[]::new));
			}

		}
	return new Tuple2<String, String>(JSONObject.toJSONString(columnDtos), JSONObject.toJSONString(componentColumns));
	}

	//修改click表
	public  Tuple2<String, String> updateClickTable(DefineGroup group,List<QuerySqlParam> sqlParamList){
		List<ColumnDto> oldColumnDtoList = JSON.parseArray(group.getCubeColumns(), ColumnDto.class);
		List<ComponentColumn> oldComponentColumns = JSON.parseArray(group.getResultColumns(), ComponentColumn.class);
		List<ComponentColumn> newComponentColumns =new ArrayList<>();
		List<String> columnString = new ArrayList<>();
		List<ColumnDto> newColumnDtoList =new ArrayList<>();
		if (sqlParamList != null && sqlParamList.size()>0){
			for (int i = 0; i < sqlParamList.size(); i++) {
				CubeData cubeData = sqlParamList.get(i).getCubeData();
				ColumnDto columnDto =new ColumnDto();
				columnDto.setCubeId(cubeData.getId());
				List<ComponentColumn> columnList =sqlParamList.get(i).getSelectColumnList();
				List<ComponentColumn> columns =new ArrayList<>();
				if (columnList !=null  && columnList.size()>0) {
					//1.构建表字段
//					List<FileStructure> structureList = new ArrayList<>();
					for (int j = 0; j < columnList.size(); j++) {
						ComponentColumn column = new ComponentColumn();
						//columnName为空时，判断columnAliasName不为空，将columnAliasName赋给columnName
						if (StringUtil.isBlank(columnList.get(j).getColumnName()) && StringUtil.isNotBlank(columnList.get(j).getColumnAliasName())){
							columnList.get(j).setColumnName(columnList.get(j).getColumnAliasName());
						}
						BeanUtils.copyProperties(columnList.get(j),column);
						FileStructure fileStructure = new FileStructure();

						fileStructure.setColumnType(columnList.get(j).getViewDataType());
						fileStructure.setComment(columnList.get(j).getColumnComment());
						fileStructure.setColumnIndex(j);
						//判断字段是否重复
						if (columnString.contains(columnList.get(j).getColumnName())){

							fileStructure.setColumnName(columnList.get(j).getColumnName()+"_"+cubeData.getId()+"_"+j);

//							FileStructure fileStructure1 = new FileStructure();
//							BeanUtils.copyProperties(fileStructure,fileStructure1);
//							fileStructure1.setColumnName(columnList.get(j).getColumnName()+"_"+i);
							column.setUserGroupColumnAliasName(fileStructure.getColumnName());
							columnString.add(fileStructure.getColumnName());
							if ( (i>0 && j!=0) || (i==0)){
								newComponentColumns.add(column);
							}
						}else {

							fileStructure.setColumnName(columnList.get(j).getColumnName());
							column.setUserGroupColumnAliasName(column.getColumnName());
							columnString.add(fileStructure.getColumnName());
							if ( (i>0 && j!=0) || (i==0)){
								newComponentColumns.add(column);;
							}
						}
						columns.add(column);
					}
				}
				columnDto.setColumns(columns);
				newColumnDtoList.add(columnDto);
			}
		}

		if (newColumnDtoList.size() >0 ){
			Map<Integer, List<ComponentColumn>> newMap = newColumnDtoList.stream().collect(Collectors.toMap(ColumnDto::getCubeId,ColumnDto::getColumns));
			Map<Integer, List<ComponentColumn>> oldMap = oldColumnDtoList.stream().collect(Collectors.toMap(ColumnDto::getCubeId,ColumnDto::getColumns));
			//对比oldMap与newMap
			for(Map.Entry<Integer, List<ComponentColumn>> entry : newMap.entrySet()) {
				Integer cubeId = entry.getKey();
				if (!oldMap.containsKey(cubeId)){//新的数据集
					CubeData cubeData = cubeDataService.getByCubeId(cubeId);
					List<FileStructure> structureList = new ArrayList<>();
					List<ComponentColumn> columnList = entry.getValue();
					for (int i=0;i< columnList.size();i++){
						FileStructure fileStructure = new FileStructure();
						fileStructure.setColumnName(columnList.get(i).getColumnName());
						fileStructure.setColumnType(columnList.get(i).getViewDataType());
						fileStructure.setComment(columnList.get(i).getColumnComment());
						fileStructure.setColumnIndex(i);
						structureList.add(fileStructure);
					}
					structureList.add(new FileStructure("the_value", ViewColumnTypeEnum.NUMBER, "分区字段", structureList.size()));
					//构造建表语句
					List<String> sqlList =  DbCreateCKTableEnum.Clickhouse.createTableSql("defineGroup_"+group.getId()+"_"+cubeId, "自定义人群: "+cubeData.getCubeName(), structureList,"the_value",structureList.get(0).getColumnName());
					executeClickService.executeSql(sqlList.stream().toArray(String[]::new));
				}else {
//					List<ComponentColumn> newColumnList = entry.getValue();
//					List<ComponentColumn> oldColumnList = oldMap.get(cubeId);
					Map<String, ComponentColumn> newColumnMap =  entry.getValue().stream().collect(Collectors.toMap(ComponentColumn::getUserGroupColumnAliasName, t->t,(t1,t2)->t1));
					Map<String, ComponentColumn> oldColumnMap =  oldMap.get(cubeId).stream().collect(Collectors.toMap(ComponentColumn::getUserGroupColumnAliasName, t->t,(t1,t2)->t1));
					StringBuilder addSqlBuilder = new StringBuilder();
					StringBuilder delSqlBuilder = new StringBuilder();
					addSqlBuilder.append(String.format("alter table %s ","defineGroup_"+group.getId()+"_"+cubeId));
					delSqlBuilder.append(String.format("alter table %s ","defineGroup_"+group.getId()+"_"+cubeId));
					boolean addFlag =false;
					boolean delFlag =false;
					for(Map.Entry<String, ComponentColumn> newColumnEntry : newColumnMap.entrySet()) {
						if (!oldColumnMap.containsKey(newColumnEntry.getKey())){
							//新增列
							addFlag=true;
							ComponentColumn column = newColumnEntry.getValue();
							addSqlBuilder.append("add column ").append(String.format("%s %s COMMENT '%s',",column.getUserGroupColumnAliasName(), DbClickHouseEnums.ColumnType.valueOf(column.getViewDataType().name()).columnType,column.getColumnTitle()));
						}

					}
					for(Map.Entry<String, ComponentColumn> oldColumnEntry : oldColumnMap.entrySet()) {
						if (!newColumnMap.containsKey(oldColumnEntry.getKey())){
							//删除列
							delFlag =true;
							ComponentColumn column = oldColumnEntry.getValue();
							delSqlBuilder.append("drop column ").append(String.format("%s ,",column.getUserGroupColumnAliasName()));
						}else {
							//如果列的别名发生变化，先删除，再添加
							ComponentColumn oldColumn = oldColumnEntry.getValue();
							ComponentColumn newColumn = newColumnMap.get(oldColumnEntry.getKey());
							if (!oldColumn.getColumnSql().equals(newColumn.getColumnSql())){
								addFlag=true;
								delFlag =true;
								addSqlBuilder.append("add column ").append(String.format("%s %s COMMENT '%s',",newColumn.getUserGroupColumnAliasName(), DbClickHouseEnums.ColumnType.valueOf(newColumn.getViewDataType().name()).columnType,newColumn.getColumnTitle()));
								delSqlBuilder.append("drop column ").append(String.format("%s ,",oldColumn.getUserGroupColumnAliasName()));
							}
						}
					}

					if (delFlag){//执行删除列sql
						delSqlBuilder.deleteCharAt(delSqlBuilder.length() - 1);
						log.info("delSqlBuilder:"+delSqlBuilder);
						executeClickService.executeSql(delSqlBuilder.toString());
					}
					if (addFlag){//执行添加列sql
						addSqlBuilder.deleteCharAt(addSqlBuilder.length() - 1);
						log.info("addSqlBuilder:"+addSqlBuilder);
						executeClickService.executeSql(addSqlBuilder.toString());
					}

				}
			}
		}
		//汇总表的操作
		if (newComponentColumns.size() >0 ){
			Map<Integer, ComponentColumn> newColumnMap = newComponentColumns.stream().collect(Collectors.toMap(ComponentColumn::getCubeColumnId,t->t,(t1,t2)->t1));
			Map<Integer, ComponentColumn> oldColumnMap = oldComponentColumns.stream().collect(Collectors.toMap(ComponentColumn::getCubeColumnId,t->t,(t1,t2)->t1));
			StringBuilder addSqlBuilder = new StringBuilder();
			StringBuilder delSqlBuilder = new StringBuilder();
			addSqlBuilder.append(String.format("alter table %s ",group.getResultTable()));
			delSqlBuilder.append(String.format("alter table %s ",group.getResultTable()));
			boolean addFlag =false;
			boolean delFlag =false;
			for(Map.Entry<Integer, ComponentColumn> newColumnEntry : newColumnMap.entrySet()) {
				if (!oldColumnMap.containsKey(newColumnEntry.getKey())){
					//新增列
					addFlag=true;
					ComponentColumn column = newColumnEntry.getValue();
					addSqlBuilder.append("add column ").append(String.format("%s %s COMMENT '%s',",column.getUserGroupColumnAliasName(), DbClickHouseEnums.ColumnType.valueOf(column.getViewDataType().name()).columnType,column.getColumnTitle()));
				}

			}
			for(Map.Entry<Integer, ComponentColumn> oldColumnEntry : oldColumnMap.entrySet()) {
				if (!newColumnMap.containsKey(oldColumnEntry.getKey())){
					//删除列
					delFlag =true;
					ComponentColumn column = oldColumnEntry.getValue();
					delSqlBuilder.append("drop column ").append(String.format("%s ,",column.getUserGroupColumnAliasName()));
				}else {
					//如果列的别名发生变化，先删除，再添加
					ComponentColumn oldColumn = oldColumnEntry.getValue();
					ComponentColumn newColumn = newColumnMap.get(oldColumnEntry.getKey());
					if (!oldColumn.getUserGroupColumnAliasName().equals(newColumn.getUserGroupColumnAliasName())){
						addFlag=true;
						delFlag =true;
						addSqlBuilder.append("add column ").append(String.format("%s %s COMMENT '%s',",newColumn.getUserGroupColumnAliasName(), DbClickHouseEnums.ColumnType.valueOf(newColumn.getViewDataType().name()).columnType,newColumn.getColumnTitle()));
						delSqlBuilder.append("drop column ").append(String.format("%s ,",oldColumn.getUserGroupColumnAliasName()));
					}

				}
			}

			if (delFlag){//执行删除列sql
				delSqlBuilder.deleteCharAt(delSqlBuilder.length() - 1);
				executeClickService.executeSql(delSqlBuilder.toString());
			}
			if (addFlag){//执行添加列sql
				addSqlBuilder.deleteCharAt(addSqlBuilder.length() - 1);
				executeClickService.executeSql(addSqlBuilder.toString());
			}

		}
		return new Tuple2<String, String>(JSONObject.toJSONString(newColumnDtoList), JSONObject.toJSONString(newComponentColumns));
	}


	//创建job
	public String saveJob(DefineGroup defineGroup,Integer jobId){
		Integer freq = defineGroup.getRequestFreq();
		AdminBiz adminBiz = new AdminBizClient(addressUrl, accessToken);
		//处理cron表达式
		String cron="";
		if (freq == 0){//立即执行
			cron= "0 */10 * * * ?";
		}else if (freq == 1){//每日执行:随机小时随机分钟
//			"0 35 2 * * ?":每天两点35分执行
			int hour = RandomUtil.randomInt(0,24);
			int min = RandomUtil.randomInt(0,60);
			cron="0 "+min+" "+hour+" * * ?";
		}else if (freq == 2){//每周执行:随机天随机小时
//			0 0 21 ? * 4 每周三晚上九点执行(周日是1，周六是7)
			int hour = RandomUtil.randomInt(0,24);
			int day = RandomUtil.randomInt(1,7);
			cron="0 0 "+hour+" ? * "+day;
		}else if (freq == 3){//每月执行:随机小时随机分钟
//			0 15 10 L * ? 每月最后一天10:15分运行。
			int hour = RandomUtil.randomInt(0,24);
			int min = RandomUtil.randomInt(0,60);
			cron="0 "+min+" "+hour+" L * ?";
		}
		if (jobId == 0){//新增定时任务
			JobInfo xxlJobInfo = new JobInfo();
			xxlJobInfo.setJobGroup(groupId);
			xxlJobInfo.setJobCron(cron);
			xxlJobInfo.setJobDesc("自定义人群："+defineGroup.getGroupName());
			xxlJobInfo.setAddTime(new Date());
			xxlJobInfo.setAuthor("1");
			xxlJobInfo.setScheduleType("CRON");
			xxlJobInfo.setScheduleConf(cron);
			xxlJobInfo.setMisfireStrategy("DO_NOTHING");
			xxlJobInfo.setExecutorRouteStrategy("FIRST");
			xxlJobInfo.setExecutorHandler("defineGroupJobHandler");
			//将人群组id与sqlList作为参数
			xxlJobInfo.setExecutorParam(defineGroup.getId()+"");
			xxlJobInfo.setExecutorTimeout(0);
			xxlJobInfo.setExecutorFailRetryCount(0);
			xxlJobInfo.setExecutorBlockStrategy("SERIAL_EXECUTION");
			xxlJobInfo.setGlueUpdatetime(new Date());
			xxlJobInfo.setGlueType("BEAN");
			xxlJobInfo.setTriggerStatus(1);//运行
			xxlJobInfo.setTriggerLastTime(0);
			xxlJobInfo.setTriggerNextTime(0);
			ReturnT<String> returnT = adminBiz.jobAdd(xxlJobInfo);
			if (ReturnT.SUCCESS_CODE == returnT.getCode()){
				return returnT.getContent();
			}
		}else {//修改定时任务
			ReturnT<String> returnT = adminBiz.getJobById(jobId);
			if (ReturnT.SUCCESS_CODE == returnT.getCode()){
				JobInfo jobInfo = JSONObject.parseObject(returnT.getContent(), JobInfo.class);
				jobInfo.setJobCron(cron);
				jobInfo.setScheduleConf(cron);
				jobInfo.setJobDesc("自定义人群："+defineGroup.getGroupName());
				adminBiz.jobUpdate(jobInfo);
			}
		}
		return "";
	}


}
