package org.springblade.usergroup.service;


import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.Timestamp;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springblade.core.tool.utils.Func;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.usergroup.mapper.ExecuteClickMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.druid.util.JdbcUtils;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.DynamicRoutingDataSource;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.metadata.IPage;

import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class ExecuteClickService {

	@Autowired
	private ExecuteClickMapper clickMapper;

//	@Autowired
//	private IReportDatasourceService reportDatasourceService;

	@Autowired
	DataSource dataSource;

	/**
	 * 执行SQL
	 * @param sqls
	 * @throws Exception
	 */
	@DS("clickhouse")
	public void executeSql(String... sqls) {
		for(String sql : sqls) {
			clickMapper.executeSql(sql);
		}
	}

	/**
	 * @param sql
	 * @return
	 */
	@DS("clickhouse")
	public Long selectCount(String sql) {
		return clickMapper.selectCount(sql);
	}
	/**
	 * @param sql
	 * @return
	 */
	@DS("clickhouse")
	public List<JSONObject> selectList(String sql) {
		return clickMapper.selectList(sql);
	}

	/**
	 * @param sql
	 * @return
	 */
	public JSONObject selectOne(String sql) {
		return clickMapper.selectOne(sql);
	}

	/**
	 * @param page
	 * @param sql
	 * @return
	 */
	public List<JSONObject> selectPageList(IPage<?> page, String sql) {
		return clickMapper.selectPageList(page, sql);
	}

	/**
	 * @param page
	 * @param sql
	 * @return
	 */
	public IPage<JSONObject> selectPage(IPage<?> page,String sql) {
		return clickMapper.selectPage(page, sql);
	}


	/**
	 * 批量入库
	 * @throws Exception
	 */
	public void batchSave(String insertSql, Map<Integer, ViewColumnTypeEnum> headerMap, Map<Integer, String>columnMap, List<Map<String, Object>> dataList) {
		PreparedStatement psStatement = null;
		Connection connection = null;
		try {
			DynamicRoutingDataSource dataSource = (DynamicRoutingDataSource) this.dataSource;
			DataSource clickhouse = dataSource.getDataSource("clickhouse");
			connection=clickhouse.getConnection();
			psStatement = connection.prepareStatement(insertSql);
			connection.setAutoCommit(false);
			long startTime = System.currentTimeMillis();
			for(Map<String, Object> data : dataList) {
				for(Map.Entry<Integer, ViewColumnTypeEnum> entry : headerMap.entrySet()) {
					int index = entry.getKey() + 1;
					ViewColumnTypeEnum columnType = entry.getValue();
					String columnName=columnMap.get(entry.getKey());
					String value = data.getOrDefault(columnName, "").toString();
					if(columnType == ViewColumnTypeEnum.DATETIME) {
						Timestamp dateValue = null;
						try {
							Date date = DateUtil.parse(value, CubeConstants.DATE_FORMAT_PATTERNS);
							dateValue = new Timestamp(date.getTime());
						} catch (Exception e) {}
						psStatement.setTimestamp(index, dateValue);
					}else if(columnType == ViewColumnTypeEnum.NUMBER) {
						try {
							value = value.replaceAll(CubeConstants.NUMBER_FORAMT_PATTERNS, "");
							psStatement.setDouble(index, Func.toDouble(value, 0D));
						} catch (Exception e) {}
					}else if(columnType == ViewColumnTypeEnum.BIGINT){
						psStatement.setLong(index, Long.parseLong(value));
					}else{
						psStatement.setString(index, value);
					}
				}
				psStatement.setLong(headerMap.size()+1, Func.toLong(data.get("the_value")));
				psStatement.addBatch();
			}
			psStatement.executeBatch();
			connection.commit();
			psStatement.clearBatch();
			long endTime = System.currentTimeMillis();
			log.info("批量入库成功|dataSize:{}, cost:{}ms", dataList.size(), endTime - startTime);
		} catch(Exception e) {
			throw new RuntimeException(e.getMessage(), e);
		} finally {
			JdbcUtils.close(psStatement);
			JdbcUtils.close(connection);
		}
	}

}
