/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.visual.controller;

import java.util.Date;
import java.util.List;

import javax.validation.Valid;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.tool.api.R;
import org.springblade.visual.entity.VisualClassifyFavourite;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springblade.visual.service.VisualClassifyFavouriteService;
import org.springblade.visual.service.VisualDataService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * 可视化分类表 控制器
 *
 * @author BladeX
 * @since 2021-03-05
 */
@RestController
@RequestMapping("visual/classifyFavourite")
@Api(value = "我的收藏分类接口", tags = "我的收藏分类接口")
public class VisualClassifyFavouriteController extends BladeController {

	@Autowired
	private VisualClassifyFavouriteService visualClassifyFavouriteService;
	
	@Autowired
	private VisualDataService visualDataService;
	
	/**
	 * 分页 可视化分类表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "数据列表", notes = "传入classifyFavourite")
	public R<List<VisualClassifyFavourite>> list(VisualClassifyFavourite classifyFavourite) {
		classifyFavourite.setCreateUserId(getUser().getUserId());
		return R.data(visualClassifyFavouriteService.list(Condition.getQueryWrapper(classifyFavourite).lambda().orderByDesc(VisualClassifyFavourite::getId)));
	}

	/**
	 * 新增或修改 可视化分类表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入visualCategory")
	public R<String> submit(@Valid @RequestBody VisualClassifyFavourite classifyFavourite) {
		if(classifyFavourite.getId() == null) {
			classifyFavourite.setCreateUserId(getUser().getUserId());
			classifyFavourite.setCreateTime(new Date());
		}
		return R.status(visualClassifyFavouriteService.saveOrUpdate(classifyFavourite));
	}
	
	/**
	 * 删除 可视化分类表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "根据ID删除", notes = "传入id")
	public R<String> remove(@ApiParam(value = "主键", required = true) @RequestParam Integer id) {
		long visualCount = visualDataService.count(Wrappers.<VisualData>lambdaQuery()
				.eq(VisualData::getVisualType, VisualTypeEnum.FAVOURITE)
				.eq(VisualData::getCategory, id)
				.eq(VisualData::getVisualDataType,0));
		if(visualCount > 0) {
			return R.fail("该分类下有大屏模版，不能删除。");
		}
		return R.status(visualClassifyFavouriteService.removeById(id));
	}
}
