/**
 * 
 */
package org.springblade.visual.controller;

import java.io.IOException;
import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.visual.entity.VisualComponentDefine;
import org.springblade.visual.service.VisualComponentDefineService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;

import cn.hutool.core.io.IORuntimeException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 *	自定义组件 操作处理
 *
 * @author wookvn
 * @date   2023-02-11
 */
@RestController
@RequestMapping("visual/componentDefine")
@Api(value = "自定义组件相关接口", tags = "自定义组件接口")
public class VisualComponentDefineController extends BladeController {

	@Autowired
	private VisualComponentDefineService visualComponentDefineService;
	
	/**
	 * 	列表数据
	 * @param visualComponentDefine
	 * @return
	 */
	@GetMapping("/page")
	@ApiOperation(value = "分页查询自定义组件", notes = "注意：该接口不返回代码部分内容；且仅查询自己创建的组件[超级管理员查看所有]", httpMethod = "GET")
	public R<IPage<VisualComponentDefine>> pageList(VisualComponentDefine visualComponentDefine, Query query){
		QueryWrapper<VisualComponentDefine> queryWrapper = Condition.getQueryWrapper(visualComponentDefine);
		queryWrapper.select(VisualComponentDefine.class, info -> !info.getColumn().equals("code_text")); //不查询代码部分
		//非超级管理员仅查看自己的数据
		if(!AuthUtil.isAdmin()) {
			queryWrapper.lambda().eq(VisualComponentDefine::getCreateUserId, getUser().getUserId());
		}
		return R.data(visualComponentDefineService.page(Condition.getPage(query), queryWrapper.lambda().orderByDesc(VisualComponentDefine::getId)));
	}
	
	/**
	 * 
	 * @param visualComponentDefine
	 * @return
	 */
	@GetMapping("/list")
	@ApiOperation(value = "查询可使用的组件列表", notes = "注意：该接口不返回代码部分内容；且返回公开和自己创建的已发布组件[超级管理员查看所有已发布]", httpMethod = "GET")
	public R<List<VisualComponentDefine>> dataList(VisualComponentDefine visualComponentDefine){
		QueryWrapper<VisualComponentDefine> queryWrapper = Condition.getQueryWrapper(visualComponentDefine);
		queryWrapper.lambda().select(VisualComponentDefine.class, info -> !info.getColumn().equals("code_text"))
			//查询已发布的组件
			.eq(VisualComponentDefine::getPublishStatus, 1);
		//非超级管理员仅查看自己创建和公开的组件
		if(!AuthUtil.isAdmin()) {
			queryWrapper.lambda().or(query -> {
				//自己创建的组件
				query.eq(VisualComponentDefine::getCreateUserId, getUser().getUserId())
				//或者
				.or()
				//公开状态的组件
				.eq(VisualComponentDefine::getViewScopeStatus, 1);
			});
		}
		return R.data(visualComponentDefineService.list(queryWrapper.lambda().orderByDesc(VisualComponentDefine::getId)));
	}
	
	/**
	 * 单条数据查询接口
	 * @param id
	 * @return
	 */
	@GetMapping("/info")
	@ApiOperation(value = "根据ID获取自定义组件", httpMethod = "GET")
	public R<VisualComponentDefine> info(@RequestParam Integer id) {
		return R.data(visualComponentDefineService.getById(id));
	}
	
	/**
	 *	修改数据接口
	 * @param visualComponentDefine
	 * @return
	 * @throws IOException 
	 * @throws IORuntimeException 
	 */
	@PostMapping("/submit")
	@ApiOperation(value = "增加或修改自定义组件", httpMethod = "POST")
	public R<String> doUpdate(@Validated @RequestBody VisualComponentDefine visualComponentDefine,
			HttpServletRequest request) throws Exception {
		if(visualComponentDefine.getId() == null) {
			visualComponentDefine.setCreateTime(LocalDateTime.now());
			visualComponentDefine.setCreateUserId(getUser().getUserId());
		}
		visualComponentDefine.setUpdateTime(LocalDateTime.now());
		visualComponentDefine.setUpdateBy(getUser().getNickName());
		return R.status(visualComponentDefineService.saveOrUpdate(visualComponentDefine));
	}
	
	/**
	 * 	删除数据接口
	 * @param id
	 * @return
	 */
	@PostMapping("/delete")
	@ApiOperation(value = "根据ID删除自定义组件", notes = "多个逗号分割", httpMethod = "POST")
	public R<String> doDelete(@RequestParam Integer[] id) {
		return R.status(visualComponentDefineService.removeByIds(Arrays.asList(id)));
	}
	
}