/**
 * 
 */
package org.springblade.visual.controller;

import java.util.List;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.visual.dto.VisualDataPermissionDoo;
import org.springblade.visual.dto.VisualDataPermissionDto;
import org.springblade.visual.service.VisualDataPermissionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSONObject;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * @author wookvn
 *
 */
@RestController
@RequestMapping("/visual/permission")
@Api(value = "大屏权限管理", tags = "大屏权限组接口")
public class VisualDataPermissionController extends BladeController {

	@Autowired
	private VisualDataPermissionService visualDataPermissionService;
	
	/**
	 * 
	 * @param permId
	 * @param datasourceId
	 * @return
	 */
	@GetMapping("/visualIdList")
	@ApiOperation(value = "根据权限组ID和分类ID查询大屏ID列表")
	public R<List<Integer>> visualIdList(@RequestParam Integer dataPermId, @RequestParam Integer visualClassifyId){
		return R.data(visualDataPermissionService.listVisualId(dataPermId, visualClassifyId));
	}
	
	/**
	 * 权限组管理-大屏设置的入口
	 * @param reportDatasourcePermission
	 * @return
	 */
	@PostMapping("/submit")
	@ApiOperation(value = "添加或修改", notes = "权限组管理-大屏设置的入口")
	public R<String> doSubmit(@RequestBody VisualDataPermissionDto visualDataPermDto) {
		return R.status(visualDataPermissionService.doSaveOrUpdate(visualDataPermDto, getUser()));
	}
	
	/**
	 * 
	 * @return
	 */
	@PostMapping("/checkPermission/{visualId}/{userId}")
	@ApiOperation(value = "判断用户是否有大屏权限")
	public R<Boolean> checkPermission(@PathVariable Integer visualId, @PathVariable Long userId){
		BladeUser bladeUser = new BladeUser();
		bladeUser.setUserId(userId);
		return R.data(visualDataPermissionService.checkVisualDataForUser(visualId, 0L, bladeUser));
	}
	
	/**
	 * 
	 * @param visualId
	 * @return
	 */
	@GetMapping("/dataPermList/{visualId}")
	@ApiOperation(value = "根据大屏ID查询权限组列表")
	public R<List<JSONObject>> dataPermList(@PathVariable Integer visualId){
		return R.data(visualDataPermissionService.listDataPerm(visualId));
	}
	
	/**
	 * 大屏列表的权限组管理入口
	 * @param reportDatasourcePermission
	 * @return
	 */
	@PostMapping("/perform")
	@ApiOperation(value = "批量添加或修改权限组与大屏关系", notes = "大屏列表-权限组管理入口【超级管理员才能操作】")
	public R<String> doPerform(@RequestBody VisualDataPermissionDoo visualDataPermDoo) {
		if(!AuthUtil.isAdmin()) {
			return R.fail("只能超级管理员操作");
		}
		return R.status(visualDataPermissionService.doPerform(visualDataPermDoo, getUser()));
	}
}