package org.springblade.visual.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.tool.api.R;
import org.springblade.visual.entity.VisualPowerClassifyFavourite;
import org.springblade.visual.entity.VisualPowerPointInfo;
import org.springblade.visual.service.VisualPowerClassifyFavouriteService;
import org.springblade.visual.service.VisualPowerPointInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Date;
import java.util.List;

@RestController
@RequestMapping("visual/power-point/classifyFavourite")
@Api(value = "我的收藏分类接口", tags = "我的收藏幻灯片分类接口")
public class VisualPowerClassifyFavouriteController extends BladeController {

    @Autowired
    private VisualPowerClassifyFavouriteService visualPowerClassifyFavouriteService;

    @Autowired
    private VisualPowerPointInfoService visualPowerPointInfoService;

    /**
     * 分页 可视化分类表
     */
    @GetMapping("/list")
    @ApiOperationSupport(order = 1)
    @ApiOperation(value = "数据列表", notes = "传入classifyFavourite")
    public R<List<VisualPowerClassifyFavourite>> list(VisualPowerClassifyFavourite classifyFavourite) {
        classifyFavourite.setCreateUserId(getUser().getUserId());
        return R.data(visualPowerClassifyFavouriteService.list(Condition.getQueryWrapper(classifyFavourite).lambda().orderByDesc(VisualPowerClassifyFavourite::getId)));
    }

    /**
     * 新增或修改 可视化分类表
     */
    @PostMapping("/submit")
    @ApiOperationSupport(order = 6)
    @ApiOperation(value = "新增或修改", notes = "传入visualCategory")
    public R<String> submit(@Valid @RequestBody VisualPowerClassifyFavourite classifyFavourite) {
        if(classifyFavourite.getId() == null) {
            classifyFavourite.setCreateUserId(getUser().getUserId());
            classifyFavourite.setCreateTime(new Date());
        }
        return R.status(visualPowerClassifyFavouriteService.saveOrUpdate(classifyFavourite));
    }

    /**
     * 删除 可视化分类表
     */
    @PostMapping("/remove")
    @ApiOperationSupport(order = 8)
    @ApiOperation(value = "根据ID删除", notes = "传入id")
    public R<String> remove(@ApiParam(value = "主键", required = true) @RequestParam Integer id) {
        long visualCount = visualPowerPointInfoService.count(Wrappers.<VisualPowerPointInfo>lambdaQuery()
                .eq(VisualPowerPointInfo::getVisualType, 1)
                .eq(VisualPowerPointInfo::getCateId, id));
        if(visualCount > 0) {
            return R.fail("该分类下有幻灯片，不能删除。");
        }
        return R.status(visualPowerClassifyFavouriteService.removeById(id));
    }
}
