/**
 * 
 */
package org.springblade.visual.controller;

import java.time.LocalDateTime;
import java.util.List;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.system.client.DataPermClient;
import org.springblade.visual.entity.VisualClassify;
import org.springblade.visual.entity.VisualPowerPointCategory;
import org.springblade.visual.entity.VisualPowerPointInfo;
import org.springblade.visual.service.VisualPowerPointCategoryService;
import org.springblade.visual.service.VisualPowerPointInfoService;
import org.springblade.visual.service.VisualPowerPointPermissionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 *	幻灯片分类 操作处理
 *
 * @author wookvn
 * @date   2023-02-04
 */
@RestController
@RequestMapping("visual/powerPointCategory")
@Api(value = "幻灯片分类相关接口", tags = "幻灯片分类接口")
public class VisualPowerPointCategoryController extends BladeController {

	@Autowired
	private VisualPowerPointCategoryService visualPowerPointCategoryService;
	
	@Autowired
	private VisualPowerPointInfoService visualPowerPointInfoService;

	@Autowired
	private VisualPowerPointPermissionService visualPowerPointPermissionService;

	@Autowired
	private DataPermClient dataPermClient;
	
	/**
	 * 	列表数据
	 * @param visualPowerPointCategory
	 * @return
	 */
	@GetMapping("/page")
	@ApiOperation(value = "分页查询幻灯片分类", httpMethod = "GET")
	public R<List<VisualPowerPointCategory>> pageList(VisualPowerPointCategory visualPowerPointCategory, Query query){
		QueryWrapper<VisualPowerPointCategory> queryWrapper = Condition.getQueryWrapper(visualPowerPointCategory);
		long currUserId = getUser().getUserId();
		// 首先查询全部的分类
		List<VisualPowerPointCategory> dataList = visualPowerPointCategoryService.list(
				queryWrapper.lambda().orderByDesc(VisualPowerPointCategory::getSortNum));
		Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(currUserId);
		if (checkSuperPermRet == null || !checkSuperPermRet) {
			// 查询所有PPT有权限的分类ID集合
			List<Integer> categoryIdList = visualPowerPointPermissionService.categoryIdListPerm(currUserId);
			//1.获取这些分类的父级ID[包含自己创建的分类父级别ID]
			dataList.stream().filter(f -> (categoryIdList.contains(f.getId()) || f.getCreateUserId().longValue() == currUserId) && f.getParentId() != 0).map(VisualPowerPointCategory::getParentId).forEach(categoryIdList::add);
			//2.删除不在权限内的分类[且createUserId不是本人的]
			dataList.removeIf(y -> !categoryIdList.contains(y.getId()) && y.getCreateUserId().longValue() != currUserId);

		}

		return R.data(dataList);
	}
	
	/**
	 *	修改数据接口
	 * @param visualPowerPointCategory
	 * @return
	 */
	@PostMapping("/submit")
	@ApiOperation(value = "增加或修改幻灯片分类", httpMethod = "POST")
	public R<String> doUpdate(@Validated @RequestBody VisualPowerPointCategory visualPowerPointCategory) {
		if(visualPowerPointCategory.getId() == null) {
			visualPowerPointCategory.setCreateTime(LocalDateTime.now());
			visualPowerPointCategory.setCreateUserId(getUser().getUserId());
		}
		if(visualPowerPointCategory.getId() == null && visualPowerPointCategory.getParentId() != null && visualPowerPointCategory.getParentId() > 0) {
			//添加子级分类，判断父级分类是否有幻灯片
			long visualCount = visualPowerPointInfoService.count(Wrappers.<VisualPowerPointInfo>lambdaQuery().eq(VisualPowerPointInfo::getCateId, visualPowerPointCategory.getParentId()));
			if(visualCount > 0) {
				return R.fail("你选择的父级分类下有幻灯片，不能新建子分类。");
			}
		}
		return R.status(visualPowerPointCategoryService.saveOrUpdate(visualPowerPointCategory));
	}
	
	/**
	 * 	删除数据接口
	 * @param id
	 * @return
	 */
	@PostMapping("/delete")
	@ApiOperation(value = "根据ID删除幻灯片分类", httpMethod = "POST")
	public R<String> doDelete(@ApiParam(value = "主键", required = true) @RequestParam Integer id) {
		long count = visualPowerPointCategoryService.count(Wrappers.<VisualPowerPointCategory>lambdaQuery().eq(VisualPowerPointCategory::getParentId, id));
		if(count > 0) {
			return R.fail("该分类下有子分类，不能删除。");
		}
		long visualCount = visualPowerPointInfoService.count(Wrappers.<VisualPowerPointInfo>lambdaQuery().eq(VisualPowerPointInfo::getCateId, id));
		if(visualCount > 0) {
			return R.fail("该分类下有幻灯片，不能删除。");
		}
		return R.status(visualPowerPointCategoryService.removeById(id));
	}
	
}