package org.springblade.visual.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.tool.api.R;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualReportCategoryFavourite;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springblade.visual.service.VisualDataService;
import org.springblade.visual.service.VisualReportCategoryFavouriteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Date;
import java.util.List;

@RestController
@RequestMapping("visual/category/favourite/report")
@Api(value = "我的收藏自助报表分类接口", tags = "我的收藏自助报表分类接口")
public class VisualReportCategoryFavouriteController extends BladeController {

    @Autowired
    private VisualReportCategoryFavouriteService visualReportCategoryFavouriteService;

    @Autowired
    private VisualDataService visualDataService;

    /**
     * 分页 可视化分类表
     */
    @GetMapping("/list")
    @ApiOperationSupport(order = 1)
    @ApiOperation(value = "数据列表", notes = "categoryFavourite")
    public R<List<VisualReportCategoryFavourite>> list(VisualReportCategoryFavourite categoryFavourite) {
        categoryFavourite.setCreateUserId(getUser().getUserId());
        return R.data(visualReportCategoryFavouriteService.list(Condition.getQueryWrapper(categoryFavourite).lambda().orderByDesc(VisualReportCategoryFavourite::getId)));
    }

    /**
     * 新增或修改 可视化分类表
     */
    @PostMapping("/submit")
    @ApiOperationSupport(order = 6)
    @ApiOperation(value = "新增或修改", notes = "传入visualCategory")
    public R<String> submit(@Valid @RequestBody VisualReportCategoryFavourite categoryFavourite) {
        if(categoryFavourite.getId() == null) {
            categoryFavourite.setCreateUserId(getUser().getUserId());
            categoryFavourite.setCreateTime(new Date());
        }
        return R.status(visualReportCategoryFavouriteService.saveOrUpdate(categoryFavourite));
    }

    /**
     * 删除 可视化分类表
     */
    @PostMapping("/remove")
    @ApiOperationSupport(order = 8)
    @ApiOperation(value = "根据ID删除", notes = "传入id")
    public R<String> remove(@ApiParam(value = "主键", required = true) @RequestParam Integer id) {
        long visualCount = visualDataService.count(Wrappers.<VisualData>lambdaQuery()
                .eq(VisualData::getVisualType, VisualTypeEnum.FAVOURITE)
                .eq(VisualData::getCategory, id)
                .eq(VisualData::getVisualDataType,1));
        if(visualCount > 0) {
            return R.fail("该分类下有大屏模版，不能删除。");
        }
        return R.status(visualReportCategoryFavouriteService.removeById(id));
    }
}
