/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.visual.controller;

import java.util.List;
import java.util.stream.Collectors;

import javax.validation.Valid;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.visual.dto.VisualReportInfoData;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualReportInfo;
import org.springblade.visual.service.VisualComponentService;
import org.springblade.visual.service.VisualDataService;
import org.springblade.visual.service.VisualReportInfoService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * 历史报告 控制器
 *
 * @author BladeX
 * @since 2021-07-02
 */
@RestController
@RequestMapping("visual/reportinfo")
@Api(value = "报告", tags = "报告接口")
public class VisualReportInfoController extends BladeController {

	@Autowired
	private VisualReportInfoService reportInfoService;
	
	@Autowired
	private VisualDataService visualDataService;
	
	@Autowired
	private VisualComponentService visualComponentService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入reportInfo")
	public R<VisualReportInfoData> detail(VisualReportInfo reportInfo) throws Exception {
		VisualReportInfo detail = reportInfoService.getOne(Condition.getQueryWrapper(reportInfo));
		VisualReportInfoData reportInfoData = new VisualReportInfoData();
		BeanUtils.copyProperties(detail, reportInfoData);
		
		LambdaQueryWrapper<VisualData> queryWrapper = Wrappers.<VisualData>lambdaQuery().eq(VisualData::getHistoryReportId, detail.getId());
		List<VisualData> visualList = visualDataService.list(queryWrapper);
		reportInfoData.setVisualList(visualList);
		visualList.forEach(item -> {
			//查询大屏使用的所有组件
			LambdaQueryWrapper<VisualComponent> _queryWrapper = Wrappers.<VisualComponent>lambdaQuery().eq(VisualComponent::getVisualId, item.getId());
			List<Integer> componentIdList = visualComponentService.listObjs(_queryWrapper, a -> Integer.valueOf(a.toString()));
			item.setComponentIds(componentIdList);
		});
		return R.data(reportInfoData);
	}

	/**
	 * 分页 历史报告
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "列表", notes = "传入reportInfo")
	public R<List<VisualReportInfo>> list(VisualReportInfo reportInfo) {
		QueryWrapper<VisualReportInfo> queryWrapper = Condition.getQueryWrapper(reportInfo);
		queryWrapper.lambda().eq(VisualReportInfo::getCreateUserId, getUser().getUserId())
		.orderByAsc(VisualReportInfo::getSortOrder).orderByDesc(VisualReportInfo::getId);
		return R.data(reportInfoService.list(queryWrapper));
	}

	/**
	 * 自定义分页 历史报告
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入reportInfo")
	public R<IPage<VisualReportInfo>> page(VisualReportInfo reportInfo, Query query) {
		 QueryWrapper<VisualReportInfo> queryWrapper = Condition.getQueryWrapper(reportInfo);
		 queryWrapper.lambda().eq(VisualReportInfo::getCreateUserId, getUser().getUserId())
			.orderByAsc(VisualReportInfo::getSortOrder).orderByDesc(VisualReportInfo::getId);
		return R.data(reportInfoService.page(Condition.getPage(query), queryWrapper));
	}

	/**
	 * 新增 历史报告
	 */
	@PostMapping("/copySave")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "复制新增", notes = "传入reportInfoData[报告的大屏只需要传入visualID即可]")
	public R<Integer> save(@RequestBody VisualReportInfoData reportInfoData) throws Exception{
		//1.判断是否选择目录
		if(reportInfoData.getFolderId() == null) {
			return R.fail("请选择报告目录");
		}
		if(reportInfoData.getVisualList() == null || reportInfoData.getVisualList().isEmpty()) {
			return R.fail("请选择大屏");
		}
		VisualReportInfo reportInfo = new VisualReportInfo();
		BeanUtils.copyProperties(reportInfoData, reportInfo);
		List<Integer> visualIdList = reportInfoData.getVisualList().stream().map(VisualData::getId).collect(Collectors.toList());
		return R.data(reportInfoService.doSave(reportInfo, visualIdList, getUser()));
	}
	
	/**
	 * 修改 历史报告
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "修改报告的基础信息", notes = "传入reportInfo")
	public R<String> update(@Valid @RequestBody VisualReportInfo reportInfo) {
		return R.status(reportInfoService.updateById(reportInfo));
	}

	/**
	 * 删除 历史报告
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "删除 remove", notes = "传入ids")
	public R<String> remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(reportInfoService.doRemove(Func.toLongList(ids)));
	}

}
