package org.springblade.visual.controller;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.system.client.DataPermClient;
import org.springblade.visual.dto.VisualDto;
import org.springblade.visual.entity.VisualClassify;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualSelfReportCategory;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springblade.visual.service.VisualComponentService;
import org.springblade.visual.service.VisualDataService;
import org.springblade.visual.service.VisualSelfReportCategoryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;

@RestController
@RequestMapping("visual/selfReport")
@Api(value = "自助报表接口", tags = "自助报表接口")
public class VisualSelfReportController extends BladeController {

    @Autowired
    private VisualDataService visualDataService;

    @Autowired
    private VisualComponentService visualComponentService;

    @Autowired
    private VisualSelfReportCategoryService visualSelfReportCategoryService;

    @Autowired
    private DataPermClient dataPermClient;

    /**
     * 详情
     */
    @GetMapping("/detail")
    @ApiOperationSupport(order = 1)
    @ApiOperation(value = "详情", notes = "传入visual")
    public R<VisualData> detail(VisualData visualData) {
        VisualData detail = visualDataService.getOne(Condition.getQueryWrapper(visualData));
        if(detail == null) {
            return R.fail("自助报表模版不存在");
        }
        //查询大屏使用的所有组件
        LambdaQueryWrapper<VisualComponent> queryWrapper = Wrappers.<VisualComponent>lambdaQuery().select(VisualComponent::getId).eq(VisualComponent::getVisualId, detail.getId());
        List<Integer> componentIdList = visualComponentService.listObjs(queryWrapper, a -> Integer.valueOf(a.toString()));
        detail.setComponentIds(componentIdList);
        return R.data(detail);
    }

    /**
     *
     * @param visualData
     * @return
     */
    @GetMapping("/list")
    @ApiOperationSupport(order = 2)
    @ApiOperation(value = "自助报表列表")
    public R<IPage<VisualData>> list(VisualData visualData, Query query){
        Long userId = getUser().getUserId();
        visualData.setVisualDataType(1);
        LambdaQueryWrapper<VisualData> queryWrapper = Condition.getQueryWrapper(visualData).lambda().eq(VisualData::getVisualType, VisualTypeEnum.GENERAL).orderByAsc(VisualData::getOrderIndex);
        Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(getUser().getUserId());
        if(checkSuperPermRet == null || !checkSuperPermRet) {
            queryWrapper.and(t -> t.eq(VisualData::getCreateUser, userId)
                    .or().inSql(VisualData::getId, "select a.self_report_id from visual_self_report_permission a inner join data_permission_user b on a.data_perm_id = b.data_perm_id where b.user_id = " + userId)
                    .or().eq(VisualData::getPublicStatus, 1)).eq(VisualData::getVisualDataType,1); // 查询自助报表
        }
        return R.data(visualDataService.page(Condition.getPage(query), queryWrapper));
    }

    /**
     *
     * @param visualData
     * @return
     */
    @GetMapping("/myList")
    @ApiOperationSupport(order = 3)
    @ApiOperation(value = "我的收藏列表")
    public R<IPage<VisualData>> mylist(VisualData visualData, Query query){
        visualData.setCreateUser(getUser().getUserId());
        visualData.setVisualType(VisualTypeEnum.FAVOURITE);
        return R.data(visualDataService.page(Condition.getPage(query), Condition.getQueryWrapper(visualData).lambda().orderByAsc(VisualData::getOrderIndex)));
    }

    /**
     *
     * @param visualData
     * @return
     */
    @PostMapping("/update")
    @ApiOperationSupport(order = 4)
    @ApiOperation(value = "更新自主报表（仅更新报表的数据，不传递参数不更新）")
    public R<String> update(@RequestBody VisualData visualData) {
        if(visualData.getCategory() != null && visualData.getCategory() > 0) {
        	long count = visualSelfReportCategoryService.count(Wrappers.<VisualSelfReportCategory>lambdaQuery()
                    .eq(VisualSelfReportCategory::getParentId, visualData.getCategory()));
            if(count > 0) {
                return R.fail("该分类下有子分类，不能放置自助报表。");
            }
        }
        return R.status(visualDataService.updateById(visualData));
    }

    /**
     *
     * @param value
     * @return
     * @throws IOException
     */
    @PostMapping("/submit")
    @ApiOperationSupport(order = 5)
    @ApiOperation(value = "新增或更新自助报表", notes = "返回值：自助报表ID")
    @ApiResponse(message = "自助报表ID", code = 200)
    public R<Integer> submit(@RequestBody String value) throws IOException {
        String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
        VisualDto visualDto = JSON.parseObject(decoderStr, VisualDto.class);
        VisualData visualData = visualDto.getVisualData();
        visualData.setVisualDataType(1);
        visualDto.setVisualData(visualData);
        return R.data(visualDataService.doSaveOrUpdate(visualDto, getUser()));
    }

    /**
     *
     * @param ids
     * @return
     */
    @PostMapping("/delete")
    @ApiOperationSupport(order = 6)
    @ApiOperation(value = "删除自助报表")
    public R<String> delete(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
        visualDataService.doRemove(Func.toIntList(ids));
        return R.success("删除成功");
    }

    /**
     * 拷贝 可视化表
     */
    @PostMapping("/copy")
    @ApiOperationSupport(order = 7)
    @ApiOperation(value = "拷贝自助报表模板")
    public R<Integer> copy(
            @ApiParam(value = "原报表id", required = true) @RequestParam Integer id,
            @ApiParam(value = "新分类id") @RequestParam(required = false) Integer newCategoryId,
            @ApiParam(value = "新大屏名称") @RequestParam(required = false) String visualName,
            @ApiParam(value = "租户id") @RequestParam(required = false) String tenantId,
            @ApiParam(value = "报表类型(默认不传递：GENERAL)") @RequestParam(required = false, value = "visualType", defaultValue = "GENERAL") VisualTypeEnum visualType) {
        String name = "[副本]";
        if (StrUtil.isNotEmpty(visualName)) {
            name = visualName;
        }
        List<Integer> visualIdList = visualDataService.doCopy(CollectionUtil.newHashSet(id), newCategoryId, null, null, visualType, getUser(), name, tenantId);
        return R.data(CollectionUtil.isNotEmpty(visualIdList) ? visualIdList.get(0) : null);
    }

    /**
     * 调整顺序
     * @return
     */
    @PostMapping("/adjustOrder")
    @ApiOperationSupport(order = 8)
    @ApiOperation(value = "调整顺序", notes = "传入自助报表ID列表【切记：保证自助报表顺序】")
    public R<String> uptOrder(@ApiParam(value = "主键集合  1,2,3", required = true) @RequestParam String ids) {
        List<VisualData> entityList = new ArrayList<>();
        List<Integer> visualIdList = Func.toIntList(ids);
        for(int i = 0; i < visualIdList.size(); i++) {
            VisualData data = new VisualData();
            data.setId(visualIdList.get(i));
            data.setOrderIndex(i);
            entityList.add(data);
        }
        return R.status(visualDataService.updateBatchById(entityList));
    }
}
