package org.springblade.visual.controller;

import com.alibaba.fastjson2.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.visual.dto.VisualSelfReportPermissionDoo;
import org.springblade.visual.dto.VisualSelfReportPermissionDto;
import org.springblade.visual.service.VisualSelfReportPermissionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("visual/report/permission")
@Api(value = "自主报表权限管理", tags = "自主报表权限组接口")
public class VisualSelfReportPermissionController extends BladeController {

    @Autowired
    private VisualSelfReportPermissionService visualSelfReportPermissionService;

    /**
     *
     * @param dataPermId
     * @param reportCategoryId
     * @return
     */
    @GetMapping("/selfReportIdList")
    @ApiOperation(value = "根据权限组ID和分类ID查询自助报表ID列表")
    public R<List<Integer>> visualIdList(@RequestParam Integer dataPermId, @RequestParam Integer reportCategoryId){
        return R.data(visualSelfReportPermissionService.listVisualId(dataPermId, reportCategoryId));
    }

    /**
     * 权限组管理-大屏设置的入口
     * @param reportPermissionDto
     * @return
     */
    @PostMapping("/submit")
    @ApiOperation(value = "添加或修改", notes = "权限组管理-自主报表的入口")
    public R<String> doSubmit(@RequestBody VisualSelfReportPermissionDto reportPermissionDto) {
        return R.status(visualSelfReportPermissionService.doSaveOrUpdate(reportPermissionDto, getUser()));
    }

    /**
     *
     * @return
     */
    @PostMapping("/checkPermission/{selfReportId}/{userId}")
    @ApiOperation(value = "判断用户是否有自助报表权限")
    public R<Boolean> checkPermission(@PathVariable Integer selfReportId, @PathVariable Long userId){
        BladeUser bladeUser = new BladeUser();
        bladeUser.setUserId(userId);
        return R.data(visualSelfReportPermissionService.checkVisualDataForUser(selfReportId, 0L, bladeUser));
    }

    /**
     *
     * @param selfReportId
     * @return
     */
    @GetMapping("/dataPermList/{selfReportId}")
    @ApiOperation(value = "根据自助报表ID查询权限组列表")
    public R<List<JSONObject>> dataPermList(@PathVariable Integer selfReportId){
        return R.data(visualSelfReportPermissionService.listDataPerm(selfReportId));
    }

    /**
     * 自助报表列表的权限组管理入口
     * @param reportPermissionDoo
     * @return
     */
    @PostMapping("/perform")
    @ApiOperation(value = "批量添加或修改权限组与自助报表关系", notes = "自助报表列表-权限组管理入口【超级管理员才能操作】")
    public R<String> doPerform(@RequestBody VisualSelfReportPermissionDoo reportPermissionDoo) {
        if(!AuthUtil.isAdmin()) {
            return R.fail("只能超级管理员操作");
        }
        return R.status(visualSelfReportPermissionService.doPerform(reportPermissionDoo, getUser()));
    }
}
