package org.springblade.visual.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.tool.api.R;
import org.springblade.system.client.DataPermClient;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualTemplateClassify;
import org.springblade.visual.service.VisualDataService;
import org.springblade.visual.service.VisualTemplateClassifyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Base64;
import java.util.Date;
import java.util.List;

@RestController
@RequestMapping("visual/template/classify")
@Api(value = "模板分类", tags = "模板分类接口")
public class VisualTemplateClassifyController extends BladeController {

    @Autowired
    private VisualTemplateClassifyService visualTemplateClassifyService;

    @Autowired
    private VisualDataService visualDataService;

    @Autowired
    private DataPermClient dataPermClient;

    @GetMapping("/detail")
    @ApiOperationSupport(order = 1)
    @ApiOperation(value = "详情", notes = "传入templateClassify")
    public R<VisualTemplateClassify> detail(VisualTemplateClassify templateClassify) {
        VisualTemplateClassify classify = visualTemplateClassifyService.getOne(Condition.getQueryWrapper(templateClassify));
        return R.data(classify);
    }

    /**
     * 分页 可视化分类表
     */
    @GetMapping("/list")
    @ApiOperationSupport(order = 2)
    @ApiOperation(value = "分页", notes = "传入visualCategory")
    public R<List<VisualTemplateClassify>> list(VisualTemplateClassify templateClassify) {
        LambdaQueryWrapper<VisualTemplateClassify> queryWrapper = Condition.getQueryWrapper(templateClassify).lambda();
        queryWrapper.and(t -> t.eq(VisualTemplateClassify::getTenantId, "111111").or().eq(VisualTemplateClassify::getTenantId, getUser().getTenantId()));
        List<VisualTemplateClassify> dataList = visualTemplateClassifyService.list(queryWrapper.orderByDesc(VisualTemplateClassify::getSortNum));
        return R.data(dataList);
//        long currUserId = getUser().getUserId();
//        Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(currUserId);
//        if(checkSuperPermRet != null && checkSuperPermRet) { //是否有超级权限
//            return R.data(dataList);
//        }else {
//            //查询所有大屏有权限的分类ID集合
//            List<Integer> classifyIdList = visualDataService.classifyIdListPerm(currUserId,0);
//            //1.获取这些分类的父级ID[包含自己创建的分类父级别ID]
//            dataList.stream().filter(f -> (classifyIdList.contains(f.getId()) || f.getCreateUserId().longValue() == currUserId) && f.getParentId() != 0).map(VisualTemplateClassify::getParentId).forEach(classifyIdList::add);
//            //2.删除不在权限内的分类[且createUserId不是本人的]
//            dataList.removeIf(y -> !classifyIdList.contains(y.getId()) && y.getCreateUserId().longValue() != currUserId);
//            return R.data(dataList);
//        }
    }

    /**
     * 新增或修改 可视化分类表
     */
    @PostMapping("/submit")
    @ApiOperationSupport(order = 6)
    @ApiOperation(value = "新增或修改", notes = "传入visualCategory")
    public R<String> submit(@Valid @RequestBody VisualTemplateClassify visualCategory) {
        if (visualCategory.getId() == null && visualCategory.getParentId() != null && visualCategory.getParentId() > 0) {
            //添加子级分类，判断父级分类是否有大屏模版
            long visualCount = visualDataService.count(Wrappers.<VisualData>lambdaQuery()
                    .eq(VisualData::getCategory, visualCategory.getParentId())
                    .eq(VisualData::getVisualDataType, 0));
            if (visualCount > 0) {
                return R.fail("你选择的父级分类下有大屏模版，不能新建子分类。");
            }
        }
        if (visualCategory.getId() == null) {
            visualCategory.setCreateUserId(getUser().getUserId());
            visualCategory.setCreateTime(new Date());
        }
        visualCategory.setTenantId(getUser().getTenantId());
        return R.status(visualTemplateClassifyService.saveOrUpdate(visualCategory));
    }

    /**
     * 删除 可视化分类表
     */
    @PostMapping("/remove")
    @ApiOperationSupport(order = 8)
    @ApiOperation(value = "根据ID删除", notes = "传入id")
    public R<String> remove(@ApiParam(value = "主键", required = true) @RequestParam Integer id) {
        long count = visualTemplateClassifyService.count(Wrappers.<VisualTemplateClassify>lambdaQuery().eq(VisualTemplateClassify::getParentId, id));
        if (count > 0) {
            return R.fail("该分类下有子分类，不能删除。");
        }
        long visualCount = visualDataService.count(Wrappers.<VisualData>lambdaQuery()
                .eq(VisualData::getCategory, id)
                .eq(VisualData::getVisualDataType, 0));
        if (visualCount > 0) {
            return R.fail("该分类下有大屏模版，不能删除。");
        }
        return R.status(visualTemplateClassifyService.removeById(id));
    }

    @ApiOperation("根据租户id查询大屏分类")
    @GetMapping("/list-tenantid")
    public R<List<VisualTemplateClassify>> getClassifyTenantId(String tenantId) {

        return R.data(visualTemplateClassifyService.list(Wrappers.<VisualTemplateClassify>lambdaQuery()
                .eq(VisualTemplateClassify::getTenantId, tenantId)));
    }
}
