package org.springblade.visual.controller;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.system.client.DataPermClient;
import org.springblade.visual.dto.TemplateDto;
import org.springblade.visual.entity.*;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springblade.visual.service.VisualComponentService;
import org.springblade.visual.service.VisualTemplateClassifyService;
import org.springblade.visual.service.VisualTemplateService;
import org.springblade.visual.vo.CopyVisualDataCubeDataVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;

@RestController
@RequestMapping("visual/template")
@Api(value = "模版接口", tags = "模版接口")
public class VisualTemplateController extends BladeController {

    @Autowired
    private VisualTemplateService visualTemplateService;

    @Autowired
    private VisualTemplateClassifyService visualTemplateClassifyService;

    @Autowired
    private VisualComponentService visualComponentService;

    /**
     * 查询模板详情
     */
    @GetMapping("/detail")
    @ApiOperationSupport(order = 1)
    @ApiOperation(value = "详情", notes = "传入visual")
    public R<VisualTemplate> detail(VisualTemplate template) {
        VisualTemplate detail = visualTemplateService.getById(template.getId());
        if(detail == null) {
            return R.fail("大屏模版不存在");
        }
        //查询大屏使用的所有组件
        LambdaQueryWrapper<VisualComponent> queryWrapper = Wrappers.<VisualComponent>lambdaQuery()
                .select(VisualComponent::getId)
                .eq(VisualComponent::getVisualId, detail.getId())
                .eq(VisualComponent::getIsMobile, 0);
        List<Integer> componentIdList = visualComponentService.listObjs(queryWrapper, a -> Integer.valueOf(a.toString()));
        detail.setComponentIds(componentIdList);
        // 查询大屏组件信息
        return R.data(detail);
    }

    /**
     * 查询模板列表
     * @return
     */
    @GetMapping("/list")
    @ApiOperationSupport(order = 2)
    @ApiOperation(value = "列表")
    public R<IPage<VisualTemplate>> list(VisualTemplate template, Query query){
//        Long userId = getUser().getUserId();
        LambdaQueryWrapper<VisualTemplate> queryWrapper = Condition.getQueryWrapper(template).lambda().orderByAsc(VisualTemplate::getOrderIndex);
//        if (template.getPublishStatus() == null || template.getPublishStatus() < 0) {
//            queryWrapper.eq(VisualTemplate::getPublishStatus, 1);
//        }
        queryWrapper.and(t-> t.eq(VisualTemplate::getTenantId,getUser().getTenantId()));
//        queryWrapper.and(t-> t.eq(VisualTemplate::getTenantId,getUser().getTenantId()).or().eq(VisualTemplate::getTenantId,"111111"));
//        Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(getUser().getUserId());
//        if(checkSuperPermRet == null || !checkSuperPermRet) {
//            queryWrapper.and(t ->{
//                t.eq(VisualData::getCreateUser, userId)
//                        .or().inSql(VisualData::getId, "select a.visual_id from visual_data_permission a inner join data_permission_user b on a.data_perm_id = b.data_perm_id where b.user_id = " + userId)
//                        .or().eq(VisualData::getPublicStatus, 1);
//            }).eq(VisualData::getVisualDataType,0);// 查询公共大屏
//        }
        return R.data(visualTemplateService.page(Condition.getPage(query), queryWrapper));
    }

    @GetMapping("/super/list")
    @ApiOperationSupport(order = 2)
    @ApiOperation(value = "不分租户查询模板列表，只有公司中台可以使用")
    public R<IPage<VisualTemplate>> superList(VisualTemplate template, Query query) {
        if (!getUser().getTenantId().equals("111111")) {
            return R.fail("没有查看的权限");
        }
        LambdaQueryWrapper<VisualTemplate> queryWrapper = Condition.getQueryWrapper(template).lambda().orderByAsc(VisualTemplate::getOrderIndex);
        return R.data(visualTemplateService.page(Condition.getPage(query), queryWrapper));
    }

    /**
     * 更新模板信息
     */
    @PostMapping("/update")
    @ApiOperationSupport(order = 3)
    @ApiOperation(value = "更新模板（仅更新模板的数据，不传递参数不更新）")
    public R<String> update(@RequestBody VisualTemplate template) {
        if(template.getCategory() != null) {
            long count = visualTemplateClassifyService.count(Wrappers.<VisualTemplateClassify>lambdaQuery().eq(VisualTemplateClassify::getParentId, template.getCategory()));
            if(count > 0) {
                return R.fail("该分类下有子分类，不能放置大屏。");
            }
        }
        return R.status(visualTemplateService.updateById(template));
    }

    /**
     * 创建模板
     */
    @PostMapping("/submit")
    @ApiOperationSupport(order = 4)
    @ApiOperation(value = "新增或更模板", notes = "返回值：大屏ID")
    @ApiResponse(message = "大屏ID", code = 200)
    public R<Integer> submitNormal(@RequestBody String value) throws IOException {
        String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
        TemplateDto templateDto = JSON.parseObject(decoderStr, TemplateDto.class);
        VisualTemplate template = templateDto.getTemplate();
        template.setVisualDataType(0);
        templateDto.setTemplate(template);
        return R.data(visualTemplateService.doSaveOrUpdate(templateDto, getUser()));
    }

    /**
     * 创建自助报表模板
     */
    @PostMapping("/submit-self")
    @ApiOperationSupport(order = 5)
    @ApiOperation(value = "新增或更模板", notes = "返回值：大屏ID")
    @ApiResponse(message = "大屏ID", code = 200)
    public R<Integer> submit(@RequestBody String value) throws IOException {
        String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
        TemplateDto templateDto = JSON.parseObject(decoderStr, TemplateDto.class);
        VisualTemplate template = templateDto.getTemplate();
        template.setVisualDataType(1);
        templateDto.setTemplate(template);
        return R.data(visualTemplateService.doSaveOrUpdate(templateDto, getUser()));
    }

    /**
     * 删除模板
     */
    @PostMapping("/delete")
    @ApiOperationSupport(order = 6)
    @ApiOperation(value = "删除模板")
    public R<String> delete(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
        visualTemplateService.doRemove(Func.toIntList(ids));
        return R.success("删除成功");
    }

    /**
     * 调整顺序
     */
    @PostMapping("/adjustOrder")
    @ApiOperationSupport(order = 7)
    @ApiOperation(value = "调整顺序", notes = "传入大屏ID列表【切记：保证大屏顺序】")
    public R<String> uptOrder(@ApiParam(value = "主键集合  1,2,3", required = true) @RequestParam String ids) {
        List<VisualTemplate> entityList = new ArrayList<VisualTemplate>();
        List<Integer> visualIdList = Func.toIntList(ids);
        for(int i = 0; i < visualIdList.size(); i++) {
            VisualTemplate template = new VisualTemplate();
            template.setId(visualIdList.get(i));
            template.setOrderIndex(i);
            entityList.add(template);
        }
        return R.status(visualTemplateService.updateBatchById(entityList));
    }

    /**
     * 发布模板
     */
    @PostMapping("/publish")
    @ApiOperationSupport(order = 8)
    @ApiOperation(value = "发布模板", notes = "传入大屏ID列")
    public R<String> publishTemplate(@ApiParam(value = "主键集合  1,2,3", required = true) @RequestParam String ids) {
        List<VisualTemplate> entityList = new ArrayList<VisualTemplate>();
        List<Integer> visualIdList = Func.toIntList(ids);
        for(int i = 0; i < visualIdList.size(); i++) {
            VisualTemplate template = new VisualTemplate();
            template.setId(visualIdList.get(i));
            template.setPublishStatus(1);
            entityList.add(template);
        }
        return R.status(visualTemplateService.updateBatchById(entityList));
    }

    /**
     * 取消发布模板
     */
    @PostMapping("/non-publish")
    @ApiOperationSupport(order = 9)
    @ApiOperation(value = "取消发布模板", notes = "传入大屏ID列")
    public R<String> nonPublishTemplate(@ApiParam(value = "主键集合  1,2,3", required = true) @RequestParam String ids) {
        List<VisualTemplate> entityList = new ArrayList<VisualTemplate>();
        List<Integer> visualIdList = Func.toIntList(ids);
        for(int i = 0; i < visualIdList.size(); i++) {
            VisualTemplate template = new VisualTemplate();
            template.setId(visualIdList.get(i));
            template.setPublishStatus(0);
            entityList.add(template);
        }
        return R.status(visualTemplateService.updateBatchById(entityList));
    }

    /**
     * 拷贝 可视化表
     */
    @PostMapping("/copy")
    @ApiOperationSupport(order = 10)
    @ApiOperation(value = "拷贝模板")
    public R<VisualTemplate> copy(
            @ApiParam(value = "原大屏id", required = true) @RequestParam Integer id,
            @ApiParam(value = "复制大屏名称", required = false) @RequestParam(required = false) String visualName,
            @ApiParam(value = "新分类id", required = false) @RequestParam(required = false) Integer newCategoryId,
            @ApiParam(value = "租户id") @RequestParam(required = false) String tenantId,
            @ApiParam(value = "大屏类型(默认不传递：GENERAL)", required = false) @RequestParam(required = false, value = "visualType", defaultValue = "GENERAL") VisualTypeEnum visualType) {
        String name = "[副本]";
        if (StrUtil.isNotEmpty(visualName)) {
            name = visualName;
        }
        VisualTemplate template = visualTemplateService.doCopy(id, newCategoryId, null, null, visualType, getUser(), name, tenantId);
        return R.data(template);
    }

    /**
     * 拷贝 可视化表
     */
    @PostMapping("/copy-cube")
    @ApiOperationSupport(order = 9)
    @ApiOperation(value = "拷屏模板和组件数据集-支持夸租户复制")
    public R<Integer> copyCube(@RequestBody CopyVisualDataCubeDataVo cubeData) {

        return visualTemplateService.doCopyCube(cubeData,getUser());
    }
}
