package org.springblade.visual.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.tencentcloudapi.tci.v20190318.models.LightStandard;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.api.R;
import org.springblade.system.client.DataPermClient;
import org.springblade.visual.dto.VisualTemplateUseSituationDto;
import org.springblade.visual.entity.VisualTemplateUseSituation;
import org.springblade.visual.service.VisualDataService;
import org.springblade.visual.service.VisualTemplateUseSituationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/template/use/situation")
@Api(value = "模板使用情况分析", tags = "模板使用情况分析接口")
public class VisualTemplateUseSituationController extends BladeController {

    @Autowired
    private VisualTemplateUseSituationService visualTemplateUseSituationService;


    @GetMapping("/count")
    @ApiOperationSupport(order = 1)
    @ApiOperation(value = "查询某个模板的总体使用人数", notes = "传入visualId")
    public R<VisualTemplateUseSituationDto> getUseCount(Integer visualId){

        List<VisualTemplateUseSituationDto> situation = visualTemplateUseSituationService.getUseCount(visualId);
        if (CollectionUtil.isEmpty(situation)) {
            return null;
        }
        return R.data(situation.get(0));
    }

    @GetMapping("/list/count")
    @ApiOperationSupport(order = 2)
    @ApiOperation(value = "统计模板总体使用情况")
    public R<List<VisualTemplateUseSituationDto>> templateUseCount(){

        return R.data(visualTemplateUseSituationService.getUseCount(null));
    }

    @GetMapping("/page")
    @ApiOperationSupport(order = 3)
    @ApiOperation(value = "查询模板的使用情况，但是不统计模板使用的数量")
    public R<IPage<VisualTemplateUseSituation>> getList(@RequestParam VisualTemplateUseSituation situation, Query query) {
        LambdaQueryWrapper<VisualTemplateUseSituation> queryWrapper = Condition.getQueryWrapper(situation).lambda()
                .orderByAsc(VisualTemplateUseSituation::getTenantId);

        return R.data(visualTemplateUseSituationService.page(Condition.getPage(query), queryWrapper));
    }



    @PostMapping("/record")
    @ApiOperationSupport(order = 4)
    @ApiOperation(value = "记录模板使用情况")
    public R<String> recordUseSituation(@RequestBody VisualTemplateUseSituation situation) {

        BladeUser user = getUser();
        situation.setUserId(user.getUserId());
        situation.setUserName(user.getUserName());
        situation.setTenantId(user.getTenantId());
        visualTemplateUseSituationService.saveOrUpdate(situation);

        return R.data("保存成功");
    }
}
