/**
 * 
 */
package org.springblade.visual.controller;

import java.util.Date;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.visual.entity.VisualWarningRecord;
import org.springblade.visual.service.VisualWarningRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.metadata.IPage;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 *	智能预警记录 操作处理
 *
 * @author wookvn
 * @date   2022-03-09
 */
@RestController
@RequestMapping("visual/warningRecord")
@Api(value = "智能预警记录相关接口", tags = {"智能预警记录相关接口"})
public class VisualWarningRecordController extends BladeController {

	@Autowired
	private VisualWarningRecordService visualWarningRecordService;
	
	/**
	 * 	列表数据
	 * @param visualWarningRecord
	 * @return
	 */
	@GetMapping("/list")
	@ApiOperation(value = "分页查询智能预警记录信息", httpMethod = "GET")
	public R<IPage<VisualWarningRecord>> list(VisualWarningRecord visualWarningRecord, Query query){
		IPage<VisualWarningRecord> page = visualWarningRecordService.page(Condition.getPage(query), Condition.getQueryWrapper(visualWarningRecord).lambda().orderByDesc(VisualWarningRecord::getId));
		return R.data(page);
	}
	
	/**
	 *	修改数据接口
	 * @param visualWarningRecord
	 * @return
	 */
	@PostMapping("/update")
	@ApiOperation(value = "根据ID更新智能预警记录", httpMethod = "POST")
	public R<String> doUpdate(@RequestBody VisualWarningRecord visualWarningRecord) {
		if(visualWarningRecord == null || visualWarningRecord.getId() == null) {
			return R.fail("参数错误，修改失败。");
		}
		visualWarningRecord.setUpdateBy(getUser().getNickName());
		visualWarningRecord.setUpdateTime(new Date());
		return R.status(visualWarningRecordService.updateById(visualWarningRecord));
	}
	
//	/**
//	 * 	删除数据接口
//	 * @param id
//	 * @return
//	 */
//	@PostMapping("/delete")
//	@ApiOperation(value = "根据ID数组删除智能预警记录(多个逗号分割)", httpMethod = "POST")
//	public R<String> doDelete(@RequestParam Integer[] id) {
//		return R.status(visualWarningRecordService.removeByIds(Arrays.asList(id)));
//	}
	
}