/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.visual.entity;

import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.List;

import org.springblade.core.mp.base.TenantEntity;
import org.springblade.visual.enums.VisualTypeEnum;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.SqlCondition;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * 报表模板实体类
 *
 * @author BladeX
 * @since 2021-07-02
 */
@Data
@TableName("visual_data")
@ApiModel(value = "Visual对象", description = "报表模板")
public class VisualData extends TenantEntity implements Serializable {

	private static final long serialVersionUID = 1L;

	/**
	 * 主键
	 */
	@ApiModelProperty(value = "主键")
	@TableId(value = "id", type = IdType.AUTO)
	private Integer id;
	/**
	 * 大屏标题
	 */
	@ApiModelProperty(value = "大屏标题")
	@TableField(condition = SqlCondition.LIKE)
	private String title;
	/**
	 * 大屏背景
	 */
	@ApiModelProperty(value = "大屏背景")
	private String backgroundUrl;
	/**
	 * 大屏类型
	 */
	@ApiModelProperty(value = "大屏类型")
	private Integer category;
	/**
	 * 历史报告id
	 */
	@ApiModelProperty(value = "历史报告id")
	private Integer historyReportId;
	/**
	 * 发布密码
	 */
	@ApiModelProperty(value = "发布密码")
	private String password;
	/**
	 * 创建人
	 */
	@ApiModelProperty(value = "创建人")
	private Long createUser;
	/**
	 * 创建部门
	 */
	@ApiModelProperty(value = "创建部门")
	private Long createDept;
	/**
	 * 创建时间
	 */
	@ApiModelProperty(value = "创建时间")
	private LocalDateTime createTime;
	/**
	 * 修改人
	 */
	@ApiModelProperty(value = "修改人")
	private Long updateUser;
	/**
	 * 修改时间
	 */
	@ApiModelProperty(value = "修改时间")
	private LocalDateTime updateTime;
	/**
	 * 状态
	 */
	@ApiModelProperty(value = "状态")
	private Integer status;
	/**
	 * 是否已删除
	 */
	@ApiModelProperty(value = "是否已删除")
	private Integer isDeleted;
	/**
	 * 排序值 从小到大排序
	 */
	@ApiModelProperty(value = "排序值 从小到大排序")
	private Integer orderIndex;

	@ApiModelProperty(value = "模版内容")
	private String content;

	@ApiModelProperty(value = "移动应用模版内容")
	private String mobileContent;
	
	@ApiModelProperty(value = "大屏公开状态(0-否/1-是)")
	private Integer publicStatus;
	
	@ApiModelProperty(value = "大屏类型")
	private VisualTypeEnum VisualType;
	
	@ApiModelProperty(value = "复制的大屏ID")
	private Integer refVisualId;
	
	@ApiModelProperty(value = "幻灯片ID")
	private Integer powerPointId;
	
	@ApiModelProperty(value = "组件ID列表【查询时候使用】")
	@TableField(exist = false)
	private List<Integer> componentIds;

	@ApiModelProperty(value = "组件内容")
	@TableField(exist = false)
	private List<VisualComponent> components;

	@ApiModelProperty(value = "组件ID列表【查询时候使用】")
	@TableField(exist = false)
	private List<Integer> mobileComponentIds;

	@ApiModelProperty(value = "移动组件内容")
	@TableField(exist = false)
	private List<VisualComponent> mobileComponents;

	@ApiModelProperty(value = "是否为自主报表 1：是，0：否")
	private Integer visualDataType;

	@ApiModelProperty(value = "是否为移动大屏(0:否，1:是)")
	private Integer isMobile;

	@ApiModelProperty("是否可以被编辑 1：可以，0：不可以")
	private Integer isEdit;
	
}
