package org.springblade.visual.entity;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.visual.dto.WarningWayDto;
import org.springblade.visual.enums.WarningWayEnum;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.SqlCondition;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonIgnore;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * 智能预警器表 visual_warning
 * 
 * @author wookvn
 * @date 2022-03-09
 */
@Data
@TableName("visual_warning")
@ApiModel(value = "VisualWarning对象", description = "智能预警器")
public class VisualWarning implements Serializable{

	private static final long serialVersionUID = 1L;
 	/** 
	 *  
	 */
	@TableId(type = IdType.AUTO)
	private Integer id;
	/** 
	 * 预警器名称 
	 */
	@ApiModelProperty(value = "预警器名称")
	@TableField(value = "name", condition = SqlCondition.LIKE)
	private String name;
	
	/**
	 * 
	 */
	@ApiModelProperty(value = "请求频率(单位：秒)")
	private Integer requestFreq;
	/** 
	 * 数据源ID 
	 */
	@ApiModelProperty(value = "数据源ID")
	@TableField(value = "datasource_id")
	private Integer datasourceId;
	/** 
	 * 数据源名称 
	 */
	@ApiModelProperty(value = "数据源名称")
	@TableField(value = "datasource_name", condition = SqlCondition.LIKE)
	private String datasourceName;
	/** 
	 * 数据集ID 
	 */
	@ApiModelProperty(value = "数据集ID")
	@TableField(value = "cube_data_id")
	private Integer cubeDataId;
	/** 
	 * 数据集名称 
	 */
	@ApiModelProperty(value = "数据集名称")
	@TableField(value = "cube_data_name", condition = SqlCondition.LIKE)
	private String cubeDataName;
	/** 
	 * 查询条件JSON 
	 */
	@ApiModelProperty(value = "查询条件JSON")
	@TableField(value = "filter_json")
	private String filterJson;
	/** 
	 * 查询条件关系JSON 
	 */
	@ApiModelProperty(value = "查询条件关系JSON")
	@TableField(value = "filter_relation_json")
	private String filterRelationJson;
	/** 
	 * 时间设置JSON 
	 */
	@ApiModelProperty(value = "时间设置JSON")
	@TableField(value = "filter_time_json")
	private String filterTimeJson;
	/** 
	 * 字段JSON 
	 */
	@ApiModelProperty(value = "字段JSON")
	@TableField(value = "column_json")
	private String columnJson;
	/**
	 * 排序JSON
	 */
	@ApiModelProperty("排序JSON")
	private String sortJson;
	/** 
	 * 字段ID列表JSON 
	 */
	@JsonIgnore
	@ApiModelProperty(hidden = true)
	@TableField(value = "column_id_list")
	private String columnIdList;
	/**
	 * 消息核心内容
	 */
	@ApiModelProperty(value = "消息核心内容")
	@TableField(value = "message_dynamic_content")
	private String messageDynamicContent;
	/**
	 * 预警消息方式
	 */
	@JsonIgnore
	@ApiModelProperty(value = "预警消息方式", hidden = true)
	@TableField(value = "warn_message_way")
	private Integer warnMessageWay;
	/** 
	 * 预警人ID 
	 */
	@ApiModelProperty(value = "预警人ID")
	@TableField(value = "warn_user_ids")
	private String warnUserIds;
	
	/** 
	 * 是否开启(0-否/1-是) 
	 */
	@ApiModelProperty(value = "是否开启(0-否/1-是)")
	@TableField(value = "enable_status")
	private Integer enableStatus;
	/**
	 * 审批状态
	 */
	@ApiModelProperty(value = "审批状态")
	@TableField(value = "approve_status")
	private Integer approveStatus;
	/**
	 * 创建时间 
	 */
	@ApiModelProperty(value = "创建时间")
	@TableField(value = "create_time")
	private Date createTime;
	/** 
	 * 创建人 
	 */
	@ApiModelProperty(value = "创建人")
	@TableField(value = "create_by")
	private String createBy;
	/** 
	 * 创建人ID
	 */
	@ApiModelProperty(value = "创建人ID")
	private Long createUserId;
	/** 
	 * 更新时间 
	 */
	@ApiModelProperty(value = "更新时间")
	@TableField(value = "update_time")
	private Date updateTime;
	
	@ApiModelProperty(value = "下一次执行时间")
	private Long nextLoopTs;
	
	@ApiModelProperty(value = "短信变量参数对照json")
	private String smsParams;
	@ApiModelProperty(value = "短信模版ID")
	private Integer smsId;
	
	@ApiModelProperty(value = "XXLJOB任务ID")
	private Integer jobId;
	
	@ApiModelProperty(value = "大屏列表基础信息JSON([{id:'1',title:'大屏一号'}] )")
	private String visualDataJson;
	
	@JsonIgnore
	@ApiModelProperty(hidden = true)
	@TableField(exist = false)
	private String formatContent;

	@ApiModelProperty(value = "选中的推送方式")
	@TableField(exist = false)
	private List<WarningWayEnum> checkedWays;

	public List<WarningWayEnum> getCheckedWays(){
		return warnMessageWay != null ? WarningWayEnum.checkedList(warnMessageWay) : checkedWays;
	}

	public Integer getWarnMessageWay() {
		return CollectionUtil.isNotEmpty(checkedWays) ? checkedWays.stream().map(WarningWayEnum::getValue).reduce((t, v) -> t | v).get() : warnMessageWay;
	}

	/**
	 * 推送方式
	 */
	@ApiModelProperty(value = "推送方式")
	@TableField(value = "warning_ways")
	private String warningWays;

	@ApiModelProperty(value = "租户id")
	private String tenantId;
}
