package org.springblade.visual.service;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.BladeUser;
import org.springblade.visual.entity.VisualExternalLinkInfo;
import org.springblade.visual.mapper.VisualExternalLinkInfoMapper;
import org.springblade.visual.utils.FileUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.time.LocalDateTime;
import java.util.UUID;


@Service
public class VisualExternalLinkInfoService extends ServiceImpl<VisualExternalLinkInfoMapper, VisualExternalLinkInfo> {

    @Value("${upload.external.path}")
    private String linkPath;

    /**
     * 分页查询
     */
    public Page<VisualExternalLinkInfo> doPage(VisualExternalLinkInfo linkInfo, Query query,BladeUser user,Boolean checkSuperPermRet) {
        Page<VisualExternalLinkInfo> page = new Page<>(query.getCurrent(), query.getSize());
        LambdaQueryWrapper<VisualExternalLinkInfo> queryWrapper = new LambdaQueryWrapper<>();
        if(checkSuperPermRet == null || !checkSuperPermRet) {
            queryWrapper.eq(VisualExternalLinkInfo::getUploadUserId, user.getUserId());
        }
        if (linkInfo != null) {
            if (StrUtil.isNotEmpty(linkInfo.getPath())) {
                queryWrapper.like(VisualExternalLinkInfo::getPath,linkInfo.getPath());
            }
            if (StrUtil.isNotEmpty(linkInfo.getTrendId())) {
                queryWrapper.eq(VisualExternalLinkInfo::getTrendId,linkInfo.getTrendId());
            }
            if (linkInfo.getStatus() != null) {
                queryWrapper.eq(VisualExternalLinkInfo::getStatus,linkInfo.getStatus());
            }
        }
        return this.page(page,queryWrapper);
    }

    /**
     * 上传三方连接
     */
    public String doUpload(MultipartFile file, BladeUser user, String trendId, String folder) {
        // 文件上传的文件夹名称
        if (StrUtil.isEmpty(folder) || folder.equals("null")) {
            folder = UUID.randomUUID().toString().replace("-", "");
        }
        String FOLDER_PATH = linkPath + "/" + folder;
        try {
            // 上传文件到指定目录
            File savePos = new File(FOLDER_PATH);
            if (!savePos.exists()) {  // 不存在，则创建该文件夹
                savePos.mkdir();
            }
            // 上传该文件至该文件夹下
            File newFile = new File(FOLDER_PATH + "/" + file.getOriginalFilename());
            if (newFile.exists()){
                // 如果存在则先删除
                FileUtil.del(FOLDER_PATH + "/" + file.getOriginalFilename());
            }
            file.transferTo(newFile);
            // 对文件进行解压
            FileUtils.unpack(newFile,new File(FOLDER_PATH));
            VisualExternalLinkInfo linkInfo = new VisualExternalLinkInfo();
            linkInfo.setUploadTime(LocalDateTime.now());
            linkInfo.setUploadUser(user.getNickName());
            linkInfo.setUploadUserId(String.valueOf(user.getUserId()));
            linkInfo.setPath(folder);
            if (StrUtil.isNotEmpty(trendId)){
                linkInfo.setTrendId(trendId);
            }
            this.save(linkInfo);
            // 上传完成后删除原文件
            newFile.delete();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return folder;
    }
}
