package org.springblade.visual.service;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.system.client.DataPermClient;
import org.springblade.visual.dto.VisualLoginFreeLinkPermissionDoo;
import org.springblade.visual.dto.VisualLoginLinkPermissionDto;
import org.springblade.visual.dto.VisualPowerPermissionDto;
import org.springblade.visual.entity.VisualLoginFreeLink;
import org.springblade.visual.entity.VisualLoginFreeLinkPermission;
import org.springblade.visual.entity.VisualPowerPointInfo;
import org.springblade.visual.entity.VisualPowerPointPermission;
import org.springblade.visual.mapper.VisualLoginFreeLinkPermissionMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * @Author: hepingguo
 * @Description:
 * @Date: 2023/3/20 10:39
 **/
@Service
public class VisualLoginFreeLinkPermissionService extends ServiceImpl<VisualLoginFreeLinkPermissionMapper, VisualLoginFreeLinkPermission> {


    @Autowired
    private VisualLoginFreeLinkService visualLoginFreeLinkService;

    @Autowired
    private DataPermClient dataPermClient;

    /**
     *
     * @param dataPermId
     * @return
     */
    public List<Integer> listLoginFreeLinkId(Integer dataPermId) {

        return baseMapper.listLoginFreeLinkId(dataPermId);
    }

    /**
     *
     * @param linkPermissionDoo
     * @param user
     * @return
     */
    public boolean doSaveOrUpdate(VisualLoginFreeLinkPermissionDoo linkPermissionDoo, BladeUser user) {

        //通过关联删除，删除大屏分类ID的数据
        baseMapper.deleteByPermId(linkPermissionDoo.getDataPermId());
        //2.再添加
        if(CollectionUtil.isNotEmpty(linkPermissionDoo.getLinkIdList())) {
            //更新公开状态为0[否则权限组不生效]
            visualLoginFreeLinkService.update(Wrappers.<VisualLoginFreeLink>lambdaUpdate()
                    .set(VisualLoginFreeLink::getPublicStatus, 0)
                    .in(VisualLoginFreeLink::getId, linkPermissionDoo.getLinkIdList()));
            List<VisualLoginFreeLinkPermission> dataList = new ArrayList<>(linkPermissionDoo.getLinkIdList().size());
            linkPermissionDoo.getLinkIdList().forEach(t -> {
                VisualLoginFreeLinkPermission entity = VisualLoginFreeLinkPermission.builder()
                        .createBy(user.getNickName())
                        .createTime(LocalDateTime.now())
                        .createUserId(user.getUserId())
                        .dataPermId(linkPermissionDoo.getDataPermId())
                        .linkId(t)
                        .build();
                dataList.add(entity);
            });
            saveBatch(dataList);
        }
        return true;
    }

    /**
     *
     * @param linkId
     * @param visualCreateUserId
     * @param bladeUser
     * @return
     */
    public boolean checkVisualDataForUser(Integer linkId, Long visualCreateUserId, BladeUser bladeUser) {

        //检查4个方面：
        //1.当前用户是否创建人
        //2.是否公开
        //3.是否超级权限组
        //4.当前用户是否拥有权限
        return  visualCreateUserId.longValue() == bladeUser.getUserId()
                || visualLoginFreeLinkService.count(Wrappers.<VisualLoginFreeLink>lambdaQuery().eq(VisualLoginFreeLink::getId, linkId).eq(VisualLoginFreeLink::getPublicStatus, 1)) > 0
                || Optional.ofNullable(dataPermClient.checkSuperPermission(bladeUser.getUserId())).orElse(false)
                || baseMapper.checkVisualForUser(linkId, bladeUser.getUserId()) > 0;
    }

    /**
     *
     * @param linkId
     * @return
     */
    public List<JSONObject> listDataPerm(Integer linkId) {

        return baseMapper.selectDataPermList(linkId);
    }

    public boolean doPerform(VisualLoginLinkPermissionDto linkPermissionDto, BladeUser user) {
        //1.根据大屏ID删除所属权限组
        baseMapper.delete(Wrappers.<VisualLoginFreeLinkPermission>lambdaQuery().eq(VisualLoginFreeLinkPermission::getLinkId,
                linkPermissionDto.getLinkId()));
        if(linkPermissionDto.getPublicStatus() != null) {
            visualLoginFreeLinkService.update(Wrappers.<VisualLoginFreeLink>lambdaUpdate().set(VisualLoginFreeLink::getPublicStatus, linkPermissionDto.getPublicStatus())
                    .eq(VisualLoginFreeLink::getId, linkPermissionDto.getLinkId()));
        }
        //2.批量添加权限组
        if(CollectionUtil.isNotEmpty(linkPermissionDto.getDataPermIdList())) {
            List<VisualLoginFreeLinkPermission> dataList = new ArrayList<>(linkPermissionDto.getDataPermIdList().size());
            linkPermissionDto.getDataPermIdList().forEach(dataPermId -> {
                VisualLoginFreeLinkPermission entity = VisualLoginFreeLinkPermission.builder()
                        .createBy(user.getNickName())
                        .createUserId(user.getUserId())
                        .createTime(LocalDateTime.now())
                        .linkId(linkPermissionDto.getLinkId())
                        .dataPermId(dataPermId)
                        .build();
                dataList.add(entity);
            });
            saveBatch(dataList);
        }
        return true;
    }
}
