/**
 * 
 */
package org.springblade.visual.service;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.system.client.DataPermClient;
import org.springblade.visual.dto.VisualPowerPermissionDoo;
import org.springblade.visual.dto.VisualPowerPermissionDto;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualDataPermission;
import org.springblade.visual.entity.VisualPowerPointInfo;
import org.springblade.visual.entity.VisualPowerPointPermission;
import org.springblade.visual.mapper.VisualPowerPointPermissionMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 *	幻灯片权限组 服务处理
 *
 * @author wookvn
 * @date   2023-02-04
 */
@Service
public class VisualPowerPointPermissionService extends ServiceImpl<VisualPowerPointPermissionMapper, VisualPowerPointPermission>{


    @Autowired
    private DataPermClient dataPermClient;

    @Autowired
    private VisualPowerPointInfoService visualPowerPointInfoService;

    /**
     * 根据用户id查询有权限的PPT的分类Id集合
     * @param currUserId
     * @return
     */
    public List<Integer> categoryIdListPerm(long currUserId) {
        return baseMapper.categoryIdListPerm(currUserId);
    }

    /**
     * 批量添加或修改权限组与幻灯片关系
     * @param visualPowerPerDto
     * @param user
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean doPerform(VisualPowerPermissionDto visualPowerPerDto, BladeUser user) {
        //1.根据大屏ID删除所属权限组
        baseMapper.delete(Wrappers.<VisualPowerPointPermission>lambdaQuery().eq(VisualPowerPointPermission::getPpId,
                visualPowerPerDto.getPowerId()));
        if(visualPowerPerDto.getPublicStatus() != null) {
            visualPowerPointInfoService.update(Wrappers.<VisualPowerPointInfo>lambdaUpdate().set(VisualPowerPointInfo::getPublicStatus, visualPowerPerDto.getPublicStatus())
                    .eq(VisualPowerPointInfo::getId, visualPowerPerDto.getPowerId()));
        }
        //2.批量添加权限组
        if(CollectionUtil.isNotEmpty(visualPowerPerDto.getDataPermIdList())) {
            List<VisualPowerPointPermission> dataList = new ArrayList<>(visualPowerPerDto.getDataPermIdList().size());
            visualPowerPerDto.getDataPermIdList().forEach(dataPermId -> {
                VisualPowerPointPermission entity = new VisualPowerPointPermission();
                entity.setCreateBy(user.getNickName());
                entity.setCreateTime(LocalDateTime.now());
                entity.setCreateUserId(user.getUserId());
                entity.setDataPermId(dataPermId);
                entity.setPpId(visualPowerPerDto.getPowerId());
                dataList.add(entity);
            });
            saveBatch(dataList);
        }
        return true;
    }

    /**
     * 根据权限组ID和分类ID查询幻灯片ID列表
     * @param dataPermId
     * @param visualCategoryId
     * @return
     */
    public List<Integer> listVisualId(Integer dataPermId, Integer visualCategoryId) {

        return baseMapper.listVisualId(dataPermId,visualCategoryId);
    }

    @Transactional(rollbackFor = Exception.class)
    public boolean doSaveOrUpdate(VisualPowerPermissionDoo visualPowerPermDto, BladeUser user) {
        //通过关联删除，删除大屏分类ID的数据
        baseMapper.deleteByCategoryIdAndPermId(visualPowerPermDto.getVisualCategoryId(), visualPowerPermDto.getDataPermId());
        //2.再添加
        if(CollectionUtil.isNotEmpty(visualPowerPermDto.getPowerIdList())) {
            //更新公开状态为0[否则权限组不生效]
            visualPowerPointInfoService.update(Wrappers.<VisualPowerPointInfo>lambdaUpdate()
                    .set(VisualPowerPointInfo::getPublicStatus, 0)
                    .in(VisualPowerPointInfo::getId, visualPowerPermDto.getPowerIdList()));
            List<VisualPowerPointPermission> dataList = new ArrayList<>(visualPowerPermDto.getPowerIdList().size());
            visualPowerPermDto.getPowerIdList().forEach(t -> {
                VisualPowerPointPermission entity = new VisualPowerPointPermission();
                entity.setCreateBy(user.getNickName());
                entity.setCreateTime(LocalDateTime.now());
                entity.setCreateUserId(user.getUserId());
                entity.setDataPermId(visualPowerPermDto.getDataPermId());
                entity.setPpId(t);
                dataList.add(entity);
            });
            saveBatch(dataList);
        }
        return true;
    }

    /**
     * 判断用户是否有幻灯片的权限
     * @param powerId
     * @param visualCreateUserId
     * @param bladeUser
     * @return
     */
    public boolean checkVisualDataForUser(Integer powerId, Long visualCreateUserId, BladeUser bladeUser) {
        //检查4个方面：
        //1.当前用户是否创建人
        //2.是否公开
        //3.是否超级权限组
        //4.当前用户是否拥有权限
        return  visualCreateUserId.longValue() == bladeUser.getUserId()
                || visualPowerPointInfoService.count(Wrappers.<VisualPowerPointInfo>lambdaQuery().eq(VisualPowerPointInfo::getId, powerId).eq(VisualPowerPointInfo::getPublicStatus, 1)) > 0
                || Optional.<Boolean>ofNullable(dataPermClient.checkSuperPermission(bladeUser.getUserId())).orElse(false)
                || baseMapper.checkVisualForUser(powerId, bladeUser.getUserId()) > 0;
    }

    public List<JSONObject> listDataPerm(Integer powerId) {

        return baseMapper.selectDataPermList(powerId);
    }
}