/**
 * 
 */
package org.springblade.visual.service;

import java.time.LocalDateTime;
import java.util.HashSet;
import java.util.List;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualReportInfo;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springblade.visual.mapper.VisualReportInfoMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

/**
 * @author wookvn
 *
 */
@Service
public class VisualReportInfoService extends ServiceImpl<VisualReportInfoMapper, VisualReportInfo> {

	@Autowired
	private VisualDataService visualDataService;
	
	/**
	 * 新增或修改
	 * @param reportInfo
	 * @param visualIdList
	 * @return
	 */
	@Transactional
	public Integer doSave(VisualReportInfo reportInfo, List<Integer> visualIdList, BladeUser bladeUser) {
		reportInfo.setCreateUserId(bladeUser.getUserId());
		reportInfo.setCreateTime(LocalDateTime.now());
		int ret = baseMapper.insert(reportInfo);
		if(ret > 0) {
			//复制大屏和组件
			visualDataService.doCopyReport(new HashSet<>(visualIdList), reportInfo.getId(), bladeUser);
		}
		return reportInfo.getId();
	}
	
	/**
	 * @param idList
	 */
	@Transactional
	public boolean doRemove(List<Long> idList) {
		baseMapper.deleteBatchIds(idList);
		LambdaQueryWrapper<VisualData> queryWrapper = Wrappers.<VisualData>lambdaQuery().eq(VisualData::getVisualType, VisualTypeEnum.REPORT).in(VisualData::getHistoryReportId, idList);
		List<Integer> visualIdList = visualDataService.listObjs(queryWrapper, a -> Integer.valueOf(a.toString()));
		if(CollectionUtil.isNotEmpty(visualIdList)) {
			visualDataService.doRemove(visualIdList);
		}
		return true;
	}
}
