package org.springblade.visual.service;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.system.client.DataPermClient;
import org.springblade.visual.dto.VisualSelfReportPermissionDoo;
import org.springblade.visual.dto.VisualSelfReportPermissionDto;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualDataPermission;
import org.springblade.visual.entity.VisualSelfReportPermission;
import org.springblade.visual.mapper.VisualSelfReportPermissionMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

@Service
public class VisualSelfReportPermissionService extends ServiceImpl<VisualSelfReportPermissionMapper, VisualSelfReportPermission> {

    @Autowired
    private VisualDataService visualDataService;

    @Autowired
    private DataPermClient dataPermClient;


    public List<Integer> listVisualId(Integer dataPermId, Integer reportCategoryId) {

        return baseMapper.selectVisualIdList(dataPermId, reportCategoryId);
    }

    @Transactional(rollbackFor = Exception.class)
    public boolean doSaveOrUpdate(VisualSelfReportPermissionDto reportPermissionDto, BladeUser user) {
        //通过关联删除，删除报表分类ID的数据
        baseMapper.deleteByClassifyIdAndPermId(reportPermissionDto.getReportCategoryId(), reportPermissionDto.getDataPermId());
        //2.再添加
        if(CollectionUtil.isNotEmpty(reportPermissionDto.getSelfReportIdList())) {
            //更新公开状态为0[否则权限组不生效]
            visualDataService.update(Wrappers.<VisualData>lambdaUpdate().set(VisualData::getPublicStatus, 0).in(VisualData::getId, reportPermissionDto.getSelfReportIdList()));
            List<VisualSelfReportPermission> dataList = new ArrayList<>(reportPermissionDto.getSelfReportIdList().size());
            reportPermissionDto.getSelfReportIdList().forEach(t -> {
                VisualSelfReportPermission entity = new VisualSelfReportPermission();
                entity.setCreateBy(user.getNickName());
                entity.setCreateTime(new Date());
                entity.setCreateUserId(user.getUserId());
                entity.setDataPermId(reportPermissionDto.getDataPermId());
                entity.setSelfReportId(t);
                dataList.add(entity);
            });
            saveBatch(dataList);
        }
        return true;
    }

    public boolean checkVisualDataForUser(Integer selfReportId, Long visualCreateUserId, BladeUser bladeUser) {
        //检查4个方面：
        //1.当前用户是否创建人
        //2.是否公开
        //3.是否超级权限组
        //4.当前用户是否拥有权限
        return  visualCreateUserId.longValue() == bladeUser.getUserId()
                || visualDataService.count(Wrappers.<VisualData>lambdaQuery().eq(VisualData::getId, selfReportId).eq(VisualData::getPublicStatus, 1)) > 0
                || Optional.<Boolean>ofNullable(dataPermClient.checkSuperPermission(bladeUser.getUserId())).orElse(false)
                || baseMapper.checkVisualForUser(selfReportId, bladeUser.getUserId()) > 0;
    }

    public List<JSONObject> listDataPerm(Integer selfReportId) {

        return baseMapper.selectDataPermList(selfReportId);
    }

    public boolean doPerform(VisualSelfReportPermissionDoo reportPermissionDoo, BladeUser user) {
        //1.根据报表ID删除所属权限组
        baseMapper.delete(Wrappers.<VisualSelfReportPermission>lambdaQuery().eq(VisualSelfReportPermission::getSelfReportId, reportPermissionDoo.getSelfReportId()));
        if(reportPermissionDoo.getPublicStatus() != null) {
            visualDataService.update(Wrappers.<VisualData>lambdaUpdate().set(VisualData::getPublicStatus, reportPermissionDoo.getPublicStatus()).eq(VisualData::getId, reportPermissionDoo.getSelfReportId()));
        }
        //2.批量添加权限组
        if(CollectionUtil.isNotEmpty(reportPermissionDoo.getDataPermIdList())) {
            List<VisualSelfReportPermission> dataList = new ArrayList<>(reportPermissionDoo.getDataPermIdList().size());
            reportPermissionDoo.getDataPermIdList().forEach(dataPermId -> {
                VisualSelfReportPermission entity = new VisualSelfReportPermission();
                entity.setCreateBy(user.getNickName());
                entity.setCreateTime(new Date());
                entity.setCreateUserId(user.getUserId());
                entity.setDataPermId(dataPermId);
                entity.setSelfReportId(reportPermissionDoo.getSelfReportId());
                dataList.add(entity);
            });
            saveBatch(dataList);
        }
        return true;
    }
}
