/**
 * 
 */
package org.springblade.bi.client;

import java.util.List;
import java.util.Map;

import org.springblade.bi.client.factory.CubeTableColumnClientFallbackFactory;
import org.springblade.bi.client.form.ReportViewFrom;
import org.springblade.bi.client.model.CubeTableColumnModel;
import org.springblade.bi.client.model.TableColumnModel;
import org.springblade.common.openfeign.OpenFeignConstants;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

/**
 * @author wookvn
 *
 */
//@RequestMapping(OpenFeignConstants.BI_SERVICE + "/cubeTableColumn")
@FeignClient(contextId = "cubeTableColumnClient", 
	value = OpenFeignConstants.BI_SERVICE, 
	fallbackFactory = CubeTableColumnClientFallbackFactory.class)
public interface CubeTableColumnClient {

	/**
	 * 根据数据集字段ID查询对象
	 * @param id
	 * @return
	 */
	@GetMapping(OpenFeignConstants.BI_SERVICE + "/cubeTableColumn/info/{id}")
	public CubeTableColumnModel getById(@PathVariable Integer id);
	
	/**
	 * 根据数据集ID查询数据集字段列表
	 * @param cubeId
	 * @return
	 */
	@GetMapping(OpenFeignConstants.BI_SERVICE + "/cubeTableColumn/listByCubeId/{cubeId}")
	public List<CubeTableColumnModel> listByCubeId(@PathVariable Integer cubeId);

	/**
	 *
	 * @param cubeId
	 * @return {@link List< CubeTableColumnModel>}
	 * @author 王瑞
	 * @date 2024/6/25
	 */
	@GetMapping(OpenFeignConstants.BI_SERVICE + "/cubeTableColumn/refresh/{cubeId}")
	public List<Map<String,Object>> refresh(@PathVariable Integer cubeId);
	/**
	 * 根据 ds 和表名查询字段信息(包含主键)
	 * @param reportViewFrom
	 * @return {@link List< TableColumnModel>}
	 * @author 王瑞
	 * @date 2024/6/25
	 */
	@PostMapping(OpenFeignConstants.BI_SERVICE + "/cubeTableColumn/listColumnsByDatasource")
	public List<TableColumnModel> listColumnsByDatasource(@RequestBody ReportViewFrom reportViewFrom);

}
